/******************************************************************************
 * @file     main.c
 * @brief    M451 series ISP example source file
 * @version  2.0.0
 * @date     22, March, 2013
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "M451Series.h"
#include "string.h"
#include "ISP_USER.h"

#define CLK_PLLCON_60MHz_XTAL     0xC026 // 60MHz
#define CLK_PLLCON_72MHz_XTAL     0xC02E // 72MHz
#define CLK_PLLCON_144MHz_XTAL    0x402E // 144MHz

#define HID_MAX_PACKET_SIZE_EP1		64


#define UART_GET_TX_POINTER(UART)    (((UART)->FIFOSTS & UART_FIFOSTS_TXPTR_Msk) >> UART_FIFOSTS_TXPTR_Pos)


static uint8_t volatile	bufhead;

BOOL bUartDataReady, bUsingUART;
volatile uint32_t g_pdid, g_timecnt;



static UART_T	*g_pUART = UART0;
static IRQn_Type	g_UARTIRQ = UART0_IRQn;

void SysTimerDelay(uint32_t us);

void NMI_Handler(void)
{
	SYS->BODCTL |= (1<< SYS_BODCTL_BODIF_Pos);//(1<<SYS_BODCR_BOD_INTF_Pos);	//SYS_BOD_INT_FLAG;
}


/** 
send data for UART
**/
	static /*__inline*/ void PutString()
{
	int i;

#if(USING_RS485)	
	PF->DOUT |= 0x8000;	//PD->DOUT |= 0x2000;	//transmit mode
	SysTimerDelay(13);//we send 64 byte once, if send 1 bytes only, please change it to 60
#endif

	for(i = 0; i < HID_MAX_PACKET_SIZE_EP1; i++)
	{
		while(UART_GET_TX_POINTER(g_pUART) >= 14);
		
		//while(g_pUART->FSR.TX_EMPTY == 0);
		g_pUART->DAT = uart_sendbuf[i];
	}

#if(USING_RS485)
	while(UART_GET_TX_POINTER(g_pUART));
	SysTimerDelay(26);////we send 64 byte once, if send 1 bytes only, please change it to 110
	PF->DOUT &= ~0x8000;	//PD->DOUT &= ~0x2000;	//reveive mode
#endif

}

/** 
UART0/UART1 interrupt handler. handle receive only
**/
void UART0_IRQHandler(void)	//UART1_IRQHandler
{  
	/*----- Determine interrupt source -----*/

	if(inpw(&g_pUART->INTSTS) & 0x11) //RDA FIFO interrupt & RDA timeout interrupt 
   	{
   		while(((inpw(&g_pUART->FIFOSTS) & (0x4000)) == 0) && (bufhead < 64))	//RX fifo not empty
			uart_rcvbuf[bufhead++] = inpw(&g_pUART->DAT);
	}
	
	if(bufhead == 64)
	{
		bUartDataReady = TRUE;
		bufhead = 0;
	}
}





//unit is 0.5us
void SysTimerDelay(uint32_t us)
{
	//SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
    SysTick->LOAD = us * 24; /* using 48MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}


void EnableCLKO(uint32_t u32ClkSrc, uint32_t u32ClkDiv)
{
    /* CLKO = clock source / 2^(u32ClkDiv + 1) */
    CLK->CLKOCTL = CLK_CLKOCTL_DIV1EN_Msk | u32ClkDiv;

    /* Enable CLKO clock source */
    CLK->APBCLK0 |= CLK_APBCLK0_CLKOCKEN_Msk;

    /* Select CLKO clock source */
    CLK->CLKSEL1 = (CLK->CLKSEL1 & (~CLK_CLKSEL1_CLKOSEL_Msk)) | u32ClkSrc;
}

/*--------------------------------------------------------------------------*/
void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable Internal RC 22.1184MHz clock */
    CLK_EnableXtalRC(CLK_PWRCTL_HIRCEN_Msk);

    /* Waiting for Internal RC clock ready */
    CLK_WaitClockReady(CLK_STATUS_HIRCSTB_Msk);

    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HIRC, CLK_CLKDIV0_HCLK(1));

    /* Enable external XTAL 12MHz clock */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for external XTAL clock ready */
    CLK_WaitClockReady(CLK_STATUS_HXTSTB_Msk);

    /* Set Flash Access Delay */
    FMC->FTCTL |= FMC_FTCTL_FOM_Msk;

    /* Set core clock */
    CLK_SetCoreClock(72000000);

    /* Enable module clock */
    CLK_EnableModuleClock(UART0_MODULE);
    CLK_EnableModuleClock(USBD_MODULE);

    /* Select module clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_HXT, CLK_CLKDIV0_UART(1));
    CLK_SetModuleClock(USBD_MODULE, 0, CLK_CLKDIV0_USB(3));
    
    /* Enable USB LDO33 */
    SYS->USBPHY = SYS_USBPHY_LDO33EN_Msk;

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set GPD multi-function pins for UART0 RXD and TXD, and Clock Output */
    SYS->GPD_MFPL = SYS_GPD_MFPL_PD0MFP_UART0_RXD | SYS_GPD_MFPL_PD1MFP_UART0_TXD | SYS_GPD_MFPL_PD6MFP_CLKO;

    /* Enable CLKO(PB8) for monitor HCLK. CLKO = HCLK/8 Hz*/
    EnableCLKO((2 << CLK_CLKSEL1_CLKOSEL_Pos), 2);
}


void UART0_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset IP */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 Baudrate */
    UART_Open(UART0, 115200);
	
//---------------------------------------------------------------------------------------------------------------------	
//---------------------------------------------------------------------------------------------------------------------	
//---------------------------------------------------------------------------------------------------------------------	
}

void CheckConfig0()
{
    unsigned int data[2] = {0};
    FMC_Read(Config0, data);
    FMC_Read(Config1, data + 1);
    if(data[0] & 0x00010000)
    {
        data[0] &= ~0x00010000;
        UpdateConfig(data, 0);
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main(void)
{
 	uint8_t volatile bufhead_bak=0;
  
	/* Unlock protected registers */
	SYS_UnlockReg();	
	
	SYS_Init();
	
	CLK->AHBCLK |= CLK_AHBCLK_ISPCKEN_Msk;
	FMC->ISPCTL |= FMC_ISPCTL_ISPEN_Msk;

	g_apromSize = GetApromSize();
	GetDataFlashInfo(&g_dataFlashAddr , &g_dataFlashSize); 	
	g_pdid = SYS->PDID;
    
	UART0_Init();
	
	/* Enable RDA\RLS\RTO Interrupt  */
	NVIC_SetPriority(g_UARTIRQ , 2);
	UART_EnableInt(UART0, (UART_INTEN_RDAIEN_Msk | UART_INTEN_RLSIEN_Msk | UART_INTEN_RXTOIEN_Msk));
 	
	
	SysTick->LOAD = 300000 * 72; /* using 72MHz cpu clock*/    		
	SysTick->VAL   =  (0x00);
	SysTick->CTRL = SysTick->CTRL | SysTick_CTRL_CLKSOURCE_Msk | SysTick_CTRL_ENABLE_Msk;//using cpu clock

	CheckConfig0();
	
	while(1)
	{
		uint32_t lcmd; 

		if((bufhead >= 4) || (bUartDataReady == TRUE))
		{
			lcmd = inpw(uart_rcvbuf);
			if(lcmd == CMD_CONNECT) 
			{
				goto _ISP;
			}
			else
			{
				bUartDataReady = FALSE;
				bufhead = 0;
			}
		}

		//if((SysTick->CTRL & (1 << 16)) != 0)//timeout, then goto APROM
		if(SysTick->CTRL & (0x10000))
			goto _APROM;
	}


	
_ISP:	
    while(1)
    {
		if(bUartDataReady == TRUE)
		{
			bUartDataReady = FALSE;
			g_timecnt = 0;
			ParseCmd(uart_rcvbuf, HID_MAX_PACKET_SIZE_EP1, FALSE);
			PutString();
		}
		//timeout happen; but byte is less than 64 bytes; host goes wrong
		if (bufhead > 0)
		{
			if(g_timecnt == 0)
				bufhead_bak = bufhead;

			SysTimerDelay(1); //1 us
			g_timecnt++;

			if(g_timecnt > 1000) // 1 ms
			{					
				g_timecnt = 0;

				if(bufhead_bak == bufhead)
				{
					bufhead = 0;
				}
			}
		}
	}
		
_APROM:

	CheckConfig0();
	outpw(&SYS->RSTSTS, 3);//clear bit
	outpw(&FMC->ISPCTL, inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
	outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

	/* Trap the CPU */
	while(1);

}



/*** (C) COPYRIGHT 2013 Nuvoton Technology Corp. ***/
