/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/


/*---------------------------------------------------------------------------------------------------------*/
/* Include related headers                                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
//#include "MINI51.h"
//#include "core_cm0.h"
#include "DrvSPI.h"
#include "DrvSYS.h"

/*---------------------------------------------------------------------------------------------------------*/
/* Global file scope (static) variables                                                                    */
/*---------------------------------------------------------------------------------------------------------*/
typedef struct
{
    uint8_t bInUse;
    PFN_DRVSPI_CALLBACK pfncallback;
    uint32_t u32userData;
} S_DRVSPI_HANDLE;

static S_DRVSPI_HANDLE g_sSpiHandler[2];

/*---------------------------------------------------------------------------------------------------------*/
/* Interrupt Handler                                                                                       */
/*---------------------------------------------------------------------------------------------------------*/
#if 0
/*---------------------------------------------------------------------------------------------------------*/
/* Function: SPI0_IRQHandler                                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    None.                                                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    SPI0 interrupt handler. Clear the SPI interrupt flag and execute the callback function.              */
/*---------------------------------------------------------------------------------------------------------*/
void SPI0_IRQHandler(void)
{
    SPI->CNTRL_BITS.IF = 1; /* write '1' to clear SPI0 interrupt flag */
    
    if(g_sSpiHandler[0].pfncallback != NULL)
    {
        g_sSpiHandler[0].pfncallback(g_sSpiHandler[0].u32userData);
    }
    
}
#endif
/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_Open                                                                                   */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*         eSpiPort     [in]: Specify the SPI port                                                         */
/*         eMode        [in]: Specify the operation mode (eDRVSPI_MASTER/eDRVSPI_SLAVE)                    */
/*         eType        [in]: Specify the transfer type (eDRVSPI_TYPE0 ~ eDRVSPI_TYPE7)                    */
/*         i32BitLength [in]: Specify the bit length in a transaction (1~32)                               */
/*                                                                                                         */
/* Returns:                                                                                                */
/*         E_DRVSPI_ERR_INIT: The specified SPI port has been opened before.                               */
/*         E_DRVSPI_ERR_BUSY: The specified SPI port is in busy status.                                    */
/*         E_DRVSPI_ERR_BIT_LENGTH: The specified bit length is out of range.                              */
/*         E_SUCCESS: Success.                                                                             */
/*                                                                                                         */
/* Description:                                                                                            */
/*       Configure the operation mode, transfer type and bit length of a transaction of the specified SPI  */
/*       port.                                                                                             */
/*       The timing waveform types:                                                                        */
/*
DRVSPI_TYPE0:          

    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----


DRVSPI_TYPE1:          

    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----


DRVSPI_TYPE2:          

    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---


DRVSPI_TYPE3:          

    CS    --|          Active state           |---
               _   _   _   _   _   _   _   _  
    CLK   ____| |_| |_| |_| |_| |_| |_| |_| |_____
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |-----
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |---


DRVSPI_TYPE4:          

    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |------
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----


DRVSPI_TYPE5:

    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----
          
    Rx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----


DRVSPI_TYPE6:

    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |------
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |------


DRVSPI_TYPE7:

    CS    --|          Active state           |---
           ___   _   _   _   _   _   _   _   ______ 
    CLK       |_| |_| |_| |_| |_| |_| |_| |_|  
              
    Tx    ----| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----
          
    Rx    --| 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |----


Master / Slave Transfer Type Matching Table

    DRVSPI_TYPE0 <==> DRVSPI_TYPE3
    DRVSPI_TYPE1 <==> DRVSPI_TYPE1
    DRVSPI_TYPE2 <==> DRVSPI_TYPE2
    DRVSPI_TYPE3 <==> DRVSPI_TYPE0
    DRVSPI_TYPE4 <==> DRVSPI_TYPE7
    DRVSPI_TYPE5 <==> DRVSPI_TYPE5
    DRVSPI_TYPE6 <==> DRVSPI_TYPE6
    DRVSPI_TYPE7 <==> DRVSPI_TYPE4
*/
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_Open(E_DRVSPI_PORT eSpiPort, E_DRVSPI_MODE eMode, E_DRVSPI_TRANS_TYPE eType, int32_t i32BitLength)
{
    int32_t i32TimeOut;
    
    if(g_sSpiHandler[eSpiPort].bInUse)
    {
        return E_DRVSPI_ERR_INIT;
    }
    
    /* Bit length 1 ~ 32 */
    if((i32BitLength <= 0) || (i32BitLength > 32))
    {
        return E_DRVSPI_ERR_BIT_LENGTH;
    }
   
    if(eSpiPort == eDRVSPI_PORT0)
    {
        CLK->APBCLK_BITS.SPI_EN           =1;
        GCR->IPRST_CTL2_BITS.SPI_RST            =1;
        GCR->IPRST_CTL2_BITS.SPI_RST            =0;
    }
    else
    {
        return E_DRVSPI_ERR_INIT;
    }
    
    /* Check busy*/
    i32TimeOut = 0x10000;
    while(SPI->CNTRL_BITS.GO_BUSY == 1)
    {
        if(i32TimeOut-- <= 0)
            return E_DRVSPI_ERR_BUSY;
    }
    
    g_sSpiHandler[eSpiPort].bInUse = TRUE;
    g_sSpiHandler[eSpiPort].pfncallback = NULL;
    g_sSpiHandler[eSpiPort].u32userData = 0;
    
    /* "i32BitLength = 0" means 32 bits */
    if(i32BitLength == 32)
    {
        i32BitLength = 0;
    }
    SPI->CNTRL_BITS.TX_BIT_LEN = i32BitLength;
    
    if(eMode == eDRVSPI_SLAVE)
        SPI->CNTRL_BITS.SLAVE = 1;
    else
        SPI->CNTRL_BITS.SLAVE = 0;
    
    /* Automatic slave select */
    SPI->SSR_BITS.AUTOSS = 1;
    
    /* Timing waveform types */
    if(eType==eDRVSPI_TYPE0)
    {
        SPI->CNTRL_BITS.CLKP = 0;
        SPI->CNTRL_BITS.TX_NEG = 0;
        SPI->CNTRL_BITS.RX_NEG = 0;
    }
    else if(eType==eDRVSPI_TYPE1)
    {
        SPI->CNTRL_BITS.CLKP = 0;
        SPI->CNTRL_BITS.TX_NEG = 1;
        SPI->CNTRL_BITS.RX_NEG = 0;
    }
    else if(eType==eDRVSPI_TYPE2)
    {
        SPI->CNTRL_BITS.CLKP = 0;
        SPI->CNTRL_BITS.TX_NEG = 0;
        SPI->CNTRL_BITS.RX_NEG = 1;
    }
    else if(eType==eDRVSPI_TYPE3)
    {
        SPI->CNTRL_BITS.CLKP = 0;
        SPI->CNTRL_BITS.TX_NEG = 1;
        SPI->CNTRL_BITS.RX_NEG = 1;
    }
    else if(eType==eDRVSPI_TYPE4)
    {
        SPI->CNTRL_BITS.CLKP = 1;
        SPI->CNTRL_BITS.TX_NEG = 0;
        SPI->CNTRL_BITS.RX_NEG = 0;
    }
    else if(eType==eDRVSPI_TYPE5)
    {
        SPI->CNTRL_BITS.CLKP = 1;
        SPI->CNTRL_BITS.TX_NEG = 1;
        SPI->CNTRL_BITS.RX_NEG = 0;
    }
    else if(eType==eDRVSPI_TYPE6)
    {
        SPI->CNTRL_BITS.CLKP = 1;
        SPI->CNTRL_BITS.TX_NEG = 0;
        SPI->CNTRL_BITS.RX_NEG = 1;
    }
    else
    {
        SPI->CNTRL_BITS.CLKP = 1;
        SPI->CNTRL_BITS.TX_NEG = 1;
        SPI->CNTRL_BITS.RX_NEG = 1;
    }
    
    return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_Close                                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort [in]:  Specify the SPI port.                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Close the specified SPI module and disable the SPI interrupt.                                        */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_Close(E_DRVSPI_PORT eSpiPort)
{
    int32_t i32TimeOut;
    
    g_sSpiHandler[eSpiPort].bInUse = FALSE;
    g_sSpiHandler[eSpiPort].pfncallback = NULL;
    g_sSpiHandler[eSpiPort].u32userData = 0;

    /* Check SPI state */
    i32TimeOut = 0x10000;
    while(SPI->CNTRL_BITS.GO_BUSY == 1)
    {
        if(i32TimeOut-- <= 0)
            break;
    }
   
   if(eSpiPort == eDRVSPI_PORT0)
    {
        NVIC_DisableIRQ(SPI_IRQn);
        GCR->IPRST_CTL2_BITS.SPI_RST=1;
        GCR->IPRST_CTL2_BITS.SPI_RST=0;
        CLK->APBCLK_BITS.SPI_EN=0;
    }   
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetEndian                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort [in]: Specify the SPI port                                                                  */
/*    eEndian  [in]: Specify LSB first or MSB first (eDRVSPI_LSB_FIRST / eDRVSPI_MSB_FIRST)                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Dertermine to transfer data with LSB first or MSB first                                              */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetEndian(E_DRVSPI_PORT eSpiPort, E_DRVSPI_ENDIAN eEndian)
{

    if(eEndian == eDRVSPI_LSB_FIRST)
        SPI->CNTRL_BITS.LSB = 1;
    else
        SPI->CNTRL_BITS.LSB = 0;

}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetBitLength                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]: Specify the SPI port                                                              */
/*    i32BitLength [in]: Specify the bit length (1~32 bits)                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    E_SUCCESS              : Success.                                                                    */
/*    E_DRVSPI_ERR_BIT_LENGTH: The bit length is out of range.                                             */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the bit length of SPI transfer.                                                                  */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_SetBitLength(E_DRVSPI_PORT eSpiPort, int32_t i32BitLength)
{

    if((i32BitLength < 1) || (i32BitLength > 32))
    {
        return E_DRVSPI_ERR_BIT_LENGTH;
    }
    if(i32BitLength == 32)
        i32BitLength = 0;

    SPI->CNTRL_BITS.TX_BIT_LEN = i32BitLength;
    
    return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetByteReorder                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]: Specify the SPI port                                                              */
/*    eOption      [in]: the options of Byte Reorder function and Byte Suspend function.                   */
/*                       eDRVSPI_BYTE_REORDER_SUSPEND_DISABLE: both Byte Reorder function and Byte Suspend */
/*                                                             function are disabled.                      */
/*                       eDRVSPI_BYTE_REORDER_SUSPEND        : both Byte Reorder function and Byte Suspend */
/*                                                             function are enabled.                       */
/*                       eDRVSPI_BYTE_REORDER                : only enable the Byte Reorder function.      */
/*                       eDRVSPI_BYTE_SUSPEND                : only enable the Byte Suspend function.      */
/*                       The Byte Suspend function is only available in 32-bit transaction.                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    E_SUCCESS              : Success.                                                                    */
/*    E_DRVSPI_ERR_BIT_LENGTH: The bit length is not 16-, 24- or 32-bit.                                   */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Enable/disable Byte Reorder function.                                                                */
/*    The Byte Reorder function is supported only in 16-, 24- and 32-bit transaction mode.                 */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_SetByteReorder(E_DRVSPI_PORT eSpiPort, E_DRVSPI_BYTE_REORDER eOption)
{
    /* The Byte Suspend function is only available in 32-bit transaction. */
    if( (eOption==eDRVSPI_BYTE_REORDER_SUSPEND)||(eOption==eDRVSPI_BYTE_SUSPEND) )
        if( (SPI->CNTRL_BITS.TX_BIT_LEN) != 0 )
            return E_DRVSPI_ERR_BIT_LENGTH;
    /* The Byte Reorder function is supported only in 16-, 24- and 32-bit transaction mode. */
    else if( eOption==eDRVSPI_BYTE_REORDER )
        if( (SPI->CNTRL_BITS.TX_BIT_LEN) % 8 )
            return E_DRVSPI_ERR_BIT_LENGTH;

    SPI->CNTRL_BITS.REORDER = eOption;

    return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetSuspendCycle                                                                        */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort    [in]: Specify the SPI port.                                                              */
/*    i32Interval [in]: In burst transfer mode, this value specified the delay clocks between successive   */
/*                      transactions. If the Byte Suspend function is enabled, it specified the delay      */
/*                      clocks among each byte. It could be 2~17 which indicate 2~17 SPI clock cycles.     */
/*                      In FIFO mode, it could be 2~15 and 0. 0 indicates the maximum suspend interval; 2  */
/*                      indicates the minimum suspend interval. Please refer to TRM for the actual suspend */
/*                      interval.                                                                          */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    E_DRVSPI_ERR_SUSPEND_INTERVAL: The suspend interval setting is out of range.                         */
/*    E_SUCCESS: Success.                                                                                  */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the number of clock cycle of the suspend interval. Only for master mode.                         */
/*    The suspend cycle setting is shared with burst mode, byte suspend function and FIFO mode.            */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_SetSuspendCycle(E_DRVSPI_PORT eSpiPort, int32_t i32Interval)
{
    if( SPI->CNTRL_BITS.FIFO==1 ) /* In FIFO mode, it could be 2~15 and 0. */
    {
        if( (i32Interval < 2) || (i32Interval > 15) )
        {
            /* If out of range, specify the maximum suspend cycle and return error code. */
            SPI->CNTRL_BITS.SP_CYCLE = 0;
            if(i32Interval==0)
                return E_SUCCESS;
            else
                return E_DRVSPI_ERR_SUSPEND_INTERVAL;
        }
        SPI->CNTRL_BITS.SP_CYCLE = i32Interval;
        return E_SUCCESS;
    }
    else /* In burst mode and byte suspend function, it could be 2~17. */
    {
        if((i32Interval < 2) || (i32Interval > 17))
        {
            /* If out of range, specify the maximum suspend cycle and return error code. */
            SPI->CNTRL_BITS.SP_CYCLE = 15;
            return E_DRVSPI_ERR_SUSPEND_INTERVAL;
        }
        SPI->CNTRL_BITS.SP_CYCLE = i32Interval-2;
        return E_SUCCESS;
    }
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetTriggerMode                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort       [in]: Specify the SPI port                                                            */
/*    eSSTriggerMode [in]: Specify the trigger mode. (eDRVSPI_EDGE_TRIGGER or eDRVSPI_LEVEL_TRIGGER)       */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the trigger mode of slave select pin. Only for slave mode.                                       */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetTriggerMode(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SSLTRIG eSSTriggerMode)
{
    SPI->SSR_BITS.SS_LTRIG = eSSTriggerMode;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetSlaveSelectActiveLevel                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*    eSSActType [in]: Select the active type of slave select pin.                                         */
/*                     eDRVSPI_ACTIVE_LOW_FALLING: Slave select pin is active low in level-trigger mode;   */
/*                                                 or falling-edge trigger in edge-trigger mode.           */
/*                     eDRVSPI_ACTIVE_HIGH_RISING: Slave select pin is active high in level-trigger mode;  */
/*                                                 or rising-edge trigger in edge-trigger mode.            */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the active level of slave select.                                                                */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetSlaveSelectActiveLevel(E_DRVSPI_PORT eSpiPort, E_DRVSPI_SS_ACT_TYPE eSSActType)
{
    SPI->SSR_BITS.SS_LVL = eSSActType;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_GetLevelTriggerStatus                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE : The transaction number and the transferred bit length met the requirements which defines in   */
/*           TX_NUM and TX_BIT_LEN among one transfer.                                                     */
/*    FALSE: The transaction number or the transferred bit length of one transaction doesn't meet the      */
/*           requirements.                                                                                 */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Get the level-trigger transmission status. Only for slave mode.                                      */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_GetLevelTriggerStatus(E_DRVSPI_PORT eSpiPort)
{
    if(SPI->SSR_BITS.LTRIG_FLAG==1)
        return TRUE;
    else
        return FALSE;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_EnableAutoSS                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Enable the automatic slave select function and set the specified slave select pin. Only for master   */
/*    mode.                                                                                                */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_EnableAutoSS(E_DRVSPI_PORT eSpiPort)
{
    SPI->SSR_BITS.AUTOSS = 1;
    SPI->SSR_BITS.SSR = 1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_DisableAutoSS                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Disable the Automatic Slave Select function and deselect slave select pins. Only for master mode.    */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_DisableAutoSS(E_DRVSPI_PORT eSpiPort)
{
    SPI->SSR_BITS.AUTOSS = 0;
    SPI->SSR_BITS.SSR = 0;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetSS                                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*   eSpiPort   [in]: Specify the SPI port                                                                 */
/*                                                                                                         */
/* Returns:                                                                                                */
/*   None.                                                                                                 */
/*                                                                                                         */
/* Description:                                                                                            */
/*   Set the slave select pin. Only for master mode.                                                       */
/*   In automatic slave select operation, call this function to select the slave select pin. The state of  */
/*   slave select pin will be controlled by hardware.                                                      */
/*   In manual slave select operation, the slave select pin will be set to active state.                   */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetSS(E_DRVSPI_PORT eSpiPort)
{
    SPI->SSR_BITS.SSR = 1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_ClrSS                                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*   eSpiPort  [in]: Specify the SPI port                                                                  */
/*                                                                                                         */
/* Returns:                                                                                                */
/*   None.                                                                                                 */
/*                                                                                                         */
/* Description:                                                                                            */
/*   Set the slave select pin to inactive state. Only for master mode.                                     */
/*   In automatic slave select operation, call this function to deselect the slave select pin. The slave   */
/*   select pin will be set to inactive state.                                                             */
/*   In manual slave select operation, the slave select pin will be set to inactive state.                 */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_ClrSS(E_DRVSPI_PORT eSpiPort)
{
    SPI->SSR_BITS.SSR = 0;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_IsBusy                                                                                 */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  The SPI port is in busy.                                                                      */
/*    FALSE: The SPI port is not in busy.                                                                  */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Check the busy status of the specified SPI port.                                                     */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_IsBusy(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI->CNTRL_BITS.GO_BUSY)?TRUE:FALSE);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_BurstTransfer                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort    [in]: Specify the SPI port                                                               */
/*    i32BurstCnt [in]: Specify the transaction number in one transfer. It could be 1 or 2.                */
/*    i32Interval [in]: Specify the delay clocks between successive transactions. It could be 2~17.        */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    E_DRVSPI_ERR_BURST_CNT        : The transaction number is out of range.                              */
/*    E_DRVSPI_ERR_SUSPEND_INTERVAL : The suspend interval setting is out of range.                        */
/*    E_SUCCESS                     : Success.                                                             */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Configure the burst transfer settings.                                                               */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_BurstTransfer(E_DRVSPI_PORT eSpiPort, int32_t i32BurstCnt, int32_t i32Interval)
{

    if((i32BurstCnt < 1) || (i32BurstCnt > 2))
    {
        return E_DRVSPI_ERR_BURST_CNT;
    }
    
    if((i32Interval < 2) || (i32Interval > 17))
    {
        return E_DRVSPI_ERR_SUSPEND_INTERVAL;
    }

    SPI->CNTRL_BITS.TX_NUM = i32BurstCnt-1;
    SPI->CNTRL_BITS.SP_CYCLE = i32Interval-2;

    return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetClockFreq                                                                           */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort  [in]: Specify the SPI port                                                                 */
/*    u32Clock1 [in]: Specify the SPI clock rate in Hz. It's the clock rate of SPI base clock and variable */
/*                    clock 1.                                                                             */
/*    u32Clock2 [in]: Specify the SPI clock rate in Hz. It's the clock rate of variable clock 2.           */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    The actual clock rate of SPI engine clock is returned.                                               */
/*    SPI engine clock rate = APB clock rate / ((DIVIDER + 1) * 2)                                         */
/*    The actual clock may different to the target SPI clock due to hardware limitation.                   */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Configure the SPI clock. Only for master mode.                                                       */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_SetClockFreq(E_DRVSPI_PORT eSpiPort, uint32_t u32Clock1, uint32_t u32Clock2)
{
    uint32_t u32Div;
    uint32_t u32Pclk;
    
    u32Pclk = SystemCoreClock;//DrvSYS_GetHCLKFreq();
    
    u32Div = 0xFFFF; /* Initial value */

    if(u32Clock2!=0)
    {
        u32Div = (((u32Pclk / u32Clock2) + 1) >> 1) - 1;
        if(u32Div > 65535)
            u32Div = 65535;
        SPI->DIVIDER_BITS.DIVIDER2 = u32Div;
    }
    else
        SPI->DIVIDER_BITS.DIVIDER2 = 0xFFFF;
    
    if(u32Clock1!=0)
    {
        u32Div = (((u32Pclk / u32Clock1) + 1) >> 1) - 1;
        if(u32Div > 0xFFFF)
            u32Div = 0xFFFF;
        SPI->DIVIDER_BITS.DIVIDER1 = u32Div;
    }
    else
        SPI->DIVIDER_BITS.DIVIDER1 = 0xFFFF;
    
    return ( u32Pclk / ((u32Div+1)*2) );
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_GetClock1Freq                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    The current SPI bus clock frequency in Hz.                                                           */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Get the SPI engine clock rate in Hz. Only for master mode.                                           */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_GetClock1Freq(E_DRVSPI_PORT eSpiPort)
{
    uint32_t u32Div;
    uint32_t u32ApbClock;

    u32ApbClock = SystemCoreClock;//DrvSYS_GetHCLKFreq();
    u32Div = SPI->DIVIDER_BITS.DIVIDER1;
    return ((u32ApbClock >> 1) / (u32Div + 1));   /* SPI_CLK = APB_CLK / ((Divider + 1) * 2) */
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_GetClock2Freq                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    The frequency of variable clock 2 in Hz.                                                             */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Get the clock rate of variable clock 2 in Hz. Only for master mode.                                  */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_GetClock2Freq(E_DRVSPI_PORT eSpiPort)
{
    uint32_t u32Div;
    uint32_t u32ApbClock;

    u32ApbClock = SystemCoreClock;//DrvSYS_GetHCLKFreq();
    u32Div = SPI->DIVIDER_BITS.DIVIDER2;
    return ((u32ApbClock >> 1) / (u32Div + 1));   /* SPI_CLK = APB_CLK / ((Divider + 1) * 2) */
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetVariableClockFunction                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*    bEnable    [in]: TRUE  -- Enable variable clock function                                             */
/*                     FALSE -- Disable variable clock function                                            */
/*    u32Pattern [in]: Specify the variable clock pattern                                                  */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the variable clock function. Only for master mode.                                               */
/*    If the bit pattern of VARCLK is '0', the output frequency of SPICLK is according to the value of     */
/*    DIVIDER.                                                                                             */
/*    If the bit pattern of VARCLK is '1', the output frequency of SPICLK is according to the value of     */
/*    DIVIDER2.                                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetVariableClockFunction(E_DRVSPI_PORT eSpiPort, uint8_t bEnable, uint32_t u32Pattern)
{
    if(bEnable)
    {
        SPI->CNTRL_BITS.VARCLK_EN = 1;
        SPI->VARCLK = u32Pattern;
    }
    else
        SPI->CNTRL_BITS.VARCLK_EN = 0;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_EnableInt                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort    [in]: Specify the SPI port                                                               */
/*    pfnCallback [in]: The callback function of SPI interrupt.                                            */
/*    u32UserData [in]: The parameter which will be passed to the callback function.                       */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Enable the SPI interrupt of the specified SPI port and install the callback function.                */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_EnableInt(E_DRVSPI_PORT eSpiPort, PFN_DRVSPI_CALLBACK pfnCallback, uint32_t u32UserData)
{
    
    if(pfnCallback != NULL)
    {
        g_sSpiHandler[eSpiPort].pfncallback = pfnCallback;
        g_sSpiHandler[eSpiPort].u32userData = u32UserData;
    }
    
    SPI->CNTRL_BITS.IE = 1;
    
    if(eSpiPort == eDRVSPI_PORT0)
    {
        NVIC_SetPriority(SPI_IRQn, (1<<__NVIC_PRIO_BITS) - 2);
        NVIC_EnableIRQ(SPI_IRQn);
    }
}   

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_DisableInt                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Disable the SPI interrupt.                                                                           */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_DisableInt(E_DRVSPI_PORT eSpiPort)
{
    if(eSpiPort == eDRVSPI_PORT0)
    {
        NVIC_DisableIRQ(SPI_IRQn);
    }
    else
    	return;
    
    g_sSpiHandler[eSpiPort].pfncallback = NULL;
    g_sSpiHandler[eSpiPort].u32userData = 0;
    
    SPI->CNTRL_BITS.IE = 0;   
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_GetIntFlag                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    0: SPI interrupt doesn't occur                                                                       */
/*    1: SPI interrupt occurs                                                                              */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Return the SPI interrupt flag                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_GetIntFlag(E_DRVSPI_PORT eSpiPort)
{
    return (SPI->CNTRL_BITS.IF);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_ClrIntFlag                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None                                                                                                 */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Clear the SPI interrupt flag                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_ClrIntFlag(E_DRVSPI_PORT eSpiPort)
{
    SPI->CNTRL_BITS.IF = 1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SingleRead                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]:  Specify the SPI port                                                               */
/*    pu32Data   [out]: Store the data got from the SPI bus.                                               */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  The data stored in pu32Data is valid.                                                         */
/*    FALSE: The data stored in pu32Data is invalid.                                                       */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Read data from SPI Rx registers and trigger SPI for next transfer.                                   */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_SingleRead(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Data)
{
    if(SPI->CNTRL_BITS.GO_BUSY==1)
        return FALSE;

    *pu32Data = SPI->RX0;
    SPI->CNTRL_BITS.GO_BUSY = 1;
    return TRUE;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SingleWrite                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort  [in]:  Specify the SPI port                                                                */
/*    pu32Data  [in]:  Store the data which will be transmitted through the SPI bus.                       */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  The data stored in pu32Data has been transferred.                                             */
/*    FALSE: The SPI is in busy. The data stored in pu32Data has not been transferred.                     */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Write data to SPI bus and trigger SPI to start transfer.                                             */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_SingleWrite(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Data)
{
    if(SPI->CNTRL_BITS.GO_BUSY==1)
        return FALSE;

    SPI->TX0 = *pu32Data;
    SPI->CNTRL_BITS.GO_BUSY = 1;
    return TRUE;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_BurstRead                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort  [in]:  Specify the SPI port                                                                */
/*    pu32Buf   [out]: Store the data got from the SPI bus.                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  The data stored in pu32Buf is valid.                                                          */
/*    FALSE: The data stored in pu32Buf is invalid.                                                        */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Read two words of data from SPI Rx registers and then trigger SPI for next transfer.                 */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_BurstRead(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf)
{
    if(SPI->CNTRL_BITS.GO_BUSY==1)
        return FALSE;

    pu32Buf[0] = SPI->RX0;
    pu32Buf[1] = SPI->RX1;
    SPI->CNTRL_BITS.GO_BUSY = 1;

    return TRUE;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_BurstWrite                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort  [in]:  Specify the SPI port                                                                */
/*    pu32Buf   [in]:  Store the data which will be transmitted through the SPI bus.                       */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  The data stored in pu32Buf has been transferred.                                              */
/*    FALSE: The SPI is in busy. The data stored in pu32Buf has not been transferred.                      */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Write two words of data to SPI bus and then trigger SPI to start transfer.                           */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_BurstWrite(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf)
{
    if(SPI->CNTRL_BITS.GO_BUSY==1)
        return FALSE;

    SPI->TX0 = pu32Buf[0];
    SPI->TX1 = pu32Buf[1];
    SPI->CNTRL_BITS.GO_BUSY = 1;

    return TRUE;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_DumpRxRegister                                                                         */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]:  Specify the SPI port                                                             */
/*    pu32Buf      [out]: Store the data got from Rx registers.                                            */
/*    u32DataCount [in]:  The count of data read from Rx registers.                                        */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    The count of data actually read from Rx registers.                                                   */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Read data from Rx registers. This function will not trigger a SPI data transfer.                     */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_DumpRxRegister(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf, uint32_t u32DataCount)
{
    if(u32DataCount>2)
        u32DataCount = 2;
    
    pu32Buf[0] = SPI->RX0;
	
	if(u32DataCount == 2)
		pu32Buf[1] = SPI->RX1;

    return u32DataCount;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetTxRegister                                                                          */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]:  Specify the SPI port                                                             */
/*    pu32Buf      [in]:  Store the data which will be written to Tx registers.                            */
/*    u32DataCount [in]:  The count of data write to Tx registers.                                         */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    The count of data actually written to Tx registers.                                                  */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Write data to Tx registers. This function will not trigger a SPI data transfer.                      */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_SetTxRegister(E_DRVSPI_PORT eSpiPort, uint32_t *pu32Buf, uint32_t u32DataCount)
{
    if(u32DataCount>2)
        u32DataCount = 2;
    
    SPI->TX0 = pu32Buf[0];
	
	if(u32DataCount == 2)
		SPI->TX1 = pu32Buf[1];

    return u32DataCount;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetGo                                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]:  Specify the SPI port                                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Set the GO_BUSY bit to trigger a SPI data trasfer.                                                   */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetGo(E_DRVSPI_PORT eSpiPort)
{
    SPI->CNTRL_BITS.GO_BUSY = 1;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_ClrGo                                                                                  */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]:  Specify the SPI port                                                             */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Clear the GO_BUSY bit.                                                                               */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_ClrGo(E_DRVSPI_PORT eSpiPort)
{
    SPI->CNTRL_BITS.GO_BUSY = 0;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetFIFOMode                                                                            */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort    [in]: Specify the SPI port                                                               */
/*    bEnable     [in]: Enable (TRUE) / Disable (FALSE)                                                    */
/*    i32Interval [in]: In FIFO mode, it could be 2~15 and 0. 0 indicates the maximum suspend interval; 2  */
/*                      indicates the minimum suspend interval. Please refer to M051 TRM for the actual    */
/*                      suspend interval.                                                                  */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    E_DRVSPI_ERR_SUSPEND_INTERVAL: The suspend interval setting is out of range.                         */
/*    E_SUCCESS: Success.                                                                                  */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Enable/disable FIFO mode. If the caller enables FIFO mode, configure the setting of suspend cycle.   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t DrvSPI_SetFIFOMode(E_DRVSPI_PORT eSpiPort, uint8_t bEnable, int32_t i32Interval)
{
    if( bEnable )
    {
        if( (i32Interval < 2) || (i32Interval > 15) )
        {
            /* If out of range, specify the maximum suspend cycle and return error code. */
            SPI->CNTRL_BITS.SP_CYCLE = 0;
            /* Enable the FIFO mode */
            SPI->CNTRL_BITS.FIFO = 1;
            if(i32Interval==0)
                return E_SUCCESS;
            else
                return E_DRVSPI_ERR_SUSPEND_INTERVAL;
        }
        /* Set the suspend cycle */
        SPI->CNTRL_BITS.SP_CYCLE = i32Interval;
        /* Enable the FIFO mode */
        SPI->CNTRL_BITS.FIFO = 1;
        return E_SUCCESS;
    }
    
    /* Disable the FIFO mode */
    SPI->CNTRL_BITS.FIFO = 0;
    return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_IsRxEmpty                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  Rx buffer is empty.                                                                           */
/*    FALSE: Rx buffer is not empty.                                                                       */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Check the status of the Rx buffer of the specified SPI port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_IsRxEmpty(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI->CNTRL_BITS.RX_EMPTY)?TRUE:FALSE);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_IsRxFull                                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  Rx buffer is full.                                                                            */
/*    FALSE: Rx buffer is not full.                                                                        */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Check the status of the Rx buffer of the specified SPI port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_IsRxFull(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI->CNTRL_BITS.RX_FULL)?TRUE:FALSE);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_IsTxEmpty                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  Tx buffer is empty.                                                                           */
/*    FALSE: Tx buffer is not empty.                                                                       */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Check the status of the Tx buffer of the specified SPI port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_IsTxEmpty(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI->CNTRL_BITS.TX_EMPTY)?TRUE:FALSE);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_IsTxFull                                                                               */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort   [in]: Specify the SPI port                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    TRUE:  Tx buffer is full.                                                                            */
/*    FALSE: Tx buffer is not full.                                                                        */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Check the status of the Tx buffer of the specified SPI port.                                         */
/*---------------------------------------------------------------------------------------------------------*/
uint8_t DrvSPI_IsTxFull(E_DRVSPI_PORT eSpiPort)
{
    return ((SPI->CNTRL_BITS.TX_FULL)?TRUE:FALSE);
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_SetDivOne                                                                              */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    eSpiPort     [in]: Specify the SPI port                                                              */
/*    bEnable      [in]: Enable (TRUE) / Disable (FALSE)                                                   */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    None.                                                                                                */
/*                                                                                                         */
/* Description:                                                                                            */
/*    If enable the feature of "divided by one", the SPI clock rate will equal the system clock frequency. */
/*---------------------------------------------------------------------------------------------------------*/
void DrvSPI_SetDivOne(E_DRVSPI_PORT eSpiPort, uint8_t bEnable)
{
    SPI->CNTRL2_BITS.DIV_ONE = bEnable;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function: DrvSPI_GetVersion                                                                             */
/*                                                                                                         */
/* Parameters:                                                                                             */
/*    None.                                                                                                */
/*                                                                                                         */
/* Returns:                                                                                                */
/*    Version number.                                                                                      */
/*                                                                                                         */
/* Description:                                                                                            */
/*    Get the version number of NUC100 SPI driver.                                                         */
/*---------------------------------------------------------------------------------------------------------*/
uint32_t DrvSPI_GetVersion(void)
{
    return DRVSPI_VERSION_NUM;
}
