/**************************************************************************//**
 * @file     FMC.h
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/11/17 2:37p $
 * @brief    MINI51X series FMC driver header file
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#ifndef __FMC_H__
#define __FMC_H__

#ifdef __cplusplus
extern "C"
{
#endif
    
/** @addtogroup MINI51X_Device_Driver MINI51X Device Driver
  @{
*/

/** @addtogroup MINI51X_FMC_Driver FMC Driver
  @{
*/

/** @addtogroup MINI51X_FMC_EXPORTED_CONSTANTS FMC Exported Constants
  @{
*/

#define inp32(port)			(*((volatile unsigned int *)(port)))
#define BOOL  uint32_t

/*---------------------------------------------------------------------------------------------------------*/
/* Define Base Address                                                                                     */
/*---------------------------------------------------------------------------------------------------------*/
#define FMC_APROM_BASE          0x00000000UL    /*!< APROM  Base Address         */  
#define FMC_LDROM_BASE          0x00100000UL    /*!< LDROM  Base Address         */
#define FMC_LDROM_END           0x00100800UL    /*!< LDROM  End Address          */
#define FMC_CONFIG_BASE         0x00300000UL    /*!< CONFIG Base Address         */

#define FMC_FLASH_PAGE_SIZE     0x200           /*!< Flash Page Size (512 Bytes) */
#define FMC_LDROM_SIZE          0x800           /*!< LDROM Size (2 Kbytes)       */

#define Config0         0x00300000
#define Config1         0x00300004

#define ISPGO           0x01
#define ISPFF           0x00000040

/*---------------------------------------------------------------------------------------------------------*/
/*  ISPCMD constant definitions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/
#define FMC_ISPCMD_READ         0x00            /*!< ISP Command: Read Flash        */
#define FMC_ISPCMD_PROGRAM      0x21            /*!< ISP Command: Program Flash     */
#define FMC_ISPCMD_PAGE_ERASE   0x22            /*!< ISP Command: Page Erase Flash  */
#define FMC_ISPCMD_READ_CID     0x0B            /*!< ISP Command: Read Company ID   */
#define FMC_ISPCMD_READ_DID     0x0C            /*!< ISP Command: Read Device ID    */
#define FMC_ISPCMD_READ_UID     0x04            /*!< ISP Command: Read Unique ID    */
#define FMC_ISPCMD_VECMAP       0x2E            /*!< ISP Command: Vector Page Remap */

#define ISP_Read        0x00
#define ISP_Program     0x21
#define ISP_PageErase   0x22

// #define FMC_BASE        0x5000C000UL		
//  
// #define FISPCON         (FMC_BASE+0x000)			
// #define ISPADR         (FMC_BASE+0x004)		
// #define ISPDAT         (FMC_BASE+0x008)			
// #define ISPCMD         (FMC_BASE+0x00C)			
// #define ISPTRG         (FMC_BASE+0x010)			
// #define DFBADDR        (FMC_BASE+0x014)
// #define FATCON         (FMC_BASE+0x018)			
// #define ICPCON         (FMC_BASE+0x01C)			
// #define RMPCON         (FMC_BASE+0x020)	

/*@}*/ /* end of group MINI51X_FMC_EXPORTED_CONSTANTS */

/** @addtogroup MINI51X_FMC_EXPORTED_FUNCTIONS FMC Exported Functions
  @{
*/

#define FMC_SET_APROM_BOOT()        (FMC->ISPCTL &= ~FMC_ISPCTL_BS_Msk)         /*!< Select booting from APROM  */
#define FMC_SET_LDROM_BOOT()        (FMC->ISPCTL |= FMC_ISPCTL_BS_Msk)          /*!< Select booting from LDROM  */
#define FMC_DISABLE_AP_UPDATE()     (FMC->ISPCTL &= ~FMC_ISPCTL_APUEN_Msk)      /*!< Disable APROM update  */
#define FMC_DISABLE_CFG_UPDATE()    (FMC->ISPCTL &= ~FMC_ISPCTL_CFGUEN_Msk)     /*!< Disable User Config update  */
#define FMC_DISABLE_LD_UPDATE()     (FMC->ISPCTL &= ~FMC_ISPCTL_LDUEN_Msk)      /*!< Disable LDROM update  */
#define FMC_DISABLE_ISP()           (FMC->ISPCTL &= ~FMC_ISPCTL_ISPEN_Msk)      /*!< Disable ISP function  */
#define FMC_ENABLE_AP_UPDATE()      (FMC->ISPCTL |=  FMC_ISPCTL_APUEN_Msk)      /*!< Enable APROM update  */
#define FMC_ENABLE_LD_UPDATE()      (FMC->ISPCTL |=  FMC_ISPCTL_LDUEN_Msk)      /*!< Enable LDROM update  */
#define FMC_ENABLE_CFG_UPDATE()     (FMC->ISPCTL |=  FMC_ISPCTL_CFGUEN_Msk)     /*!< Enable User Config update  */
#define FMC_ENABLE_ISP()            (FMC->ISPCTL |=  FMC_ISPCTL_ISPEN_Msk)      /*!< Enable ISP function  */
#define FMC_GET_FAIL_FLAG()         ((FMC->ISPCTL & FMC_ISPCTL_ISPFF_Msk) ? 1 : 0)  /*!< Get ISP fail flag  */


// extern void FMC_SetBootSource(int32_t i32BootSrc);
// extern void FMC_Close(void);
// extern void FMC_DisableAPUpdate(void);
// extern void FMC_DisableConfigUpdate(void);
// extern void FMC_DisableLDUpdate(void);
// extern void FMC_EnableAPUpdate(void);
// extern void FMC_EnableConfigUpdate(void);
// extern void FMC_EnableLDUpdate(void);
extern int32_t FMC_Erase(uint32_t u32PageAddr);
// extern int32_t FMC_GetBootSource(void);
// extern void FMC_Open(void);
//extern uint32_t FMC_Read (uint32_t u32Addr);
extern int32_t FMC_Read(uint32_t u32Addr, unsigned int * u32Data);
// extern uint32_t FMC_ReadCID(void);
// extern uint32_t FMC_ReadDID(void);
// extern uint32_t FMC_ReadPID(void);
// extern uint32_t FMC_ReadUCID(uint32_t u32Index);
// extern uint32_t FMC_ReadUID(uint32_t u32Index);
// extern uint32_t FMC_ReadDataFlashBaseAddr(void);
// extern void FMC_SetVectorPageAddr(uint32_t u32PageAddr);
extern int FMC_Write(uint32_t u32Addr, uint32_t u32Data);
// extern int32_t FMC_ReadConfig(uint32_t *u32Config, uint32_t u32Count);
// extern int32_t FMC_WriteConfig(uint32_t *u32Config, uint32_t u32Count);

extern void EraseAP(BOOL bAprom, unsigned int addr_start, unsigned int addr_end);
extern void ReadData(unsigned int addr_start, unsigned int addr_end, unsigned int* data);
extern void WriteData(unsigned int addr_start, unsigned int addr_end, unsigned int *data);
extern void UpdateConfig(unsigned int *data, unsigned int *res);
extern void GetDataFlashInfo(uint32_t *addr, uint32_t *size);

/*@}*/ /* end of group MINI51X_FMC_EXPORTED_FUNCTIONS */

/*@}*/ /* end of group MINI51X_FMC_Driver */

/*@}*/ /* end of group MINI51X_Device_Driver */

#ifdef __cplusplus
}
#endif


#endif

