/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "NUC029FAE.h"

#include "FMC.h"

#define USING_GPIO				0
#define SUPPORT_WRITECKSUM		0
#define USING_AUTODETECT 		1	


#define CMD_UPDATE_APROM	0x000000A0
#define CMD_UPDATE_CONFIG	0x000000A1
#define CMD_READ_CONFIG		0x000000A2
#define CMD_ERASE_ALL		0x000000A3
#define CMD_SYNC_PACKNO		0x000000A4
#define CMD_GET_FWVER		0x000000A6
#define CMD_RUN_APROM		0x000000AB
#define CMD_RUN_LDROM		0x000000AC
#define CMD_RESET			0x000000AD
#define CMD_CONNECT			0x000000AE
#define CMD_DISCONNECT		0x000000AF

#define CMD_GET_DEVICEID	0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define PACKET_SIZE	64

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2

static UART_T	*g_pUART = UART;
//static IRQn_Type	g_UARTIRQ = UART_IRQn;

#define UART_GET_TX_POINTER(UART)    (((UART)->FSR & UART_FSR_TX_POINTER_Msk) >> UART_FSR_TX_POINTER_Pos)

#define FW_VERSION		0x25

#undef	printf
#define printf(...)


static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus = DISCONNECTED;
__align(4) static uint8_t uart_rcvbuf[64];
__align(4) static uint8_t uart_sendbuf[64];
__align(4) static uint8_t aprom_buf[PAGE_SIZE];
BOOL bUartDataReady;
BOOL bUsbInReady, bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid, g_timecnt;
int SysTimerDelay(uint32_t us);

#if(SUPPORT_WRITECKSUM)
static uint32_t g_ckbase = (0x20000 - 8);
static void CheckCksumBase(void);
#endif

void NMI_Handler(void)
{
//	GCR->BODCTL_BITS.INTF = 1;
	SYS->BODCTL |= SYS_BODCR_BOD_INTF_Msk;
}


/*
* send data for UART
*/
static __inline void PutString()
{
	int i;
	
	for(i = 0; i<PACKET_SIZE; i++)
	{
		//while( UART0->FSR.TE == 0);
		while(UART_GET_TX_POINTER(g_pUART) >= 14);
		g_pUART->THR = uart_sendbuf[i];
	}
}

/*
* UART0 interrupt handler. handle receive only
*/
void UART_IRQHandler(void)
{  
	/*----- Determine interrupt source -----*/
	
		if (inpw(&g_pUART->ISR/*COM_ISR*/) & 0x11) //RDA FIFO interrupt or RDA timeout interrupt 
   		{
   			while(((inpw(&g_pUART->FSR/*COM_FSR*/) & (0x4000)) == 0) && (bufhead < 64))//RX fifo not empty
				uart_rcvbuf[bufhead++] = inpw(&g_pUART->RBR/*COM_RX*/);
		}
	
		if(bufhead == 64)
		{
			if(g_connStatus == DISCONNECTED)
			{
				// check first command is connect
				if(uart_rcvbuf[0] == CMD_CONNECT)
				{
					bUartDataReady = TRUE;
					bufhead = 0;
				}
			}
			else
			{
				bUartDataReady = TRUE;
				bufhead = 0;
			}
		}

}


static __inline void UartInit(void)
{
    /* Multi-Function Pin: Enable UART0:Tx Rx */
  	SYS->P1_MFP &= ~(SYS_MFP_P12_Msk | SYS_MFP_P13_Msk);
	SYS->P1_MFP |= (SYS_MFP_P12_RXD | SYS_MFP_P13_TXD);

    /* Configure GCR to reset UART0 */
	SYS->IPRSTC2 |= SYS_IPRSTC2_UART_RST_Msk;
	SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART_RST_Msk;
										  
    /* Enable UART clock */
//    CLK->APBCLK_BITS.UART_EN = 1;
	CLK->APBCLK |= CLK_APBCLK_UART_EN_Msk;

    /* Select UART clock source */
//    CLK->CLKSEL1_BITS.UART_S = 2;//0:12M; 2:22.1184M
	CLK->CLKSEL1 |= CLK_CLKSEL1_UART_S_Msk;	 

    /* Data format */
//    UART0->LCR_BITS.WLS = 3;
	UART->LCR |= UART_LCR_WLS_Msk;

	outpw(&g_pUART->FCR, 0x32);//14
//	UART0->TOR_BITS.TOIC = 0x40;
	UART->TOR = (UART->TOR & ~UART_TOR_TOIC_Msk) | 0x40;


    /* Configure the baud rate */
    //BaudRateCalculator(SystemFrequency, 115200, &UART0->BAUD);
#if 0//12M
    *((__IO uint32_t *)&g_pUART->BAUD) = 0x3F000066;//115200
    //*((__IO uint32_t *)&UART0->BAUD) = 0xB;//57600
#else //22.1184M
	 *((__IO uint32_t *)&g_pUART->BAUD) = 0x2F00000A;//115200
	 //*((__IO uint32_t *)&UART0->BAUD) = 0x3F00017B;//0x16//57600
#endif

    NVIC_SetPriority (UART_IRQn, 2);
    NVIC_EnableIRQ(UART_IRQn);
    
    //UART0->IER.RDA_IEN = 1;UART0->IER.TOC_IEN = 1;UART0->IER.TOC_EN = 1;//Enable Timeout Counter
//	outpw(&UART0->IER, 0x811);
	UART_ENABLE_INT(UART , UART_IER_TIME_OUT_EN_Msk | UART_IER_RDA_IEN_Msk | UART_IER_RLS_IEN_Msk | UART_IER_RTO_IEN_Msk);

}

static uint16_t Checksum (unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for(c = 0 , i = 0 ; i < len; i++) 
	{
        c += buf[i];
    }
    return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}

int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t lcmd, packno, srclen, i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	
	
	response = uart_sendbuf;

	pSrc = buffer;
	srclen = len;
	//cksum = Checksum(pSrc, srclen);
	lcmd = inpw(pSrc);
	
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	
	if((lcmd) && (lcmd != CMD_RESEND_PACKET))
		gcmd = lcmd;
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	
   	if((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;

	if(lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;	
	//else if(lcmd == CMD_READ_CONFIG)
	//{
		//FMC_Read(Config0, (uint32_t*)(response+8));
		//FMC_Read(Config1, (uint32_t*)(response+12));
	//}
	else if(lcmd == CMD_GET_DEVICEID)
	{
		//outpw(response+8, g_pdid);
		outpw(response+8, SYS->PDID);
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		outpw(&SYS->RSTSRC, 3);//clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFE);//ISP disable
		}

		outpw(&FMC->ISPCON, i);
		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

		/* Trap the CPU */
		while(1);
	}
	else if(lcmd == CMD_CONNECT)
	{
		g_connStatus = CONNECTED;
		g_packno = 1;
	}
	else if(lcmd == CMD_DISCONNECT)
	{
		g_connStatus = DISCONNECTED;
	}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		
		if (lcmd == CMD_ERASE_ALL)          //erase APROM + data flash
		{
			EraseAP(FALSE, 0, g_apromSize);        //erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);                   //don't erase data flash
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		else
			bUpdateApromCmd = TRUE;
	}


#if(SUPPORT_WRITECKSUM)
	else if(lcmd == CMD_WRITE_CHECKSUM)//write cksum to aprom last
	{
		cktotallen = inpw(pSrc);
		lcksum = inpw(pSrc+4);
		CheckCksumBase();
		ReadData(g_ckbase & 0xFFE00, g_ckbase, (uint32_t*)aprom_buf);
		outpw(aprom_buf+PAGE_SIZE - 8, cktotallen);
		outpw(aprom_buf+PAGE_SIZE - 4, lcksum);
		FMC_Erase(g_ckbase & 0xFFE00);
		WriteData(g_ckbase & 0xFFE00, g_ckbase + 8, (uint32_t*)aprom_buf);
	}
#endif
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCON)&0x2)? 2 : 1);
	}
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)//g_dataFlashAddr
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			StartAddress = 0;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;

		//return 0;
	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	else if (lcmd == CMD_RESEND_PACKET)
	{  	   
	   StartAddress -= LastDataLen;
	   TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%PAGE_SIZE) >= (PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+PAGE_SIZE);
		goto out;

	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
			TotalLen -= srclen;
		else{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
		}

		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
//		memset(pSrc, 0, srclen);
	
		{
			uint32_t *p = (uint32_t*)pSrc;
			for(i = 0; i<srclen/4; i++)
				p[i] = 0;
		}
		
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	//NVIC_DisableIRQ(USBD_IRQn);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;
	//NVIC_EnableIRQ(USBD_IRQn);
	//TotalLen -= srclen;

	return 0;
}

//0: timeout cleared by interrrupt
//1: timeout happen
int SysTimerDelay(uint32_t us)
{
  //SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
	SysTick->LOAD = us * 22; /* using 22MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) 
				| (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);

    return 1;
}

#if(SUPPORT_WRITECKSUM)
static void CheckCksumBase()
{
	//skip data flash
	unsigned int aprom_end = g_apromSize, data;
	int result;
	
	result = FMC_Read(0x1F000 - 8, &data);
	if(result == 0)//128K flash
    {
    	FMC_Read(Config0, &data);
    	if((data&0x01)==0)//DFEN enable
    	{
    		FMC_Read(Config1, &aprom_end);
    	}
    	g_ckbase = aprom_end - 8;
    	return;
    }
    else// less than 128K
    {
    	aprom_end = 0x10000;//64K
    	do{
		    result = FMC_Read(aprom_end - 8, &data);
		    if(result == 0)
		    	break;
    		aprom_end = aprom_end/2;
    	}while(aprom_end > 4096);

		g_ckbase = aprom_end - 8;
	}
}

uint8_t CheckCmd()
{
	int i;
	for(i = 0; i < bufhead; i++)
	{
		if(uart_rcvbuf[i] == 0xA4)//CMD_SYNC_PACKNO
			return TRUE;
	}
	return FALSE;
}
#endif


//the smallest of APROM size is 2K 
static uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}


#define GET_ISPPIN_STATUS()	(GPIO3->PIN & (1 << 2))//p3.2 for Mini51 if don't using AUTO_DETECT
#define UNLOCKREG(x)        *((__IO uint32_t *)(GCR_BASE + 0x100)) = 0x59;*((__IO uint32_t *)(GCR_BASE + 0x100)) = 0x16;*((__IO uint32_t *)(GCR_BASE + 0x100)) = 0x88


int32_t main()
{
    extern uint32_t SystemFrequency;
    uint8_t volatile	bufhead_bak=0;
#if(SUPPORT_WRITECKSUM)
    uint32_t totallen, cksum;
#endif
  
	SYS_UnlockReg();
	
  
	CLK->PWRCON |= CLK_PWRCON_OSC22M_EN_Msk;
 
  /* Waiting for 22M Xtal stalble */
 	while((CLK->CLKSTATUS & CLK_CLKSTATUS_OSC22M_STB_Msk) == 0);
  
	FMC->ISPCON |= FMC_ISPCON_ISPEN_Msk;

	g_apromSize = GetApromSize();
	GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
	
	g_pdid = SYS->PDID;

	UartInit();
		
	//timeout 30ms
	SysTick->LOAD = 30000 * 22; /* using 22MHz cpu clock*/
	SysTick->VAL   =  (0x00);
	SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

	while(1)
	{
		uint32_t lcmd;
		if((bufhead >= 4) || (bUartDataReady == TRUE))
		{
			lcmd = inpw(uart_rcvbuf);
			if(lcmd == CMD_CONNECT) 
				goto _ISP;
			else
			{
				bUartDataReady = FALSE;
				bufhead = 0;
			}
		}

		if((SysTick->CTRL & (1 << 16)) != 0)//timeout
		{
			//APROM
			outpw(&SYS->RSTSRC, 3);	//clear bit
	  	outpw(&FMC->ISPCON, inpw(&FMC->ISPCON) & 0xFFFFFFFC);
	  	outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));	//SYSRESETREQ
	
			/* Trap the CPU */
			while(1);
		}
			
	}
	
_ISP:
    while(1)
    {

    	if(bUartDataReady == TRUE)
    	{
				g_timecnt = 0;
				ParseCmd(uart_rcvbuf, PACKET_SIZE, FALSE);
				PutString();
				bUartDataReady = FALSE;
    	}

    	//timeout happen; but byte is less than 64 bytes; host goes wrong
			if(bufhead > 0)
			{
				 if(g_timecnt == 0)
						bufhead_bak = bufhead;
				 
				 SysTimerDelay(1);//1us
					g_timecnt++;
				 
				 if(g_timecnt > 2000)//2ms
				 {
					 g_timecnt = 0;
					 if(bufhead_bak == bufhead)
					 {
						//g_connStatus = DISCONNECTED;
							bufhead = 0;
					 }
				 }
			}
    }

}








































































































