/****************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/11/27 5:58p $
 * @brief    NUC029 Series UART Interface Controller Driver Sample Code
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include <stdio.h>
#include "NUC029xAN.h"
#include "FMC.h"

#define PLL_CLOCK           50000000


#define USING_AUTODETECT 		1	


#define CMD_UPDATE_APROM	0x000000A0
#define CMD_UPDATE_CONFIG	0x000000A1
#define CMD_READ_CONFIG		0x000000A2
#define CMD_ERASE_ALL		  0x000000A3
#define CMD_SYNC_PACKNO		0x000000A4
#define CMD_GET_FWVER		  0x000000A6
#define CMD_RUN_APROM		  0x000000AB
#define CMD_RUN_LDROM		  0x000000AC
#define CMD_RESET			    0x000000AD
#define CMD_CONNECT			  0x000000AE
#define CMD_DISCONNECT		0x000000AF

#define CMD_GET_DEVICEID	0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET     0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	  0x00000004UL

#define PACKET_SIZE	64

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2

#define UART_GET_TX_POINTER(UART)    (((UART)->FSR & UART_FSR_TX_POINTER_Msk) >> UART_FSR_TX_POINTER_Pos)

#define FW_VERSION		0x25

static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus = DISCONNECTED;
__align(4) static uint8_t uart_rcvbuf[64];
__align(4) static uint8_t uart_sendbuf[64];
__align(4) static uint8_t aprom_buf[PAGE_SIZE];
BOOL bUartDataReady;
BOOL bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid, g_timecnt;
static volatile int  g_enter_ISP;

void WDT_IRQHandler(void)
{

	if(WDT_GET_TIMEOUT_INT_FLAG())
	{
		/* Clear WDT time-out interrupt flag */
		WDT_CLEAR_TIMEOUT_INT_FLAG();
		bufhead = 0;

		WDT->WTCR &= ~WDT_WTCR_WTE_Msk;//_WDT_DISABLE_COUNTING();
	}

}

void timer_init()
{
	/* 
	 *  Configure Timer A, clock source from XTL_22M. Prescale 22
	 *  Give 1 seconds counter => 1000000
	 */
	CLK->CLKSEL1 |= CLK_CLKSEL1_TMR0_S_Msk;	
	CLK->APBCLK |= CLK_APBCLK_TMR0_EN_Msk; 
	TIMER0->TCSR = 0;                  	/* disable timer */
	TIMER0->TISR = 0x3;                 /* write 1 to clear for safty */
	TIMER0->TCSR = (21 << TIMER_TCSR_PRESCALE_Pos) |	// pre-scale = 22
                   (0x3 << TIMER_TCSR_MODE_Pos) | 		// continuous mode
                   TIMER_TCSR_CEN_Msk;

}

uint32_t get_timer_value()
{
	return TIMER0->TDR;
}

/*
* send data for UART
*/
static __inline void PutString()
{
	int i;
	
	for(i = 0; i < PACKET_SIZE; i++)
	{
		while(UART_GET_TX_POINTER(UART0) >= 14);	
		
		UART0->THR = uart_sendbuf[i];
	}
}

/*
* UART0 interrupt handler. handle receive only
*/
void UART0_IRQHandler(void)
{  
	/*----- Determine interrupt source -----*/
	
    if (inpw(&UART0->ISR) & 0x11) //RDA FIFO interrupt or RDA timeout interrupt 
    {
    while(((inpw(&UART0->FSR) & (0x4000)) == 0) && (bufhead < 64))//RX fifo not empty
    uart_rcvbuf[bufhead++] = inpw(&UART0->RBR/*COM_RX*/);
    }

    WDT->WTCR |= WDT_WTCR_WTR_Msk;//WDT_RESET_COUNTER();
    WDT->WTCR |= WDT_WTCR_WTE_Msk;//WDT_ENABLE_COUNTING;	


    if(bufhead == 64)
    {
        if(g_connStatus == DISCONNECTED)
        {
            // check first command is connect
            if(uart_rcvbuf[0] == CMD_CONNECT)
            {
                bUartDataReady = TRUE;
                bufhead = 0;
            }
            else
            {
                bUartDataReady = FALSE;
                bufhead = 0;
            }
        }
        else
        {
            bUartDataReady = TRUE;
            bufhead = 0;
        }
    }
		
}

static uint16_t Checksum (unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for(c = 0 , i = 0 ; i < len; i++) 
	{
        c += buf[i];
    }
    return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}

uint32_t Global_cmd;
int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t lcmd, packno, srclen, i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	
	
	response = uart_sendbuf;

	pSrc = buffer;
	srclen = len;
	//cksum = Checksum(pSrc, srclen);
	lcmd = inpw(pSrc);
	
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	
	if((lcmd) && (lcmd != CMD_RESEND_PACKET))
		gcmd = lcmd;
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	
   	if((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;

	if(lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;	
	else if(lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, SYS->PDID);
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		outpw(&SYS->RSTSRC, 3);//clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFE);//ISP disable
		}

		outpw(&FMC->ISPCON, i);
		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

		/* Trap the CPU */
		while(1);
	}
	else if(lcmd == CMD_CONNECT)
	{
		g_connStatus = CONNECTED;
		g_packno = 1;
		g_enter_ISP = 1;
	}
	else if(lcmd == CMD_DISCONNECT)
	{
		g_connStatus = DISCONNECTED;
	}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		
		if (lcmd == CMD_ERASE_ALL)          //erase APROM + data flash
		{
			EraseAP(FALSE, 0, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);//don't erase data flash
		
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		else
			bUpdateApromCmd = TRUE;
	}
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCON)&0x2)? 2 : 1);
	}
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			StartAddress = 0;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;

		//return 0;
	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	else if(lcmd == CMD_RESEND_PACKET)
	{  	   
		StartAddress -= LastDataLen;
		TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%PAGE_SIZE) >= (PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+PAGE_SIZE);
		goto out;

	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
		{
			TotalLen -= srclen;
		}	
		else
		{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
		}

		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
//		memset(pSrc, 0, srclen);
	
		{
			uint32_t *p = (uint32_t*)pSrc;
			for(i = 0; i<srclen/4; i++)
				p[i] = 0;
		}
		
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;

	return 0;
}

//the smallest of APROM size is 2K 
static uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
				return size;
		else 
			size *= 2;
	}while(1);
}

void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Enable Internal RC 22.1184MHz clock */
    CLK_EnableXtalRC(CLK_PWRCON_OSC22M_EN_Msk);

    /* Waiting for Internal RC clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_OSC22M_STB_Msk);

    /* Switch HCLK clock source to Internal RC and HCLK source divide 1 */
    CLK_SetHCLK(CLK_CLKSEL0_HCLK_S_HIRC, CLK_CLKDIV_HCLK(1));

    /* Enable external XTAL 12MHz clock */
    CLK_EnableXtalRC(CLK_PWRCON_XTL12M_EN_Msk);

    /* Waiting for external XTAL clock ready */
    CLK_WaitClockReady(CLK_CLKSTATUS_XTL12M_STB_Msk);

    /* Set core clock as PLL_CLOCK from PLL */
    CLK_SetCoreClock(PLL_CLOCK);

    /* Enable UART module clock */
    CLK_EnableModuleClock(UART0_MODULE);

    /* Select UART module clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART_S_HXT, CLK_CLKDIV_UART(1));

    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/

    /* Set P3 multi-function pins for UART0 RXD and TXD */
    SYS->P3_MFP &= ~(SYS_MFP_P30_Msk | SYS_MFP_P31_Msk);
    SYS->P3_MFP |= (SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0);

}

void UART0_Init()
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init UART                                                                                               */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Reset UART0 module */
    SYS_ResetModule(UART0_RST);

    /* Configure UART0 and set UART0 Baudrate */
    UART_Open(UART0, 115200);
}

/*---------------------------------------------------------------------------------------------------------*/
/* UART Test Sample                                                                                        */
/* Test Item                                                                                               */
/* It sends the received data to HyperTerminal.                                                            */
/*---------------------------------------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------------------------------------*/
/* MAIN function                                                                                           */
/*---------------------------------------------------------------------------------------------------------*/

int main(void)
{
    uint8_t volatile	bufhead_bak=0;
    
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Init System, peripheral clock and multi-function I/O */
    SYS_Init();

    CLK->AHBCLK |= CLK_AHBCLK_ISP_EN_Msk;
		FMC->ISPCON |= FMC_ISPCON_ISPEN_Msk;

		g_apromSize = GetApromSize();
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
		g_enter_ISP = 0;
		g_pdid = SYS->PDID;

    /* Init UART0 for printf and testing */
    UART0_Init();
    
    
   	UART_ENABLE_INT(UART0 , UART_IER_TIME_OUT_EN_Msk | UART_IER_RDA_IEN_Msk | UART_IER_RLS_IEN_Msk | UART_IER_RTO_IEN_Msk);
    NVIC_EnableIRQ(UART0_IRQn);
    
    WDT->WTCR = WDT_TIMEOUT_2POW6 | WDT_WTCR_WTIE_Msk | WDT_WTCR_WTE_Msk |
		WDT_WTCR_WTR_Msk | WDT_WTCR_WTIF_Msk;
		NVIC_EnableIRQ(WDT_IRQn);
		WDT->WTCR |=  WDT_WTCR_WTR_Msk;  // RESET WDT FLAG
    
    timer_init();

    while(1)
    {
			
   		if (bUartDataReady == TRUE)
   		{
   			bUartDataReady = FALSE;
    		g_timecnt = 0;
   			ParseCmd(uart_rcvbuf, PACKET_SIZE, FALSE);
   			PutString();
    	}
   		
			
			if (!g_enter_ISP && (get_timer_value() > 0x8000))// 32ms
			{
					
					outpw(&SYS->RSTSRC, 3);	//clear bit
					outpw(&FMC->ISPCON, inpw(&FMC->ISPCON) & 0xFFFFFFFC);
					outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));	//SYSRESETREQ
	
					/* Trap the CPU */
					while(1);
			}
	  }		

}
