/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "NUC1xx.h"

#include "Driver/DrvUSB.h"
#include "Driver/DrvGPIO.h"
#include "Driver/DrvCAN.h"
#include "USB/HIDDevice.h"
#include "USB/USBInfra.h"
#include "FMC.h"

#define DBG_PRINTF          printf
//#define DBG_PRINTF(...)          

#define CMD_SET_CAN_ID			0x000000BB

#define CMD_UPDATE_APROM	    0x000000A0
#define CMD_UPDATE_CONFIG	    0x000000A1
#define CMD_READ_CONFIG		    0x000000A2
#define CMD_ERASE_ALL		    0x000000A3
#define CMD_SYNC_PACKNO		    0x000000A4
#define CMD_GET_FWVER		    0x000000A6
#define CMD_RUN_APROM		    0x000000AB
#define CMD_RUN_LDROM		    0x000000AC
#define CMD_RESET			    0x000000AD
#define CMD_CONNECT			    0x000000AE

#define CMD_GET_DEVICEID	    0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2


#define PACKET_SIZE     64

#define CAN_ID			0x100

E_USBINFRA_USB_STATE eUsbState;
static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus = DISCONNECTED;
static volatile BOOL bCANDataReady = FALSE;
static volatile uint32_t CanId = 0;
__align(4) static uint8_t rcvbuf[PACKET_SIZE];
__align(4) static uint8_t sendbuf[PACKET_SIZE];

volatile BOOL bUsbDataReady, bUartDataReady;
volatile BOOL bUsbInReady;
uint32_t g_apromSize=0x20000;
volatile uint32_t g_pdid, g_timecnt;

void Delay(uint32_t delayCnt);
void my_memcpy(void *dest, void *src, int32_t size);
extern void WordsCpy(void *dest, void *src, int32_t size);

/*
* USB interrupt handler
*/
void USBD_IRQHandler(void)
{
    //printf("USB relative events\n");
	USB_ParseEvent();
}

/*----------------------------------------------------------------------------*/
/*  CAN Callback Function      												  */
/*----------------------------------------------------------------------------*/
void RxCallbackFun(uint8_t u8status)
{
	STR_CANMSG_T rMsg;
    uint32_t u32IntId;
	int32_t i;

    u32IntId = u8status-1;

	for(i=0; i<8; i++)
	{
    	DrvCAN_ReadMsgObj(MSG(u32IntId-7+i),TRUE, &rMsg);
		memcpy(&rcvbuf[i<<3], &rMsg.Data[0], 8);
	}

	bCANDataReady = TRUE;
}

/*
* init HID usb
*/
static void UsbHid(void)
{
	//USB_Init();
	HID_Open();	
	
    NVIC_SetPriority (USBD_IRQn, 2);//(1<<__NVIC_PRIO_BITS) - 2);
    NVIC_EnableIRQ(USBD_IRQn);

	/* Enable float-detection interrupt. */
	_DRVUSB_ENABLE_FLD_INT();

	/* Enable USB-related interrupts. */
	_DRVUSB_ENABLE_MISC_INT(IEF_WAKEUP | IEF_WAKEUPEN | IEF_FLD | IEF_USB | IEF_BUS);

	
}



//unit is 0.5us
void SysTimerDelay(uint32_t us)
{
#ifndef UART_ONLY
    SysTick->LOAD = us * 24; /* using 48MHz cpu clock*/
#else
	SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
#endif
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = (1 << SYSTICK_CLKSOURCE) | (1<<SYSTICK_ENABLE);//using cpu clock

    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}


static __inline void UartInit(void)
{
    /* Multi-Function Pin: Enable UART0:Tx Rx */
    //SYS->GPBMFP.UART0_RX = 1;
    //SYS->GPBMFP.UART0_TX = 1;
    outpw(&SYS->GPBMFP, 0x03);

    /* Configure GCR to reset UART0 */
    SYS->IPRSTC2.UART0_RST = 1;
    SYS->IPRSTC2.UART0_RST = 0;

    /* Enable UART clock */
    SYSCLK->APBCLK.UART0_EN = 1;

    /* Select UART clock source */
    SYSCLK->CLKSEL1.UART_S = 0;//0:12M; 2:22.1184M

    /* Data format */
    UART0->LCR.WLS = 3;
	//UART0->FCR.RFR = 1;//reset rx fifo
	//UART0->FCR.RFITL = 5;//46
	outpw(&UART0->FCR, 0x52);
	UART0->TOR = 0x40;

    /* Configure the baud rate */
#if 1    
    //if SYSCLK->CLKSEL1.UART_S = 0
    //BaudRateCalculator(SystemFrequency, 115200, &UART0->BAUD);
    *((__IO uint32_t *)&UART0->BAUD) = 0x3F000066;//115200
    //*((__IO uint32_t *)&UART0->BAUD) = 0xB;//57600
#else
    //if SYSCLK->CLKSEL1.UART_S = 2//22.1184M
    *((__IO uint32_t *)&UART0->BAUD) = 0x2F00000A;//115200
    //*((__IO uint32_t *)&UART0->BAUD) = 0x16;//57600
#endif
    
    //NVIC_SetPriority (UART0_IRQn, 2);
    //NVIC_EnableIRQ(UART0_IRQn);
    
    //UART0->IER.RDA_IEN = 1;UART0->IER.TOC_IEN = 1;UART0->IER.TOC_EN = 1;//Enable Timeout Counter
	//outpw(&UART0->IER, 0x811);
}

BOOL CAN_SendData()
{
	STR_CANMSG_T msg1;
	int32_t i;

    msg1.FrameType= DATA_FRAME;
    msg1.IdType   = CAN_STD_ID;
    msg1.DLC      = 8;
    msg1.Id       = CanId;

	for(i=0; i<8; i++)
	{
		memcpy(&msg1.Data[0], &sendbuf[i<<3], 8);
		if(DrvCAN_SetTxMsgObj(MSG(i),&msg1) < 0)     
	    {
	        printf("Set Tx Msg Object failed\n");
	        return FALSE;
	    }

	    DrvCAN_SetTxRqst(MSG(i)); 
	}

	return TRUE;
}

BOOL CAN_RcvData()
{
	while(!bCANDataReady);
	bCANDataReady = FALSE;

	return TRUE;
}

static __inline void CANInit(void)
{
	/* Select CAN Multi-Function */
	outpw(&SYS->GPDMFP, inpw(&SYS->GPDMFP) | (0x3<<6));
	
	/* CAN INIT */
	GPIOB->PMD.PMD12 = 3;
	GPIOB->DOUT &= 0xEFFF;

	DrvCAN_Init();
	if(DrvCAN_Open(250)<0)
        while(1);

    /*Set Operation and install callback function */
    DrvCAN_EnableInt(CAN_CON_IE/*|CAN_CON_SIE*/);
    
    DrvCAN_SetRxMsgObj(MSG(8),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(9),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(10),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(11),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(12),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(13),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(14),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(15),CAN_STD_ID, CAN_ID,TRUE);

    DrvCAN_InstallCallback(CALLBACK_MSG, (CAN_CALLBACK)RxCallbackFun);
}

static BOOL bIsUpdate;
static uint32_t gu32TotalLength;

int ByPassCmd(unsigned char *buffer, uint8_t len)
{
    int32_t status;
    uint32_t cmd;

    /* Host didn't get previous status back, return error */
    if(bUsbInReady == TRUE)
        return FALSE;

    status = TRUE;

    memcpy(sendbuf, buffer, PACKET_SIZE);
	status = CAN_SendData();
    if(!status) return status;
    
    cmd = inp32(buffer);
    
    DBG_PRINTF("cmd:%x %x\n", cmd, inp32(buffer+4));

    if((cmd == 0) && bIsUpdate && gu32TotalLength && 0) 
    {
        /* data phase of aprom update */
        DBG_PRINTF("pkt:%d  len:%d\n", inp32(&sendbuf[4]), gu32TotalLength);
        gu32TotalLength -= 56;

        if(gu32TotalLength <= 0)
        {
            bIsUpdate = 0;
            SysTimerDelay(100000);//50ms

            /* Get update status */
			status = CAN_RcvData();
            if(!status) return status;
            memcpy(g_HID_sDevice.au8IntInBuffer, rcvbuf, PACKET_SIZE);
        	my_memcpy((void*) g_HID_ar8UsbBuf1, rcvbuf, HID_INT_BUFFER_SIZE);
        	
        	_DRVUSB_TRIG_EP(2,HID_MAX_PACKET_SIZE_EP1);
        
            bUsbInReady = TRUE;
            
            return TRUE;
        }    
    }
    else
    {

    
        /* Delay according to different commands */
        /*if((cmd == CMD_SYNC_PACKNO) || 
           (cmd == CMD_GET_FWVER) ||
           (cmd == CMD_GET_DEVICEID) ||
           (cmd == CMD_READ_CONFIG) ||
           (cmd == CMD_UPDATE_CONFIG) ||
           (cmd == CMD_GET_FLASHMODE) )
        {
            //while(bUsbInReady == TRUE);
            SysTimerDelay(100000);        
    
        }
        else if((cmd == CMD_WRITE_CHECKSUM))
        {
            SysTimerDelay(400000);//0.2s
        }
        else */if((cmd == CMD_RUN_APROM) || 
                (cmd == CMD_RUN_LDROM) || 
                (cmd == CMD_RESET) || 
              0)
        {
            /* No ack */
            return TRUE;
        }
        else if(cmd == CMD_UPDATE_APROM)
        {
            //uint32_t j;

            bIsUpdate = 1;
            gu32TotalLength =  inp32(&sendbuf[0xc]);
            DBG_PRINTF("Write AP, size:%d\n", gu32TotalLength);
            gu32TotalLength -= 48;
    
            /*for (j = 0; j < 10; j++)
            {
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
            }*/

        }
        /*else if (cmd == CMD_ERASE_ALL)
        {
            uint32_t i;

            for (i = 0; i < 6; i++)
            {
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
            }
        }
        else
        {
                SysTimerDelay(100000);//50ms

        }*/
    
		status = CAN_RcvData();
        if(!status) while(1);
        //if(!status) return status;
        memcpy(g_HID_sDevice.au8IntInBuffer, rcvbuf, PACKET_SIZE);
    	my_memcpy((void*) g_HID_ar8UsbBuf1, rcvbuf, HID_INT_BUFFER_SIZE);
    	
    	_DRVUSB_TRIG_EP(2,HID_MAX_PACKET_SIZE_EP1);
    
        bUsbInReady = TRUE;
    
        DBG_PRINTF("Rcv: %x %x %x %x\n", inp32(rcvbuf), inp32(rcvbuf+4),inp32(rcvbuf+8), inp32(rcvbuf+0xc));
    
    }

    return status;
}

int32_t main()
{
    extern uint32_t SystemFrequency;
    volatile uint32_t pinst;
	volatile uint32_t cmd;
    uint8_t volatile	bufhead_bak;
	
    UNLOCKREG();

	SYSCLK->PWRCON.XTL12M_EN = 1;

    /* Waiting for 12M/22M Xtal stalble */
    SysTimerDelay(5000);
    SYSCLK->CLKSEL0.HCLK_S = 4;//4:22M
    FMC->ISPCON.ISPEN = 1;


    /* Enable PLL */
	SYSCLK->PLLCON.OE = 0;
	SYSCLK->PLLCON.PD = 0;
    //Delay(1000);
    SysTimerDelay(100);

	SYSCLK->CLKSEL0.HCLK_S = 2;
    SysTimerDelay(100);

	/* The PLL must be 48MHz x N times when using USB */

    bUsbDataReady = 0;
    bUartDataReady = 0;
    bUsbInReady = 0;
    bIsUpdate = 0;
    gu32TotalLength = 0;

    UartInit();

	CANInit();
   
	/* Process USB event by interrupt */    
	UsbHid();

    DBG_PRINTF(" >> ISP to CAN converter <<\n");

    while(1)
    {
    
	   	if (bUsbDataReady == TRUE)
    	{
    		memcpy((uint8_t*)&cmd, g_HID_sDevice.au8IntOutBuffer, 4);
			if(cmd == CMD_SET_CAN_ID)
			{
				memcpy((uint8_t*)&CanId, g_HID_sDevice.au8IntOutBuffer+4, 4);
			}
			else
				ByPassCmd(g_HID_sDevice.au8IntOutBuffer, HID_MAX_PACKET_SIZE_EP1);
            
            bUsbDataReady = FALSE;
			_DRVUSB_TRIG_EP(3,HID_MAX_PACKET_SIZE_EP1);
    	}
	}

}

void my_memcpy(void *dest, void *src, int32_t size)
{
	WordsCpy(dest, src, size);
}










