/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/*
	using CAN0(GPD6(CANRX0),GPD7(CANTX0))
*/
#include <stdio.h>
#include <string.h>
#include "NUC1xx.h"

#include "Driver/DrvCAN.h"
#include "Driver/DrvGPIO.h"
#include "FMC.h"

#define CMD_UPDATE_APROM	0x000000A0
#define CMD_UPDATE_CONFIG	0x000000A1
#define CMD_READ_CONFIG		0x000000A2
#define CMD_ERASE_ALL		0x000000A3
#define CMD_SYNC_PACKNO		0x000000A4
#define CMD_GET_FWVER		0x000000A6
#define CMD_RUN_APROM		0x000000AB
#define CMD_RUN_LDROM		0x000000AC
#define CMD_RESET			0x000000AD

#define CMD_GET_DEVICEID	0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA


#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define PACKET_SIZE	64

#define CAN_ID		0x102

__align(4) static uint8_t rcvbuf[PACKET_SIZE];
__align(4) static uint8_t sendbuf[PACKET_SIZE];
__align(4) static uint8_t aprom_buf[PAGE_SIZE];
//#pragma arm section rwdata = ".bss"
BOOL bCANDataReady, bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid;
static uint32_t g_ckbase = (0x20000 - 8);

static void CheckCksumBase(void);

/*---------------------------------------------------------------------------------------------------------*/
/*  CAN0 Callback Function									                                   */
/*---------------------------------------------------------------------------------------------------------*/
void RxCallbackFun(uint8_t u8status)
{
	STR_CANMSG_T rMsg;
    uint32_t u32IntId;
	int32_t i;

    u32IntId = u8status-1;

	for(i=0; i<8; i++)
	{
    	DrvCAN_ReadMsgObj(MSG(u32IntId-7+i),TRUE, &rMsg);
		memcpy(&rcvbuf[i<<3], &rMsg.Data[0], 8);
	}

	bCANDataReady = TRUE;
}

BOOL CAN_SendData()
{
	STR_CANMSG_T msg1;
	int32_t i;

    msg1.FrameType= DATA_FRAME;
    msg1.IdType   = CAN_STD_ID;
    msg1.DLC      = 8;
    msg1.Id       = 0x100;

	for(i=0; i<8; i++)
	{
		memcpy(&msg1.Data[0], &sendbuf[i<<3], 8);
		if(DrvCAN_SetTxMsgObj(MSG(i),&msg1) < 0)     
	    {
	        return FALSE;
	    }

	    DrvCAN_SetTxRqst(MSG(i)); 
	}

	return TRUE;
}

static __inline void CANInit(void)
{
	/* Select CAN Multi-Function */
	outpw(&SYS->GPDMFP, inpw(&SYS->GPDMFP) | (0x3<<6));
	
	/* CAN INIT */
	GPIOB->PMD.PMD12 = 3;
	GPIOB->DOUT &= 0xEFFF;

	DrvCAN_Init();
	if(DrvCAN_Open(250)<0)
        while(1);

    /*Set Operation and install callback function */
    DrvCAN_EnableInt(CAN_CON_IE);
    
    DrvCAN_SetRxMsgObj(MSG(8),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(9),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(10),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(11),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(12),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(13),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(14),CAN_STD_ID, CAN_ID,FALSE);
	DrvCAN_SetRxMsgObj(MSG(15),CAN_STD_ID, CAN_ID,TRUE);

	DrvCAN_InstallCallback(CALLBACK_MSG, (CAN_CALLBACK)RxCallbackFun);
}

static uint16_t Checksum (unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for (c=0, i=0; i < len; i++) {
        c += buf[i];
    }
    return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}

static int ParseCmd(unsigned char *buffer, uint8_t len, BOOL flag)
{
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, g_packno = 1;
	uint8_t *response ;
	uint16_t cksum, lcksum;
	uint32_t	lcmd, packno, srclen, cktotallen, /*ckstart, */i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	

	response = sendbuf;
	pSrc = buffer;
	srclen = len;

	lcmd = inpw(pSrc);
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	if(lcmd)
		gcmd = lcmd;

	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	if(packno != g_packno)
		goto out;

	if(lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;//version 2.3
	//else if(lcmd == CMD_READ_CONFIG)
	//{
	//	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));
	//}
	else if(lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, SYS->PDID);
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		SYS->RSTSRC.RSTS_POR=1;//clear bit
		SYS->RSTSRC.RSTS_PAD=1;//clear bit
		/* Set BS */		
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFE);//ISP disable
		}

		outpw(&FMC->ISPCON, i);
		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

		/* Trap the CPU */
		while(1);
	}

	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		
		if((regcnf0 & 0x4) && ((security == 0)||(lcmd == CMD_ERASE_ALL)))//erase APROM + data flash
		{
			EraseAP(FALSE, 0, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);//don't erase data flash
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		else
			bUpdateApromCmd = TRUE;
	}



	if(lcmd == CMD_WRITE_CHECKSUM)//write cksum to aprom last
	{
		cktotallen = inpw(pSrc);
		lcksum = inpw(pSrc+4);
		//printf("cktotallen=%x,lcksum=%x\n",cktotallen, lcksum);
		//printf("cksum\n");
		CheckCksumBase();
		ReadData(g_ckbase & 0xFFE00, g_ckbase, (uint32_t*)aprom_buf);
		outpw(aprom_buf+PAGE_SIZE - 8, cktotallen);
		outpw(aprom_buf+PAGE_SIZE - 4, lcksum);
		FMC_Erase(g_ckbase & 0xFFE00);
		WriteData(g_ckbase & 0xFFE00, g_ckbase + 8, (uint32_t*)aprom_buf);
	}
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCON)&0x2)? 2 : 1);
	}
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)//g_dataFlashAddr
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			StartAddress = 0;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;
		//printf("StartAddress=%x,TotalPadLen=%d\n",StartAddress, TotalPadLen);
		//return 0;
	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
			TotalLen -= srclen;
		else{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
	    }

		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		//memset(pSrc, 0, srclen);
		{
			uint32_t *p = (uint32_t*)pSrc;
			for(i = 0; i<srclen/4; i++)
				p[i] = 0;
		}
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	outph(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;

	return 0;
}

static void CheckCksumBase()
{
	//skip data flash
	unsigned int aprom_end = g_apromSize, data;
	int result;
	
	result = FMC_Read(0x1F000 - 8, &data);
	if(result == 0)//128K flash
    {
    	FMC_Read(Config0, &data);
    	if((data&0x01)==0)//DFEN enable
    	{
    		FMC_Read(Config1, &aprom_end);
    	}
    	g_ckbase = aprom_end - 8;
    	return;
    }
    else// less than 128K
    {
    	aprom_end = 0x10000;//64K
    	do{
		    result = FMC_Read(aprom_end - 8, &data);
		    if(result == 0)
		    	break;
    		aprom_end = aprom_end/2;
    	}while(aprom_end > 4096);

		g_ckbase = aprom_end - 8;
	}
}


//the smallest of APROM size is 2K 
static uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}


int32_t main()
{
    extern uint32_t SystemFrequency;
    uint32_t totallen, cksum;
  
    UNLOCKREG();
	
    g_pdid = SYS->PDID;
    SYSCLK->PWRCON.XTL12M_EN = 1;
	/* Waiting for 12M Xtal stable */
	while(!(SYSCLK->CLKSTATUS & 0x01));

    SYSCLK->PWRCON.OSC22M_EN = 1;
    /* Waiting for 22M Xtal stable */
	while(!(SYSCLK->CLKSTATUS & 0x10));
	
    /* ISPEN must be set before FMC_Read */
    FMC->ISPCON.ISPEN = 1;

	SYSCLK->PLLCON.OE = 0;
	SYSCLK->PLLCON.PD = 0;
	/* Waiting for PLL stable */
    while(!(SYSCLK->CLKSTATUS & 0x04));

	SYSCLK->CLKSEL0.HCLK_S = 2;
    
    g_apromSize = GetApromSize();
	GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
    
    CheckCksumBase();
    
    FMC_Read(g_ckbase, &totallen);
	FMC_Read(g_ckbase+4, &cksum);
   
    if(((GPIOB->PIN & (1 << 15)) == 0) || ((inpw(&SYS->RSTSRC)&0x3) == 0x00)
    	/*|| (totallen > g_apromSize) || (CalCheckSum(0x0, totallen) != cksum)*/)//SYSRESETREQ reset, or cksum error, run ISP
    {
    	SYSCLK->CLKSEL0.HCLK_S = 2;//22M
    	//printf("LDROM %x %x, RSTSRC:%x\n", totallen, cksum, inpw(&SYS->RSTSRC));
		/* Process USB event by interrupt */    
		CANInit();
    }
    else//change to APROM
    {
		//if((inpw(&SYS->RSTSRC)&0x3) == 0x1)
		{ 	//after power on
			SYS->RSTSRC.RSTS_POR=1;//clear bit
			SYS->RSTSRC.RSTS_PAD=1;//clear bit
	   		outpw(&FMC->ISPCON, inpw(&FMC->ISPCON) & 0xFFFFFFFC);
	   		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));//SYSRESETREQ
			//SYS->IPRST0.CPU_RST = 1;//Set CPU reset
		}

		/* Trap the CPU */
		while(1);
    }

    while(1)
    {
		while(!bCANDataReady);
		if(bCANDataReady == TRUE)
    	{
    		ParseCmd(rcvbuf, PACKET_SIZE, TRUE);    		
			CAN_SendData();
			bCANDataReady = FALSE;
    	}
    	
    }

}







































































































