/******************************************************************************
 * @file     ADC.h
 * @version  V1.0
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series ADC Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __ADC_H__
#define __ADC_H__

#include "NUC123Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/*  ADDRx constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_DATA_VALID             (1UL<<ADC_ADDR_VALID_Pos) /*!< The corresponding channel analog input conversion complete flag */
#define ADC_DATA_OVERRUN           (1UL<<ADC_ADDR_OVERRUN_Pos) /*!< Indicate data in RSLT[15:0] is overwritten or not */

/*---------------------------------------------------------------------------------------------------------*/
/*  ADCR constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_START                  (1UL<<ADC_ADCR_ADST_Pos) /*!< Start A/D conversion */
#define ADC_STOP                   (0UL<<ADC_ADCR_ADST_Pos) /*!< Stop A/D conversion */

#define ADC_PDMA_EN          (1UL<<ADC_ADCR_PTEN_Pos) /*!< Enable PDMA data transfer in ADDR 0~7 */
#define ADC_PDMA_DIS         (0UL<<ADC_ADCR_PTEN_Pos) /*!< Disable PDMA data transfer in ADDR 0~7 */

#define ADC_TRG_EN             (1UL<<ADC_ADCR_TRGEN_Pos) /*!< Enable triggering of A/D conversion by external STADC pin */
#define ADC_TRG_DIS            (0UL<<ADC_ADCR_TRGEN_Pos) /*!< Disable triggering of A/D conversion by external STADC pin */

#define ADC_LOW_TRG            (0UL<<ADC_ADCR_TRGCOND_Pos) /*!< External pin STADC trigger event is low level */
#define ADC_HIGH_TRG           (1UL<<ADC_ADCR_TRGCOND_Pos) /*!< External pin STADC trigger event is high level */
#define ADC_FALLING_TRG        (2UL<<ADC_ADCR_TRGCOND_Pos) /*!< External pin STADC trigger event is falling edge */
#define ADC_RISING_TRG         (3UL<<ADC_ADCR_TRGCOND_Pos) /*!< External pin STADC trigger event is rising edge */

#define ADC_TRGSRC_STADC             (0UL<<ADC_ADCR_TRGS_Pos) /*!< A/D conversion is started by external STADC pin */
#define ADC_TRGSRC_PWM               (1UL<<ADC_ADCR_TRGS_Pos) /*!< A/D conversion is started by PWM center-aligned trigger */

#define ADC_MD_SINGLE            (0UL<<ADC_ADCR_ADMD_Pos) /*!< A/D Converter Operation Mode is single mode */
#define ADC_MD_SINGLE_SCAN       (2UL<<ADC_ADCR_ADMD_Pos) /*!< A/D Converter Operation Mode is single scan mode */
#define ADC_MD_CONT_SCAN   (3UL<<ADC_ADCR_ADMD_Pos) /*!< A/D Converter Operation Mode is continuous scan mode */

#define ADC_INT_EN                 (1UL<<ADC_ADCR_ADIE_Pos) /*!< Enable A/D interrupt function */
#define ADC_INT_DIS                (0UL<<ADC_ADCR_ADIE_Pos) /*!< Disable A/D interrupt function */

#define ADC_ENABLE                 (1UL<<ADC_ADCR_ADEN_Pos) /*!< Enable A/D converter */
#define ADC_DISABLE                (0UL<<ADC_ADCR_ADEN_Pos) /*!< Disable A/D converter */

/*---------------------------------------------------------------------------------------------------------*/
/*  ADCHER constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_CH7_EXT_IN       (0UL<<ADC_ADCHER_PRESEL_Pos) /*!< External analog input as ADC channel 7 input source */
#define ADC_CH7_BANDGAP      (1UL<<ADC_ADCHER_PRESEL_Pos) /*!< Internal bandgap voltage as ADC channel 7 input source */

#define ADC_CH7_EN               (1UL<<ADC_ADCHER_CHEN7_Pos) /*!< Enable analog input channel 7 */
#define ADC_CH7_DIS              (0UL<<ADC_ADCHER_CHEN7_Pos) /*!< Disable analog input channel 7 */

#define ADC_CH6_EN               (1UL<<ADC_ADCHER_CHEN6_Pos) /*!< Enable analog input channel 6 */
#define ADC_CH6_DIS              (0UL<<ADC_ADCHER_CHEN6_Pos) /*!< Disable analog input channel 6 */

#define ADC_CH5_EN               (1UL<<ADC_ADCHER_CHEN5_Pos) /*!< Enable analog input channel 5 */
#define ADC_CH5_DIS              (0UL<<ADC_ADCHER_CHEN5_Pos) /*!< Disable analog input channel 5 */

#define ADC_CH4_EN               (1UL<<ADC_ADCHER_CHEN4_Pos) /*!< Enable analog input channel 4 */
#define ADC_CH4_DIS              (0UL<<ADC_ADCHER_CHEN4_Pos) /*!< Disable analog input channel 4 */

#define ADC_CH3_EN               (1UL<<ADC_ADCHER_CHEN3_Pos) /*!< Enable analog input channel 3 */
#define ADC_CH3_DIS              (0UL<<ADC_ADCHER_CHEN3_Pos) /*!< Disable analog input channel 3 */

#define ADC_CH2_EN               (1UL<<ADC_ADCHER_CHEN2_Pos) /*!< Enable analog input channel 2 */
#define ADC_CH2_DIS              (0UL<<ADC_ADCHER_CHEN2_Pos) /*!< Disable analog input channel 2 */

#define ADC_CH1_EN               (1UL<<ADC_ADCHER_CHEN1_Pos) /*!< Enable analog input channel 1 */
#define ADC_CH1_DIS              (0UL<<ADC_ADCHER_CHEN1_Pos) /*!< Disable analog input channel 1 */

#define ADC_CH0_EN               (1UL<<ADC_ADCHER_CHEN0_Pos) /*!< Enable analog input channel 0 */
#define ADC_CH0_DIS              (0UL<<ADC_ADCHER_CHEN0_Pos) /*!< Disable analog input channel 0 */

/*---------------------------------------------------------------------------------------------------------*/
/*  ADCMPRx constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_CMP_CH0          (0UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 0 conversion result is selected to be compared */
#define ADC_CMP_CH1          (1UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 1 conversion result is selected to be compared */
#define ADC_CMP_CH2          (2UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 2 conversion result is selected to be compared */
#define ADC_CMP_CH3          (3UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 3 conversion result is selected to be compared */
#define ADC_CMP_CH4          (4UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 4 conversion result is selected to be compared */
#define ADC_CMP_CH5          (5UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 5 conversion result is selected to be compared */
#define ADC_CMP_CH6          (6UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 6 conversion result is selected to be compared */
#define ADC_CMP_CH7          (7UL<<ADC_ADCMPR_CMPCH_Pos) /*!< Channel 7 conversion result is selected to be compared */

#define ADC_CMP_GE       (1UL<<ADC_ADCMPR_CMPCOND_Pos) /*!< When A/D conversion result is greater or equal to CMPD, internal match counter increase one */
#define ADC_CMP_LESS                 (0UL<<ADC_ADCMPR_CMPCOND_Pos) /*!< When A/D conversion result is less than CMPD, internal match counter increase one */

#define ADC_CMP_INT_EN               (1UL<<ADC_ADCMPR_CMPIE_Pos) /*!< Enable compare function interrupt */
#define ADC_CMP_INT_DIS              (0UL<<ADC_ADCMPR_CMPIE_Pos) /*!< Disable compare function interrupt */

#define ADC_CMP_EN           (1UL<<ADC_ADCMPR_CMPEN_Pos) /*!< Enable compare function */
#define ADC_CMP_DIS          (0UL<<ADC_ADCMPR_CMPEN_Pos) /*!< Disable compare function */

/*---------------------------------------------------------------------------------------------------------*/
/*  ADSR constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_BUSY               (1UL<<ADC_ADSR_BUSY_Pos) /*!< Indicate A/D converter is busy at conversion or is in idle state */
#define ADC_CMP1_IF          (1UL<<ADC_ADSR_CMPF1_Pos) /*!< Flag of conversion result in ADDR meets ADCMPR1 setting or not */
#define ADC_CMP0_IF          (1UL<<ADC_ADSR_CMPF0_Pos) /*!< Flag of conversion result in ADDR meets ADCMPR0 setting or not */
#define ADC_CONV_DONE_IF   (1UL<<ADC_ADSR_ADF_Pos) /*!< A/D Conversion End Flag */

/*---------------------------------------------------------------------------------------------------------*/
/*  Misc constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define ADC_CH7               (1UL<<ADC_ADCHER_CHEN7_Pos) /*!< Bit 7 of ADCHER register */
#define ADC_CH6               (1UL<<ADC_ADCHER_CHEN6_Pos) /*!< Bit 6 of ADCHER register */
#define ADC_CH5               (1UL<<ADC_ADCHER_CHEN5_Pos) /*!< Bit 5 of ADCHER register */
#define ADC_CH4               (1UL<<ADC_ADCHER_CHEN4_Pos) /*!< Bit 4 of ADCHER register */
#define ADC_CH3               (1UL<<ADC_ADCHER_CHEN3_Pos) /*!< Bit 3 of ADCHER register */
#define ADC_CH2               (1UL<<ADC_ADCHER_CHEN2_Pos) /*!< Bit 2 of ADCHER register */
#define ADC_CH1               (1UL<<ADC_ADCHER_CHEN1_Pos) /*!< Bit 1 of ADCHER register */
#define ADC_CH0               (1UL<<ADC_ADCHER_CHEN0_Pos) /*!< Bit 0 of ADCHER register */

/*---------------------------------------------------------------------------------------------------------*/
/* Macro functions                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/**
 * @brief       Check ADC conversion result status
 *
 * @param[in]   ch  ADC channel. It could be 0~7
 *
 * @retval      FALSE  Conversion result is not valid
 * @retval      TRUE   Conversion data in result register is valid
 *
 * @details     Check ADC conversion reault status, valid or not
 */
#define ADC_IS_RSLT_VALID(ch)          (ADC->ADDR[(ch)] & ADC_DATA_VALID)


/**
 * @brief       Check if ADC result register has been overwritten or not
 *
 * @param[in]   ch  ADC channel. It could be 0~7
 *
 * @retval      FALSE  Data in RSLT[15:0] is recent conversion result
 * @retval      TRUE   Data in RSLT[15:0] is overwritten 
 *
 * @details     Check if ADC result register RSLT[15:0] has been overwritten or not
 */
#define ADC_IS_RSLT_OVERRUN(ch)          (ADC->ADDR[(ch)] & ADC_DATA_OVERRUN)


/**
 * @brief       Get ADC conversion result
 *
 * @param[in]   ch  ADC channel. It could be 0~7
 *
 * @return      10-bit ADC conversion result
 *
 * @details     Get ADC conversion result from result register RSLT[15:0]
 */
#define ADC_GET_RSLT(ch)            (ADC->ADDR[(ch)] & 0x3FF)


/**
 * @brief       Start A/D conversion
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Start A/D conversion
 */
#define ADC_START_CONVERSION()                 (ADC->ADCR |= ADC_START)


/**
 * @brief       Stop A/D conversion
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Stop A/D conversion
 */
#define ADC_STOP_CONVERSION()                  (ADC->ADCR &= (~ADC_ADCR_ADST_Msk))


/**
 * @brief       Enable PDMA transfer
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable PDMA data transfer in ADDR 0~7
 */
#define ADC_PDMA_ENABLE()                (ADC->ADCR |= ADC_PDMA_EN)


/**
 * @brief       Disable PDMA transfer
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable PDMA data transfer in ADDR 0~7
 */
#define ADC_PDMA_DISABLE()               (ADC->ADCR &= (~ADC_ADCR_PTEN_Msk))


/**
 * @brief       Enable triggering of A/D conversion by external STADC pin
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable triggering of A/D conversion by external STADC pin, \n
 *              ADC external trigger function is only supported in Single-cycle Scan mode
 */
#define ADC_TRG_ENABLE()                  (ADC->ADCR |= ADC_TRG_EN)


/**
 * @brief       Disable triggering of A/D conversion by external STADC pin
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable triggering of A/D conversion by external STADC pin
 */
#define ADC_TRG_DISABLE()                 (ADC->ADCR &= (~ADC_ADCR_TRGEN_Msk))


/**
 * @brief       Set external trigger condition
 *
 * @param[in]   cond  External trigger condition. It could be \n
 *                    ADC_LOW_TRG, ADC_HIGH_TRG, ADC_FALLING_TRG, ADC_RISING_TRG 
 *
 * @return      None
 *
 * @details     Set external pin STADC trigger event is low, high level or falling, rising edge
 */
#define ADC_SET_TRGCOND(cond)            (ADC->ADCR = ADC->ADCR & (~ADC_ADCR_TRGCOND_Msk) | (cond))


/**
 * @brief       Set hardware trigger source
 *
 * @param[in]   src  Hardware trigger source. It could be \n
 *                   ADC_TRGSRC_STADC, ADC_TRGSRC_PWM
 *
 * @return      None
 *
 * @details     Set hardware trigger source
 */
#define ADC_SET_TRGSRC(src)            (ADC->ADCR = ADC->ADCR & (~ADC_ADCR_TRGS_Msk) | (src))


/**
 * @brief       Set A/D converter Operation Mode
 *
 * @param[in]   mode  A/D converter Operation Mode. It could be \n
 *                    ADC_MD_SINGLE, ADC_MD_SINGLE_SCAN, ADC_MD_CONT_SCAN
 *
 * @return      None
 *
 * @details     Set A/D converter Operation Mode
 */
#define ADC_SET_MODE(mode)           (ADC->ADCR = ADC->ADCR & (~ADC_ADCR_ADMD_Msk) | (mode))


/**
 * @brief       Enable A/D interrupt function
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable A/D interrupt function
 */
#define ADC_INT_ENABLE()                 (ADC->ADCR |= ADC_INT_EN)


/**
 * @brief       Disable A/D interrupt function
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable A/D interrupt function
 */
#define ADC_INT_DISABLE()                (ADC->ADCR &= (~ADC_ADCR_ADIE_Msk))


/**
 * @brief       Enable A/D converter
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Enable A/D converter
 */
#define ADC_CONVERTER_ENABLE()                 (ADC->ADCR |= ADC_ENABLE)


/**
 * @brief       Disable A/D converter
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Disable A/D converter
 */
#define ADC_CONVERTER_DISABLE()                (ADC->ADCR &= (~ADC_ADCR_ADEN_Msk))


/**
 * @brief       Set A/D channel 7 input source
 *
 * @param[in]   src  A/D channel 7 input source. It could be \n
 *                   ADC_CH7_EXT_IN, ADC_CH7_BANDGAP
 *
 * @return      None
 *
 * @details     Set A/D channel 7 input source
 */
#define ADC_SET_CH7_SRC(src)          (ADC->ADCHER = ADC->ADCHER & (~ADC_ADCHER_PRESEL_Msk) | (src))


/**
 * @brief       Enable A/D analog input channels
 *
 * @param[in]   ch  A/D channel. It could be one of following or their 'OR' combination \n
 *                  ADC_CH0, ADC_CH1, ADC_CH2, ADC_CH3, \n
 *                  ADC_CH4, ADC_CH5, ADC_CH6, ADC_CH7
 *
 * @return      None
 *
 * @details     Enable A/D analog input for the specified channels
 */
#define ADC_CH_ENABLE(ch)                 (ADC->ADCHER |= (ch))


/**
 * @brief       Disable A/D analog input channels
 *
 * @param[in]   ch  A/D channel. It could be one of following or their 'OR' combination \n
 *                  ADC_CH0, ADC_CH1, ADC_CH2, ADC_CH3, \n
 *                  ADC_CH4, ADC_CH5, ADC_CH6, ADC_CH7
 *
 * @return      None
 *
 * @details     Disable A/D analog input for the specified channels
 */
#define ADC_CH_DISABLE(ch)                 (ADC->ADCHER &= ~(ch))


/**
 * @brief       Set ADC compare data
 *
 * @param[in]   ch    ADC comparator channel. It could be 0~1
 * @param[in]   data  Compar2 data to be set
 *
 * @return      None
 *
 * @details     Set ADC compare data which is used to compare with conversion result of specified channel
 */
#define ADC_SET_CMP_DATA(ch, data)       (ADC->ADCMPR[(ch)] = ADC->ADCMPR[(ch)] & (~ADC_ADCMPR_CMPD_Msk) | ((data)<<ADC_ADCMPR_CMPD_Pos))


/**
 * @brief       Set ADC Compare match count
 *
 * @param[in]   ch   ADC comparator channel. It could be 0~1
 * @param[in]   cnt  Compare match count to be set
 *
 * @return      None
 *
 * @details     Set ADC Compare match count, When internal counter reaches to (CMPMATCNT +1), CMPFx will be set
 */
#define ADC_SET_CMP_CNT(ch, cnt)     (ADC->ADCMPR[(ch)] = ADC->ADCMPR[(ch)] & (~ADC_ADCMPR_CMPMATCNT_Msk) | ((cnt)<<ADC_ADCMPR_CMPMATCNT_Pos))


/**
 * @brief       Set ADC Compare channel
 *
 * @param[in]   cmp_ch  ADC comparator channel. It could be 0~1
 * @param[in]   adc_ch  ADC channel to be compared. It could be \n
 *                      ADC_CMP_CH0, ADC_CMP_CH1, ADC_CMP_CH2, ADC_CMP_CH3
 *                      ADC_CMP_CH4, ADC_CMP_CH5, ADC_CMP_CH6, ADC_CMP_CH7
 *
 * @return      None
 *
 * @details     Set ADC Compare channel of specified comparator
 */
#define ADC_SET_CMP_CH(cmp_ch, adc_ch)     (ADC->ADCMPR[(cmp_ch)] = ADC->ADCMPR[(cmp_ch)] & (~ADC_ADCMPR_CMPCH_Msk) | (adc_ch))


/**
 * @brief       Set ADC compare conditions
 *
 * @param[in]   ch    ADC comparator channel. It could be 0~1
 * @param[in]   cond  Compare conditions. It could be ADC_CMP_GE, ADC_CMP_LESS
 *
 * @return      None
 *
 * @details     When the relations between A/D conversion result and CMPD match the setting, internal match counter increase one
 */
#define ADC_SET_CMP_COND(ch, cond)         (ADC->ADCMPR[(ch)] = ADC->ADCMPR[(ch)] & (~ADC_ADCMPR_CMPCOND_Msk) | (cond))


/**
 * @brief       Enable ADC compare interrupt function
 *
 * @param[in]   ch  Comparator channel. It could be 0~1
 *
 * @return      None
 *
 * @details     Enable ADC controller compare interrupt function
 */
#define ADC_CMP_INT_ENABLE(ch)                 (ADC->ADCMPR[(ch)] |= ADC_CMP_INT_EN)


/**
 * @brief       Disable ADC compare interrupt function
 *
 * @param[in]   ch  Comparator channel. It could be 0~1
 *
 * @return      None
 *
 * @details     Disable ADC controller compare interrupt function
 */
#define ADC_CMP_INT_DISABLE(ch)                (ADC->ADCMPR[(ch)] &= (~ADC_ADCMPR_CMPIE_Msk))


/**
 * @brief       Enable ADC compare function
 *
 * @param[in]   ch  Comparator channel. It could be 0~1
 *
 * @return      None
 *
 * @details     Enable ADC controller to compare CMPD[11:0] with specified channel conversion result
 */
#define ADC_CMP_ENABLE(ch)                 (ADC->ADCMPR[(ch)] |= ADC_CMP_EN)


/**
 * @brief       Disable ADC compare function
 *
 * @param[in]   ch  Comparator channel. It could be 0~1
 *
 * @return      None
 *
 * @details     Disable ADC controller compare function
 */
#define ADC_CMP_DISABLE(ch)                (ADC->ADCMPR[(ch)] &= (~ADC_ADCMPR_CMPEN_Msk))


/**
 * @brief       Get ADC current conversion channel
 *
 * @param       None
 *
 * @return      When BUSY = 1, current conversion channel. When BUSY = 0, number of the next conversion channel. 
 *
 * @details     Get ADC current or next conversion channel, depending on BUSY bit
 */
#define ADC_GET_CURRENT_CH()        ((ADC->ADSR & ADC_ADSR_CHANNEL_Msk)>>ADC_ADSR_CHANNEL_Pos)


/**
 * @brief       Get ADC converter status
 *
 * @param       None
 *
 * @retval      FALSE  A/D converter is in Idle state
 * @retval      TRUE   A/D converter is busy at conversion
 *
 * @details     Get ADC converter status, busy or idle
 */
#define ADC_GET_CONVERTER_STATUS()             (ADC->ADSR & ADC_BUSY)


/**
 * @brief       Get ADC compare1 interrupt status
 *
 * @param       None
 *
 * @retval      FALSE  Conversion result in ADDR does not meet ADCMPR1 setting
 * @retval      TRUE   Conversion result in ADDR meets ADCMPR1 setting
 *
 * @details     Get ADC compare1 interrupt status
 */
#define ADC_GET_CMP1_IF()          (ADC->ADSR & ADC_CMP1_IF)


/**
 * @brief       Clear ADC compare1 interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear ADC compare1 interrupt status
 */
#define ADC_CLR_CMP1_IF()          (ADC->ADSR = ADC_CMP1_IF)


/**
 * @brief       Get ADC compare0 interrupt status
 *
 * @param       None
 *
 * @retval      FALSE  Conversion result in ADDR does not meet ADCMPR0 setting
 * @retval      TRUE   Conversion result in ADDR meets ADCMPR0 setting
 *
 * @details     Get ADC compare0 interrupt status
 */
#define ADC_GET_CMP0_IF()          (ADC->ADSR & ADC_CMP0_IF)


/**
 * @brief       Clear ADC compare0 interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear ADC compare0 interrupt status
 */
#define ADC_CLR_CMP0_IF()          (ADC->ADSR = ADC_CMP0_IF)


/**
 * @brief       Get A/D conversion end interrupt status
 *
 * @param       None
 *
 * @retval      FALSE  Conversion result in ADDR does not meet ADCMPR1 setting
 * @retval      TRUE   When A/D conversion ends on specified channel in single mode or on all specified channels in scan mode
 *
 * @details     Get A/D conversion end interrupt status
 */
#define ADC_GET_CONV_END_IF()        (ADC->ADSR & ADC_CONV_DONE_IF)


/**
 * @brief       Clear A/D conversion end interrupt status
 *
 * @param       None
 *
 * @return      None
 *
 * @details     Clear A/D conversion end interrupt status
 */
#define ADC_CLR_CONV_END_IF()        (ADC->ADSR = ADC_CONV_DONE_IF)


/**
 * @brief       Get ADC PDMA current transfer data
 *
 * @param       None
 *
 * @retval      ADC PDMA current transfer data
 *
 * @details     Get ADC PDMA current transfer data
 */
#define ADC_GET_PDMA_CURRENT_DATA()      (ADC->ADPDMA)

#endif
