/**************************************************************************//**
 * @file     I2S.h
 * @version  V2.0
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series I2S Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __I2S_H__
#define __I2S_H__

/*---------------------------------------------------------------------------------------------------------*/
/* Include related headers                                                                                 */
/*---------------------------------------------------------------------------------------------------------*/
#include "NUC123Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/* I2S constant definitions                                                                                */
/*---------------------------------------------------------------------------------------------------------*/

#define I2S_CFG_FORMAT_PCMA         (I2S_CON_PCM_Msk|0)                     /*!< I2S Data Format: PCM mode A */
#define I2S_CFG_FORMAT_PCMB         (I2S_CON_PCM_Msk|I2S_CON_FORMAT_Msk)    /*!< I2S Data Format: PCM mode B */
#define I2S_CFG_FORMAT_I2S          0                                       /*!< I2S Data Format: I2S */

#define I2S_CFG_CHANNEL_MONO_LEFT   (I2S_CON_MONO_Msk|I2S_CON_RXLCH_Msk)    /*!< I2S Channel: Mono Left channel */
#define I2S_CFG_CHANNEL_MONO_RIGHT  (I2S_CON_MONO_Msk|I2S_CON_RXRCH_Msk)    /*!< I2S Channel: Mono Right channel */
#define I2S_CFG_CHANNEL_STEREO      0                                       /*!< I2S Channel: Stereo */

#define I2S_CFG_MASTER              0                                       /*!< I2S Master Mode */
#define I2S_CFG_SLAVE               I2S_CON_SLAVE_Msk                       /*!< I2S Slave Mode */
#define I2S_CFG_MCK_EN              I2S_CON_MCLKEN_Msk                      /*!< I2S MCK Enable */
#define I2S_CFG_MCK_DIS             0                                       /*!< I2S MCK Disable */

#define I2S_CFG_RX_THRES_1          (0ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 1-word */
#define I2S_CFG_RX_THRES_2          (1ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 2-word */
#define I2S_CFG_RX_THRES_3          (2ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 3-word */
#define I2S_CFG_RX_THRES_4          (3ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 4-word */
#define I2S_CFG_RX_THRES_5          (4ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 5-word */
#define I2S_CFG_RX_THRES_6          (5ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 6-word */
#define I2S_CFG_RX_THRES_7          (6ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 7-word */
#define I2S_CFG_RX_THRES_8          (7ul<<I2S_CON_RXTH_Pos)                 /*!< I2S Rx Buffer Threshold 8-word */

#define I2S_CFG_TX_THRES_0          (0ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 1-word */
#define I2S_CFG_TX_THRES_1          (1ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 2-word */
#define I2S_CFG_TX_THRES_2          (2ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 3-word */
#define I2S_CFG_TX_THRES_3          (3ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 4-word */
#define I2S_CFG_TX_THRES_4          (4ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 5-word */
#define I2S_CFG_TX_THRES_5          (5ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 6-word */
#define I2S_CFG_TX_THRES_6          (6ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 7-word */
#define I2S_CFG_TX_THRES_7          (7ul<<I2S_CON_TXTH_Pos)                 /*!< I2S Tx Buffer Threshold 8-word */

#define I2S_CFG_WORDWIDTH_8         (0ul<<I2S_CON_WORDWIDTH_Pos)            /*!< I2S Data Size 8-bit */
#define I2S_CFG_WORDWIDTH_16        (1ul<<I2S_CON_WORDWIDTH_Pos)            /*!< I2S Data Size 16-bit */
#define I2S_CFG_WORDWIDTH_24        (2ul<<I2S_CON_WORDWIDTH_Pos)            /*!< I2S Data Size 24-bit */
#define I2S_CFG_WORDWIDTH_32        (3ul<<I2S_CON_WORDWIDTH_Pos)            /*!< I2S Data Size 32-bit */

#define I2S_CFG_TXDMA               I2S_CON_TXDMA_Msk                       /*!< I2S Tx DMA enable */
#define I2S_CFG_RXDMA               I2S_CON_RXDMA_Msk                       /*!< I2S Rx DMA enable */

#define I2S_CFG_MUTE                I2S_CON_MUTE_Msk                        /*!< I2S Mute Enable */
#define I2S_CFG_RXEN                I2S_CON_RXEN_Msk                        /*!< I2S Rx Enable */
#define I2S_CFG_TXEN                I2S_CON_TXEN_Msk                        /*!< I2S Tx Enable */
#define I2S_CFG_EN                  I2S_CON_I2SEN_Msk                       /*!< I2S Enable */

#define I2S_INT_LZC                 I2S_IE_LZCIE_Msk                        /*!< I2S Interrupt Left Channel Zero Cross */
#define I2S_INT_RZC                 I2S_IE_RZCIE_Msk                        /*!< I2S Interrupt Right Channel Zero Cross */
#define I2S_INT_TXTH                I2S_IE_TXTHIE_Msk                       /*!< I2S Interrupt Tx under threshold */
#define I2S_INT_TXOVF               I2S_IE_TXOVFIE_Msk                      /*!< I2S Interrupt Tx Overflow */
#define I2S_INT_TXUDF               I2S_IE_TXUDFIE_Msk                      /*!< I2S Interrupt Tx underflow */
#define I2S_INT_RXTH                I2S_IE_RXTHIE_Pos                       /*!< I2S Interrupt Rx over threshold */
#define I2S_INT_RXOVF               I2S_IE_RXOVFIE_Pos                      /*!< I2S Interrupt Rx Overflow */
#define I2S_INT_RXUDF               I2S_IE_RXUDFIE_Pos                      /*!< I2S Interrupt Rx underflow */

#define I2S_INTF_LZC                I2S_STATUS_LZCF_Msk                     /*!< I2S Interrupt flag Left Channel Zero Cross */
#define I2S_INTF_RZC                I2S_STATUS_RZCF_Msk                     /*!< I2S Interrupt flag Right Channel Zero Cross */
#define I2S_INTF_TXOVF              I2S_STATUS_TXOVF_Msk                    /*!< I2S Interrupt flag Tx Overflow */
#define I2S_INTF_TXUDF              I2S_STATUS_TXUDF_Msk                    /*!< I2S Interrupt flag Tx Underflow */
#define I2S_INTF_RXOVF              I2S_STATUS_RXOVF_Msk                    /*!< I2S Interrupt flag Rx Overflow */
#define I2S_INTF_RXUDF              I2S_STATUS_RXUDF_Msk                    /*!< I2S Interrupt flag Rx Underflow */
#define I2S_INTF_TXINT              I2S_STATUS_I2STXINT_Msk                 /*!< I2S Interrupt flag Tx Interrupt */
#define I2S_INTF_RXINT              I2S_STATUS_I2SRXINT_Msk                 /*!< I2S Interrupt flag Rx Interrupt */

#define I2S_STAT_TX_BUSY            I2S_STATUS_TXBUSY_Msk                   /*!< I2S State Tx Busy */
#define I2S_STAT_TX_EMPTY           I2S_STATUS_TXEMPTY_Msk                  /*!< I2S State Tx FIFO Empty */
#define I2S_STAT_TX_FULL            I2S_STATUS_TXFULL_Msk                   /*!< I2S State Tx FIFO Full */
#define I2S_STAT_TX_THRESHOLD       I2S_STATUS_TXTHF_Msk                    /*!< I2S State Tx FIFO Under Threshold */
#define I2S_STAT_RX_EMPTY           I2S_STATUS_RXEMPTY_Msk                  /*!< I2S State Rx FIFO Empty */
#define I2S_STAT_RX_FULL            I2S_STATUS_RXFULL_Msk                   /*!< I2S State Rx FIFO Full */
#define I2S_STAT_RX_THRESHOLD       I2S_STATUS_RXTHF_Msk                    /*!< I2S State Rx FIFO Over Threshold */
#define I2S_STAT_CHANNEL_MASK       I2S_STATUS_RIGHT_Msk                    /*!< I2S State Current Channel Mask */
#define I2S_STAT_CHANNEL_RIGHT      I2S_STATUS_RIGHT_Msk                    /*!< I2S State Current Channel is Right Channel */
#define I2S_STAT_CHANNEL_LEFT       0                                       /*!< I2S State Current Channel is Left Channel */

/*---------------------------------------------------------------------------------------------------------*/
/*  Define Macros and functions                                                                            */
/*---------------------------------------------------------------------------------------------------------*/

/**
 * @brief      Clear Rx FIFO
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function clears I2S Rx FIFO.
 *
 */
#define I2S_CLR_RXFIFO()            I2S->CON|=I2S_CON_CLR_RXFIFO_Msk

/**
 * @brief      Clear Tx FIFO
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function clears I2S Tx FIFO.
 *
 */
#define I2S_CLR_TXFIFO()            I2S->CON|=I2S_CON_CLR_TXFIFO_Msk

/**
 * @brief      Assign I2S configuration
 *
 * @param[in]  Config      configuration value, should be ored by:
 *                          - I2S_CFG_MASTER/I2S_CFG_SLAVE
 *                          - I2S_CFG_MCK_EN/I2S_CFG_MCK_DIS
 *                          - I2S_CFG_CHANNEL_MONO_LEFT/I2S_CFG_CHANNEL_MONO_RIGHT/I2S_CFG_CHANNEL_STEREO
 *                          - I2S_CFG_FORMAT_PCMA/I2S_CFG_FORMAT_PCMB/I2S_CFG_FORMAT_I2S
 *                          - I2S_CFG_WORDWIDTH_8/I2S_CFG_WORDWIDTH_16/I2S_CFG_WORDWIDTH_24/I2S_CFG_WORDWIDTH_32
 *                          - I2S_CFG_MUTE
 *                          - I2S_CFG_RXEN
 *                          - I2S_CFG_RXDMA
 *                          - I2S_CFG_TXEN
 *                          - I2S_CFG_TXDMA
 *                          - I2S_CFG_EN
 *                          - I2S_CFG_TX_THRES_X
 *                          - I2S_CFG_RX_THRES_X
 *
 * @return     None
 *
 * @details    This function Assign I2S configuration.
 *
 */
#define I2S_ASSIGN_CONFIG(c)        I2S->CON=(c)

/**
 * @brief      Enable I2S Rx DMA
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function enables I2S Rx DMA.
 *
 */
#define I2S_ENABLE_RXDMA()          I2S->CON|=I2S_CFG_RXDMA

/**
 * @brief      Disable I2S Rx DMA
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function disables I2S Rx DMA.
 *
 */
#define I2S_DISABLE_RXDMA()         I2S->CON&=~I2S_CFG_RXDMA

/**
 * @brief      Enable I2S Tx DMA
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function enables I2S Tx DMA.
 *
 */
#define I2S_ENABLE_TXDMA()          I2S->CON|=I2S_CFG_TXDMA

/**
 * @brief      Disable I2S Tx DMA
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    This function disables I2S Tx DMA.
 *
 */
#define I2S_DISABLE_TXDMA()         I2S->CON&=~I2S_CFG_TXDMA

/**
 * @brief      Configure I2S Data Format
 *
 * @param[in]  Format      configuration value, should be ored by:
 *                          - I2S_CFG_WORDWIDTH_8/I2S_CFG_WORDWIDTH_16/I2S_CFG_WORDWIDTH_24/I2S_CFG_WORDWIDTH_32
 *
 * @return     None
 *
 * @details    Configure I2S data format to be 8, 16, 24, 32 bits.
 *
 */
#define I2S_SET_DATAFORMAT(f)       I2S->CON=(I2S->CON&~I2S_CON_WORDWIDTH_Msk)|(f)

/**
 * @brief      Configure I2S Channel
 *
 * @param[in]  Channel     configuration value, should be ored by:
 *                          - I2S_CFG_CHANNEL_MONO_LEFT/I2S_CFG_CHANNEL_MONO_RIGHT/I2S_CFG_CHANNEL_STEREO
 *
 * @return     None
 *
 * @details    Configure I2S channel to be mono-left, mono-right or stereo.
 *
 */
#define I2S_SET_CHANNEL(c)          I2S->CON=(I2S->CON&~(I2S_CON_MONO_Msk|I2S_CON_RXRCH_Msk|I2S_CON_RXLCH_Msk))|(c)

/**
 * @brief      Enable I2S MCK
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Enable I2S MCK clock output.
 *
 */
#define I2S_ENABLE_MCK()            I2S->CON|=I2S_CON_MCLKEN_Msk

/**
 * @brief      Disable I2S MCK
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Disable I2S MCK clock output.
 *
 */
#define I2S_DISABLE_MCK()           I2S->CON&=~I2S_CON_MCLKEN_Msk

/**
 * @brief      Enable I2S Transact
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Enable I2S to send data
 *
 */
#define I2S_ENABLE_TX()             I2S->CON|=I2S_CON_TXEN_Msk

/**
 * @brief      Disable I2S Transact
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Disable I2S from sending data
 *
 */
#define I2S_DISABLE_TX()            I2S->CON&=~I2S_CON_TXEN_Msk

/**
 * @brief      Enable I2S Receive
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Enable I2S to receive data
 *
 */
#define I2S_ENABLE_RX()             I2S->CON|=I2S_CON_RXEN_Msk

/**
 * @brief      Disable I2S Recivie
 *
 * @param[in]  None
 *
 * @return     None
 *
 * @details    Disable I2S from receiving data
 *
 */
#define I2S_DISABLE_RX()            I2S->CON&=~I2S_CON_RXEN_Msk

/**
 * @brief      Set I2S MCK Frequency
 *
 * @param[in]  Frequency  MCK Frequency
 *
 * @return     None
 *
 * @details    Set I2S MCK Frequency.
 *
 */
#define I2S_SET_MCK_DIV(d)           I2S->CLKDIV = I2S->CLKDIV&~I2S_CLKDIV_MCLK_DIV_Msk|((d)<<I2S_CLKDIV_MCLK_DIV_Pos)

/**
 * @brief      Set I2S Bit Clock Frequency
 *
 * @param[in]  Frequency  Bit Clock Frequency
 *
 * @return     None
 *
 * @details    Set I2S Bit Clock Frequency.
 *
 */
#define I2S_SET_BITCLK_DIV(d)        I2S->CLKDIV = I2S->CLKDIV&~I2S_CLKDIV_BCLK_DIV_Msk|((d)<<I2S_CLKDIV_BCLK_DIV_Pos)

/**
 * @brief      Enable I2S Interrupt
 *
 * @param[in]  Interrupt  I2S Interrupt, should be ored by:
 *                         - I2S_INT_LZC
 *                         - I2S_INT_RZC
 *                         - I2S_INT_TXTH
 *                         - I2S_INT_TXOVF
 *                         - I2S_INT_TXUDF
 *                         - I2S_INT_RXTH
 *                         - I2S_INT_RXOVF
 *                         - I2S_INT_RXUDF
 *
 * @return     None
 *
 * @details    Enable I2S interrupt according to the mask value.
 *
 */
#define I2S_ENABLE_INT(i)           I2S->IE|=(i)

/**
 * @brief      Disable I2S Interrupt
 *
 * @param[in]  Interrupt  I2S Interrupt, should be ored by:
 *                         - I2S_INT_LZC
 *                         - I2S_INT_RZC
 *                         - I2S_INT_TXTH
 *                         - I2S_INT_TXOVF
 *                         - I2S_INT_TXUDF
 *                         - I2S_INT_RXTH
 *                         - I2S_INT_RXOVF
 *                         - I2S_INT_RXUDF
 *
 * @return     None
 *
 * @details    Disable I2S interrupt according to the mask value.
 *
 */
#define I2S_DISABLE_INT(i)          I2S->IE&=~(i)

/**
 * @brief      Get I2S Interrupt
 *
 * @param[in]  None
 *
 * @return     Interrupt  I2S Interrupt, should be ored by:
 *                         - I2S_INT_LZC
 *                         - I2S_INT_RZC
 *                         - I2S_INT_TXTH
 *                         - I2S_INT_TXOVF
 *                         - I2S_INT_TXUDF
 *                         - I2S_INT_RXTH
 *                         - I2S_INT_RXOVF
 *                         - I2S_INT_RXUDF
 *
 * @details    Get I2S interrupt.
 *
 */
#define I2S_GET_INT()               I2S->IE

/**
 * @brief      Clear I2S Interrupt Flag
 *
 * @param[in]  None
 *
 * @return     Flag       I2S Interrupt flag, should be ored by:
 *                         - I2S_INTF_LZC
 *                         - I2S_INTF_RZC
 *                         - I2S_INTF_TXOVF
 *                         - I2S_INTF_TXUDF
 *                         - I2S_INTF_RXOVF
 *                         - I2S_INTF_RXUDF
 *                         - I2S_INTF_TXINT
 *                         - I2S_INTF_RXINT
 *
 * @details    Clear I2S Interrupt Flag.
 *
 */
#define I2S_GET_INTF()              I2S->STATUS

/**
 * @brief      Clear I2S Interrupt Flag
 *
 * @param[in]  Flag       I2S Interrupt flag, should be ored by:
 *                         - I2S_INTF_LZC
 *                         - I2S_INTF_RZC
 *                         - I2S_INTF_TXOVF
 *                         - I2S_INTF_TXUDF
 *                         - I2S_INTF_RXOVF
 *                         - I2S_INTF_RXUDF
 *
 * @return     None
 *
 * @details    Clear I2S Interrupt Flag.
 *
 */
#define I2S_CLR_INTF(i)             I2S->STATUS|=(i)

/**
 * @brief      Get I2S State
 *
 * @param[in]  None
 *
 * @return     State      I2S State, ored by:
 *                         - I2S_STAT_TX_BUSY
 *                         - I2S_STAT_TX_EMPTY
 *                         - I2S_STAT_TX_FULL
 *                         - I2S_STAT_TX_UNDER_THRES
 *                         - I2S_STAT_RX_EMPTY
 *                         - I2S_STAT_RX_FULL
 *                         - I2S_STAT_RX_OVER_THRES
 *
 * @details    Get I2S State.
 *
 */
#define I2S_GET_STATUS()            I2S->STATUS

/**
 * @brief      Get I2S Channel for current data
 *
 * @param[in]  None
 *
 * @retval     I2S_STAT_CHANNEL_RIGHT        Current data is right channel
 * @retval     I2S_STAT_CHANNEL_LEFT         Current data is left channel
 *
 * @details    Get I2S Channel for current data.
 *
 */
#define I2S_GET_CUR_CHANNEL()       (I2S->STATUS&I2S_STAT_CHANNEL_MASK)

/**
 * @brief      Write Data to I2S
 *
 * @param[in]  Data       data to be wiritten to I2S
 *
 * @return     None
 *
 * @details    Write Data to I2S.
 *
 */
#define I2S_SET_DATA(d)             I2S->TXFIFO=(d)

/**
 * @brief      Read Data from I2S
 *
 * @param[in]  None
 *
 * @return     Data       data read from I2S
 *
 * @details    Read Data from I2S.
 *
 */
#define I2S_GET_DATA()              I2S->RXFIFO


#endif

