/******************************************************************************
 * @file     PDMA.h
 * @version  V1.0
 * $Revision: 1 $
 * $Date: 14/09/09 4:43p $
 * @brief    NUC123 Series PDMA Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __PDMA_H__
#define __PDMA_H__

#include "NUC123Series.h"

/*---------------------------------------------------------------------------------------------------------*/
/*  PDMA_CSRx constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_CSR_TRIG_EN                (1UL<<PDMA_CSR_TRIG_EN_Pos) /*!< PDMA data read or write transfer Enabled */

#define PDMA_CSR_APB_TWS_32BIT          (0UL<<PDMA_CSR_APB_TWS_Msk) /*!< One word (32-bit) is transferred for each PDMA operation */
#define PDMA_CSR_APB_TWS_8BIT           (1UL<<PDMA_CSR_APB_TWS_Msk) /*!< One byte (8-bit) is transferred for each PDMA operation */
#define PDMA_CSR_APB_TWS_16BIT          (2UL<<PDMA_CSR_APB_TWS_Msk) /*!< One half word (16-bit) is transferred for each PDMA operation */

#define PDMA_CSR_DAD_INC                (0UL<<PDMA_CSR_DAD_SEL_Msk) /*!< Transfer destination address is increasing successively */
#define PDMA_CSR_DAD_FIXED              (2UL<<PDMA_CSR_DAD_SEL_Msk) /*!< Transfer destination address is fixed */

#define PDMA_CSR_SAD_INC                (0UL<<PDMA_CSR_SAD_SEL_Msk) /*!< Transfer source address is increasing successively */
#define PDMA_CSR_SAD_FIXED              (2UL<<PDMA_CSR_SAD_SEL_Msk) /*!< Transfer source address is fixed */

#define PDMA_CSR_MODE_MEM2MEM           (0UL<<PDMA_CSR_MODE_SEL_Msk) /*!< Memory to Memory mode */
#define PDMA_CSR_MODE_APB2MEM           (1UL<<PDMA_CSR_MODE_SEL_Msk) /*!< Peripheral to Memory mode */
#define PDMA_CSR_MODE_MEM2APB           (2UL<<PDMA_CSR_MODE_SEL_Msk) /*!< Memory to Peripheral mode */

#define PDMA_CSR_SW_RST                 (1UL<<PDMA_CSR_SW_RST_Msk) /*!< Reset PDMA internal state machine, pointers and internal buffer. Contents of control register will not be cleared */

#define PDMA_CSR_PDMACEN                (1UL<<PDMA_CSR_PDMACEN_Msk) /*!< Enables PDMAs operation */

/*---------------------------------------------------------------------------------------------------------*/
/*  PDMA_IERx constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_IER_BLKD_INT_EN            (1UL<<PDMA_ISR_BLKD_IE_Pos) /*!< Enable PDMA transfer done interrupt */
#define PDMA_IER_BLKD_INT_DIS           (0UL<<PDMA_ISR_BLKD_IE_Pos) /*!< Disable PDMA transfer done interrupt */

#define PDMA_IER_TABORT_INT_EN          (1UL<<PDMA_IER_TABORT_IE_Pos) /*!< Enable PDMA transfer target abort interrupt */
#define PDMA_IER_TABORT_INT_DIS         (0UL<<PDMA_IER_TABORT_IE_Pos) /*!< Disable PDMA transfer target abort interrupt */

/*---------------------------------------------------------------------------------------------------------*/
/*  PDMA_ISRx constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_ISR_BLKD_IF                (1UL<<PDMA_ISR_BLKD_IF_Pos) /*!< PDMA transfer done interrupt flag */
#define PDMA_ISR_TABORT_IF              (1UL<<PDMA_ISR_TABORT_IF_Pos) /*!< PDMA transfer target abort interrupt flag */

/*---------------------------------------------------------------------------------------------------------*/
/*  DMA_GCRCSR constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_GCRCSR_CRC_CLK_EN          (1UL<<PDMA_GCRCSR_CRC_CLK_EN_Pos) /*!< Enable CRC controller clock */
#define PDMA_GCRCSR_CRC_CLK_DIS         (0UL<<PDMA_GCRCSR_CRC_CLK_EN_Pos) /*!< Disable CRC controller clock */

#define PDMA_GCRCSR_CH5_CLK_EN          (1UL<<PDMA_GCRCSR_CLK5_EN_Pos) /*!< Enable PDMA controller channel 5 clock */
#define PDMA_GCRCSR_CH5_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK5_EN_Pos) /*!< Disable PDMA controller channel 5 clock */

#define PDMA_GCRCSR_CH4_CLK_EN          (1UL<<PDMA_GCRCSR_CLK4_EN_Pos) /*!< Enable PDMA controller channel 4 clock */
#define PDMA_GCRCSR_CH4_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK4_EN_Pos) /*!< Disable PDMA controller channel 4 clock */

#define PDMA_GCRCSR_CH3_CLK_EN          (1UL<<PDMA_GCRCSR_CLK3_EN_Pos) /*!< Enable PDMA controller channel 3 clock */
#define PDMA_GCRCSR_CH3_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK3_EN_Pos) /*!< Disable PDMA controller channel 3 clock */

#define PDMA_GCRCSR_CH2_CLK_EN          (1UL<<PDMA_GCRCSR_CLK2_EN_Pos) /*!< Enable PDMA controller channel 2 clock */
#define PDMA_GCRCSR_CH2_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK2_EN_Pos) /*!< Disable PDMA controller channel 2 clock */

#define PDMA_GCRCSR_CH1_CLK_EN          (1UL<<PDMA_GCRCSR_CLK1_EN_Pos) /*!< Enable PDMA controller channel 1 clock */
#define PDMA_GCRCSR_CH1_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK1_EN_Pos) /*!< Disable PDMA controller channel 1 clock */

#define PDMA_GCRCSR_CH0_CLK_EN          (1UL<<PDMA_GCRCSR_CLK0_EN_Pos) /*!< Enable PDMA controller channel 0 clock */
#define PDMA_GCRCSR_CH0_CLK_DIS         (0UL<<PDMA_GCRCSR_CLK0_EN_Pos) /*!< Disable PDMA controller channel 0 clock */

/*---------------------------------------------------------------------------------------------------------*/
/*  DMA_PDSSR0 constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_PDSSR0_SPI2_TX_CH0         (0UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI2_TX_CH1         (1UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI2_TX_CH2         (2UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI2_TX_CH3         (3UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI2_TX_CH4         (4UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI2_TX_CH5         (5UL<<PDMA_PDSSR0_SPI2_TXSEL_Pos) /*!< Connect SPI2_TX to PDMA channel 5 */

#define PDMA_PDSSR0_SPI2_RX_CH0         (0UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI2_RX_CH1         (1UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI2_RX_CH2         (2UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI2_RX_CH3         (3UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI2_RX_CH4         (4UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI2_RX_CH5         (5UL<<PDMA_PDSSR0_SPI2_RXSEL_Pos) /*!< Connect SPI2_RX to PDMA channel 5 */

#define PDMA_PDSSR0_SPI1_TX_CH0         (0UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI1_TX_CH1         (1UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI1_TX_CH2         (2UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI1_TX_CH3         (3UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI1_TX_CH4         (4UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI1_TX_CH5         (5UL<<PDMA_PDSSR0_SPI1_TXSEL_Pos) /*!< Connect SPI1_TX to PDMA channel 5 */

#define PDMA_PDSSR0_SPI1_RX_CH0         (0UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI1_RX_CH1         (1UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI1_RX_CH2         (2UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI1_RX_CH3         (3UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI1_RX_CH4         (4UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI1_RX_CH5         (5UL<<PDMA_PDSSR0_SPI1_RXSEL_Pos) /*!< Connect SPI1_RX to PDMA channel 5 */

#define PDMA_PDSSR0_SPI0_TX_CH0         (0UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI0_TX_CH1         (1UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI0_TX_CH2         (2UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI0_TX_CH3         (3UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI0_TX_CH4         (4UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI0_TX_CH5         (5UL<<PDMA_PDSSR0_SPI0_TXSEL_Pos) /*!< Connect SPI0_TX to PDMA channel 5 */

#define PDMA_PDSSR0_SPI0_RX_CH0         (0UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 0 */
#define PDMA_PDSSR0_SPI0_RX_CH1         (1UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 1 */
#define PDMA_PDSSR0_SPI0_RX_CH2         (2UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 2 */
#define PDMA_PDSSR0_SPI0_RX_CH3         (3UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 3 */
#define PDMA_PDSSR0_SPI0_RX_CH4         (4UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 4 */
#define PDMA_PDSSR0_SPI0_RX_CH5         (5UL<<PDMA_PDSSR0_SPI0_RXSEL_Pos) /*!< Connect SPI0_RX to PDMA channel 5 */

/*---------------------------------------------------------------------------------------------------------*/
/*  DMA_PDSSR1 constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_PDSSR1_ADC_RX_CH0          (0UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 0 */
#define PDMA_PDSSR1_ADC_RX_CH1          (1UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 1 */
#define PDMA_PDSSR1_ADC_RX_CH2          (2UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 2 */
#define PDMA_PDSSR1_ADC_RX_CH3          (3UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 3 */
#define PDMA_PDSSR1_ADC_RX_CH4          (4UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 4 */
#define PDMA_PDSSR1_ADC_RX_CH5          (5UL<<PDMA_PDSSR1_ADC_RXSEL_Pos) /*!< Connect ADC_RX to PDMA channel 5 */

#define PDMA_PDSSR1_USB_TX_CH0          (0UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 0 */
#define PDMA_PDSSR1_USB_TX_CH1          (1UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 1 */
#define PDMA_PDSSR1_USB_TX_CH2          (2UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 2 */
#define PDMA_PDSSR1_USB_TX_CH3          (3UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 3 */
#define PDMA_PDSSR1_USB_TX_CH4          (4UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 4 */
#define PDMA_PDSSR1_USB_TX_CH5          (5UL<<PDMA_PDSSR1_USB_TXSEL_Pos) /*!< Connect USB_TX to PDMA channel 5 */

#define PDMA_PDSSR1_USB_RX_CH0          (0UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 0 */
#define PDMA_PDSSR1_USB_RX_CH1          (1UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 1 */
#define PDMA_PDSSR1_USB_RX_CH2          (2UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 2 */
#define PDMA_PDSSR1_USB_RX_CH3          (3UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 3 */
#define PDMA_PDSSR1_USB_RX_CH4          (4UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 4 */
#define PDMA_PDSSR1_USB_RX_CH5          (5UL<<PDMA_PDSSR1_USB_RXSEL_Pos) /*!< Connect USB_RX to PDMA channel 5 */

#define PDMA_PDSSR1_UART1_TX_CH0        (0UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 0 */
#define PDMA_PDSSR1_UART1_TX_CH1        (1UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 1 */
#define PDMA_PDSSR1_UART1_TX_CH2        (2UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 2 */
#define PDMA_PDSSR1_UART1_TX_CH3        (3UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 3 */
#define PDMA_PDSSR1_UART1_TX_CH4        (4UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 4 */
#define PDMA_PDSSR1_UART1_TX_CH5        (5UL<<PDMA_PDSSR1_UART1_TXSEL_Pos) /*!< Connect UART1_TX to PDMA channel 5 */

#define PDMA_PDSSR1_UART1_RX_CH0        (0UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 0 */
#define PDMA_PDSSR1_UART1_RX_CH1        (1UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 1 */
#define PDMA_PDSSR1_UART1_RX_CH2        (2UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 2 */
#define PDMA_PDSSR1_UART1_RX_CH3        (3UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 3 */
#define PDMA_PDSSR1_UART1_RX_CH4        (4UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 4 */
#define PDMA_PDSSR1_UART1_RX_CH5        (5UL<<PDMA_PDSSR1_UART1_RXSEL_Pos) /*!< Connect UART1_RX to PDMA channel 5 */

#define PDMA_PDSSR1_UART0_TX_CH0        (0UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 0 */
#define PDMA_PDSSR1_UART0_TX_CH1        (1UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 1 */
#define PDMA_PDSSR1_UART0_TX_CH2        (2UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 2 */
#define PDMA_PDSSR1_UART0_TX_CH3        (3UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 3 */
#define PDMA_PDSSR1_UART0_TX_CH4        (4UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 4 */
#define PDMA_PDSSR1_UART0_TX_CH5        (5UL<<PDMA_PDSSR1_UART0_TXSEL_Pos) /*!< Connect UART0_TX to PDMA channel 5 */

#define PDMA_PDSSR1_UART0_RX_CH0        (0UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 0 */
#define PDMA_PDSSR1_UART0_RX_CH1        (1UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 1 */
#define PDMA_PDSSR1_UART0_RX_CH2        (2UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 2 */
#define PDMA_PDSSR1_UART0_RX_CH3        (3UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 3 */
#define PDMA_PDSSR1_UART0_RX_CH4        (4UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 4 */
#define PDMA_PDSSR1_UART0_RX_CH5        (5UL<<PDMA_PDSSR1_UART0_RXSEL_Pos) /*!< Connect UART0_RX to PDMA channel 5 */

/*---------------------------------------------------------------------------------------------------------*/
/*  DMA_GCRISR constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_GCRISR_INTR_STATUS         (1UL<<PDMA_GCRISR_INTR_Pos) /*!< Interrupt status of PDMA controller */
#define PDMA_GCRISR_INTRCRC_STATUS      (1UL<<PDMA_GCRISR_INTRCRC_Pos) /*!< Interrupt status of CRC controller */
#define PDMA_GCRISR_INTR5_STATUS        (1UL<<PDMA_GCRISR_INTR5_Pos) /*!< Interrupt status of PDMA channel 5 */
#define PDMA_GCRISR_INTR4_STATUS        (1UL<<PDMA_GCRISR_INTR4_Pos) /*!< Interrupt status of PDMA channel 4 */
#define PDMA_GCRISR_INTR3_STATUS        (1UL<<PDMA_GCRISR_INTR3_Pos) /*!< Interrupt status of PDMA channel 3 */
#define PDMA_GCRISR_INTR2_STATUS        (1UL<<PDMA_GCRISR_INTR2_Pos) /*!< Interrupt status of PDMA channel 2 */
#define PDMA_GCRISR_INTR1_STATUS        (1UL<<PDMA_GCRISR_INTR1_Pos) /*!< Interrupt status of PDMA channel 1 */
#define PDMA_GCRISR_INTR0_STATUS        (1UL<<PDMA_GCRISR_INTR0_Pos) /*!< Interrupt status of PDMA channel 0 */

/*---------------------------------------------------------------------------------------------------------*/
/*  DMA_PDSSR2 constant definitions.                                                                        */
/*---------------------------------------------------------------------------------------------------------*/
#define PDMA_PDSSR2_PWM3_RX_CH0         (0UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 0 */
#define PDMA_PDSSR2_PWM3_RX_CH1         (1UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 1 */
#define PDMA_PDSSR2_PWM3_RX_CH2         (2UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 2 */
#define PDMA_PDSSR2_PWM3_RX_CH3         (3UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 3 */
#define PDMA_PDSSR2_PWM3_RX_CH4         (4UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 4 */
#define PDMA_PDSSR2_PWM3_RX_CH5         (5UL<<PDMA_PDSSR2_PWM3_RXSEL_Pos) /*!< Connect PWM3_RX to PDMA channel 5 */

#define PDMA_PDSSR2_PWM2_RX_CH0         (0UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 0 */
#define PDMA_PDSSR2_PWM2_RX_CH1         (1UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 1 */
#define PDMA_PDSSR2_PWM2_RX_CH2         (2UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 2 */
#define PDMA_PDSSR2_PWM2_RX_CH3         (3UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 3 */
#define PDMA_PDSSR2_PWM2_RX_CH4         (4UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 4 */
#define PDMA_PDSSR2_PWM2_RX_CH5         (5UL<<PDMA_PDSSR2_PWM2_RXSEL_Pos) /*!< Connect PWM2_RX to PDMA channel 5 */

#define PDMA_PDSSR2_PWM1_RX_CH0         (0UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 0 */
#define PDMA_PDSSR2_PWM1_RX_CH1         (1UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 1 */
#define PDMA_PDSSR2_PWM1_RX_CH2         (2UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 2 */
#define PDMA_PDSSR2_PWM1_RX_CH3         (3UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 3 */
#define PDMA_PDSSR2_PWM1_RX_CH4         (4UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 4 */
#define PDMA_PDSSR2_PWM1_RX_CH5         (5UL<<PDMA_PDSSR2_PWM1_RXSEL_Pos) /*!< Connect PWM1_RX to PDMA channel 5 */

#define PDMA_PDSSR2_PWM0_RX_CH0         (0UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 0 */
#define PDMA_PDSSR2_PWM0_RX_CH1         (1UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 1 */
#define PDMA_PDSSR2_PWM0_RX_CH2         (2UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 2 */
#define PDMA_PDSSR2_PWM0_RX_CH3         (3UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 3 */
#define PDMA_PDSSR2_PWM0_RX_CH4         (4UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 4 */
#define PDMA_PDSSR2_PWM0_RX_CH5         (5UL<<PDMA_PDSSR2_PWM0_RXSEL_Pos) /*!< Connect PWM0_RX to PDMA channel 5 */

#define PDMA_PDSSR2_I2S_TX_CH0          (0UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 0 */
#define PDMA_PDSSR2_I2S_TX_CH1          (1UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 1 */
#define PDMA_PDSSR2_I2S_TX_CH2          (2UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 2 */
#define PDMA_PDSSR2_I2S_TX_CH3          (3UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 3 */
#define PDMA_PDSSR2_I2S_TX_CH4          (4UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 4 */
#define PDMA_PDSSR2_I2S_TX_CH5          (5UL<<PDMA_PDSSR2_I2S_TXSEL_Pos) /*!< Connect I2S_TX to PDMA channel 5 */

#define PDMA_PDSSR2_I2S_RX_CH0          (0UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 0 */
#define PDMA_PDSSR2_I2S_RX_CH1          (1UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 1 */
#define PDMA_PDSSR2_I2S_RX_CH2          (2UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 2 */
#define PDMA_PDSSR2_I2S_RX_CH3          (3UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 3 */
#define PDMA_PDSSR2_I2S_RX_CH4          (4UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 4 */
#define PDMA_PDSSR2_I2S_RX_CH5          (5UL<<PDMA_PDSSR2_I2S_RXSEL_Pos) /*!< Connect I2S_RX to PDMA channel 5 */

/*---------------------------------------------------------------------------------------------------------*/
/* Macro functions                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
/**
 * @brief       Enable PDMA transfer
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Enable PDMA read or write transfer of specified channel
 */
#define PDMA_TRIG_ENABLE(CH)                   ((CH)->CSR |= PDMA_CSR_TRIG_EN)


/**
 * @brief       Set PDMA transfer width
 *
 * @param[in]   CH     PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   width  PDMA transfer width. It could be PDMA_CSR_APB_TWS_32BIT, PDMA_CSR_APB_TWS_8BIT, PDMA_CSR_APB_TWS_16BIT
 *
 * @return      None
 *
 * @details     Set PDMA transfer width of specified channel
 */
#define PDMA_SET_TRANS_WIDTH(CH, width)        ((CH)->CSR = (CH)->CSR & (~PDMA_CSR_APB_TWS_Msk) | (width))


/**
 * @brief       Set PDMA destination address direction
 *
 * @param[in]   CH   PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   dir  PDMA address direction. It could be PDMA_CSR_DAD_INC, PDMA_CSR_DAD_FIXED
 *
 * @return      None
 *
 * @details     Set PDMA destination address direction of specified channel
 */
#define PDMA_SET_DEST_DIRECTION(CH, dir)       ((CH)->CSR = (CH)->CSR & (~PDMA_CSR_DAD_SEL_Msk) | (dir))


/**
 * @brief       Set PDMA source address direction
 *
 * @param[in]   CH   PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   dir  PDMA address direction. It could be PDMA_CSR_SAD_INC, PDMA_CSR_SAD_FIXED
 *
 * @return      None
 *
 * @details     Set PDMA source address direction of specified channel
 */
#define PDMA_SET_SOURCE_DIRECTION(CH, dir)     ((CH)->CSR = (CH)->CSR & (~PDMA_CSR_SAD_SEL_Msk) | (dir))


/**
 * @brief       Set PDMA transfer mode
 *
 * @param[in]   CH     PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   mode  PDMA transfer mode. It could be PDMA_CSR_MODE_MEM2MEM, PDMA_CSR_MODE_APB2MEM, PDMA_CSR_MODE_MEM2APB
 *
 * @return      None
 *
 * @details     Set PDMA transfer mode of specified channel
 */
#define PDMA_SET_TRANS_MODE(CH, mode)          ((CH)->CSR = (CH)->CSR & (~PDMA_CSR_MODE_SEL_Msk) | (mode))


/**
 * @brief       PDMA software engine reset
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Reset the internal state machine, pointers and internal buffer. \n
 *              The contents of control register will not be cleared. \n
 *              This bit will be automatically cleared after few clock cycles
 */
#define PDMA_SOFTWARE_RESET(CH)                ((CH)->CSR |= PDMA_CSR_SW_RST)


/**
 * @brief       Enable PDMA channel
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Enable specified PDMA channel
 */
#define PDMA_CHANNEL_ENABLE(CH)                ((CH)->CSR |= PDMA_CSR_PDMACEN)


/**
 * @brief       Set PDMA transfer source address
 *
 * @param[in]   CH    PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   addr  PDMA transfer 32-bit address, must word aligned
 *
 * @return      None
 *
 * @details     Set PDMA transfer source address of specified channel
 */
#define PDMA_SET_SOURCE_ADDR(CH, addr)         ((CH)->SAR = ((addr) & 0xFFFFFFFCUL))


/**
 * @brief       Set PDMA transfer destination address
 *
 * @param[in]   CH    PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   addr  PDMA transfer 32-bit address, must word aligned
 *
 * @return      None
 *
 * @details     Set PDMA transfer destination address of specified channel
 */
#define PDMA_SET_DEST_ADDR(CH, addr)           ((CH)->DAR = ((addr) & 0xFFFFFFFCUL))


/**
 * @brief       Set PDMA transfer byte count
 *
 * @param[in]   CH   PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 * @param[in]   cnt  PDMA transfer 16-bit byte count
 *
 * @return      None
 *
 * @details     Set PDMA transfer byte count of specified channel
 */
#define PDMA_SET_TRANS_COUNT(CH, cnt)          ((CH)->BCR = (cnt))


/**
 * @brief       Get PDMA internal buffer pointer
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      PDMA internal buffer pointer
 *
 * @details     Get PDMA internal buffer pointer of specified channel
 */
#define PDMA_GET_INTERNAL_BUF_POINTER(CH)      ((CH)->POINT)


/**
 * @brief       Get PDMA current source address
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      Source address where the PDMA transfer just occurs
 *
 * @details     Get PDMA current source address of specified channel
 */
#define PDMA_GET_CURRENT_SOURCE_ADDR(CH)       ((CH)->CSAR)


/**
 * @brief       Get PDMA current destination address
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      Destination address where the PDMA transfer just occurs
 *
 * @details     Get PDMA current destination address of specified channel
 */
#define PDMA_GET_CURRENT_DEST_ADDR(CH)         ((CH)->CDAR)


/**
 * @brief       Get PDMA current transfer count
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      The current remained byte count of PDMA
 *
 * @details     Get PDMA current transfer count of specified channel
 */
#define PDMA_GET_CURRENT_TRANS_COUNT(CH)       ((CH)->CBCR)


/**
 * @brief       Enable PDMA transfer done interrupt
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Enable PDMA transfer done interrupt of specified channel
 */
#define PDMA_TRANS_DONE_INT_ENABLE(CH)         ((CH)->IER |= PDMA_IER_BLKD_INT_EN)


/**
 * @brief       Disable PDMA transfer done interrupt
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Disable PDMA transfer done interrupt of specified channel
 */
#define PDMA_TRANS_DONE_INT_DISABLE(CH)        ((CH)->IER = (CH)->IER & (~PDMA_IER_BLKD_IE_Msk) | PDMA_IER_BLKD_INT_DIS)


/**
 * @brief       Get PDMA transfer done interrupt status
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @retval      0   PDMA has not finished all transfer on the specified channel
 * @retval      1   PDMA has finished all transfer on the specified channel
 *
 * @details     Get PDMA transfer done interrupt status of specified channel
 */
#define PDMA_GET_TRANS_DONE_INT_STATUS(CH)     (((CH)->ISR & PDMA_ISR_BLKD_IF)?1:0)


/**
 * @brief       Clear PDMA transfer done interrupt status
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Clear PDMA transfer done interrupt status of specified channel
 */
#define PDMA_CLR_TRANS_DONE_INT_STATUS(CH)     ((CH)->ISR = PDMA_ISR_BLKD_IF)


/**
 * @brief       Enable PDMA target abort interrupt
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Enable PDMA target abort interrupt of specified channel
 */
#define PDMA_TARGET_ABORT_INT_ENABLE(CH)       ((CH)->IER |= PDMA_IER_TABORT_INT_EN)


/**
 * @brief       Disable PDMA target abort interrupt
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Disable PDMA target abort interrupt of specified channel
 */
#define PDMA_TARGET_ABORT_INT_DISABLE(CH)      ((CH)->IER = (CH)->IER & (~PDMA_IER_TABORT_IE_Msk) | PDMA_IER_TABORT_INT_DIS)


/**
 * @brief       Get PDMA target abort interrupt status
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @retval      0   No bus ERROR response received
 * @retval      1   Bus ERROR response received
 *
 * @details     Get PDMA target abort interrupt status of specified channel
 */
#define PDMA_GET_TARGET_ABORT_INT_STATUS(CH)   (((CH)->ISR & PDMA_ISR_TABORT_IF)?1:0)


/**
 * @brief       Clear PDMA target abort interrupt status
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      None
 *
 * @details     Clear PDMA target abort interrupt status of specified channel
 */
#define PDMA_CLR_TARGET_ABORT_INT_STATUS(CH)   ((CH)->ISR = PDMA_ISR_TABORT_IF)


/**
 * @brief       Get PDMA shared buffer FIFO 0
 *
 * @param[in]   CH  PDMA channel struct pointer. It could be PDMA0, PDMA1, PDMA2, PDMA3, PDMA4, PDMA5
 *
 * @return      Internal buffer of specified PDMA channel
 *
 * @details     Get PDMA shared buffer FIFO 0 of specified channel
 */
#define PDMA_GET_SHARED_BUF_FIFO_0(CH)         ((CH)->SBUF)


/**
 * @brief       Enable PDMA controller clock
 *
 * @param[in]   ch  PDMA channel. It could be 0~5
 *
 * @return      None
 *
 * @details     Enable PDMA controller clock of specified channel
 */
#define PDMA_CLOCK_ENABLE(ch)                  (PDMA_GCR->GCRCSR |= (1 << ((ch)+8)))


/**
 * @brief       Disable PDMA controller clock
 *
 * @param[in]   ch  PDMA channel. It could be 0~5
 *
 * @return      None
 *
 * @details     Disable PDMA controller clock of specified channel
 */
#define PDMA_CLOCK_DISABLE(ch)                 (PDMA_GCR->GCRCSR &= (~(1 << ((ch)+8))))


/**
 * @brief       Get PDMA controller interrupt status
 *
 * @param       None
 *
 * @retval      0   No interrupt at PDMA controller
 * @retval      1   PDMA controller generated an interrupt
 *
 * @details     Get PDMA controller interrupt status
 */
#define PDMA_GET_INTR_STATUS()                 ((PDMA_GCR->GCRISR & PDMA_GCRISR_INTR_STATUS)?1:0)


/**
 * @brief       Get PDMA channel interrupt status
 *
 * @param[in]   ch  PDMA channel. It could be 0~5
 *
 * @retval      0   No interrupt at specified channel of PDMA controller
 * @retval      1   PDMA controller generated an interrupt on the specified channel
 *
 * @details     Get PDMA interrupt status of specified channel
 */
#define PDMA_GET_CH_INTR_STATUS(ch)            ((PDMA_GCR->GCRISR & (1<<(ch)))?1:0)


/**
 * @brief       Specify PDMA channel for SPI2_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI2_TX_CH0, PDMA_PDSSR0_SPI2_TX_CH1, PDMA_PDSSR0_SPI2_TX_CH2, \n
 *                       PDMA_PDSSR0_SPI2_TX_CH3, PDMA_PDSSR0_SPI2_TX_CH4, PDMA_PDSSR0_SPI2_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI2_TX
 */
#define PDMA_SPI2_TX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI2_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for SPI2_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI2_RX_CH0, PDMA_PDSSR0_SPI2_RX_CH1, PDMA_PDSSR0_SPI2_RX_CH2, \n
 *                       PDMA_PDSSR0_SPI2_RX_CH3, PDMA_PDSSR0_SPI2_RX_CH4, PDMA_PDSSR0_SPI2_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI2_RX
 */
#define PDMA_SPI2_RX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI2_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for SPI1_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI1_TX_CH0, PDMA_PDSSR0_SPI1_TX_CH1, PDMA_PDSSR0_SPI1_TX_CH2, \n
 *                       PDMA_PDSSR0_SPI1_TX_CH3, PDMA_PDSSR0_SPI1_TX_CH4, PDMA_PDSSR0_SPI1_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI1_TX
 */
#define PDMA_SPI1_TX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI1_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for SPI1_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI1_RX_CH0, PDMA_PDSSR0_SPI1_RX_CH1, PDMA_PDSSR0_SPI1_RX_CH2, \n
 *                       PDMA_PDSSR0_SPI1_RX_CH3, PDMA_PDSSR0_SPI1_RX_CH4, PDMA_PDSSR0_SPI1_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI1_RX
 */
#define PDMA_SPI1_RX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI1_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for SPI0_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI0_TX_CH0, PDMA_PDSSR0_SPI0_TX_CH1, PDMA_PDSSR0_SPI0_TX_CH2, \n
 *                       PDMA_PDSSR0_SPI0_TX_CH3, PDMA_PDSSR0_SPI0_TX_CH4, PDMA_PDSSR0_SPI0_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI0_TX
 */
#define PDMA_SPI0_TX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI0_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for SPI0_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR0_SPI0_RX_CH0, PDMA_PDSSR0_SPI0_RX_CH1, PDMA_PDSSR0_SPI0_RX_CH2, \n
 *                       PDMA_PDSSR0_SPI0_RX_CH3, PDMA_PDSSR0_SPI0_RX_CH4, PDMA_PDSSR0_SPI0_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for SPI0_RX
 */
#define PDMA_SPI0_RX_CH(channel)               (PDMA_GCR->PDSSR0 = PDMA_GCR->PDSSR0 & (~PDMA_PDSSR0_SPI0_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for ADC_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_ADC_RX_CH0, PDMA_PDSSR1_ADC_RX_CH1, PDMA_PDSSR1_ADC_RX_CH2, \n
 *                       PDMA_PDSSR1_ADC_RX_CH3, PDMA_PDSSR1_ADC_RX_CH4, PDMA_PDSSR1_ADC_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for ADC_RX
 */
#define PDMA_ADC_RX_CH(channel)                (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_ADC_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for USB_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_USB_TX_CH0, PDMA_PDSSR1_USB_TX_CH1, PDMA_PDSSR1_USB_TX_CH2, \n
 *                       PDMA_PDSSR1_USB_TX_CH3, PDMA_PDSSR1_USB_TX_CH4, PDMA_PDSSR1_USB_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for USB_TX
 */
#define PDMA_USB_TX_CH(channel)                (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_USB_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for USB_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_USB_RX_CH0, PDMA_PDSSR1_USB_RX_CH1, PDMA_PDSSR1_USB_RX_CH2, \n
 *                       PDMA_PDSSR1_USB_RX_CH3, PDMA_PDSSR1_USB_RX_CH4, PDMA_PDSSR1_USB_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for USB_RX
 */
#define PDMA_USB_RX_CH(channel)                (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_USB_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for UART1_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_UART1_TX_CH0, PDMA_PDSSR1_UART1_TX_CH1, PDMA_PDSSR1_UART1_TX_CH2, \n
 *                       PDMA_PDSSR1_UART1_TX_CH3, PDMA_PDSSR1_UART1_TX_CH4, PDMA_PDSSR1_UART1_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for UART1_TX
 */
#define PDMA_UART1_TX_CH(channel)              (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_UART1_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for UART1_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_UART1_RX_CH0, PDMA_PDSSR1_UART1_RX_CH1, PDMA_PDSSR1_UART1_RX_CH2, \n
 *                       PDMA_PDSSR1_UART1_RX_CH3, PDMA_PDSSR1_UART1_RX_CH4, PDMA_PDSSR1_UART1_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for UART1_RX
 */
#define PDMA_UART1_RX_CH(channel)              (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_UART1_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for UART0_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_UART0_TX_CH0, PDMA_PDSSR1_UART0_TX_CH1, PDMA_PDSSR1_UART0_TX_CH2, \n
 *                       PDMA_PDSSR1_UART0_TX_CH3, PDMA_PDSSR1_UART0_TX_CH4, PDMA_PDSSR1_UART0_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for UART0_TX
 */
#define PDMA_UART0_TX_CH(channel)              (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_UART0_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for UART0_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR1_UART0_RX_CH0, PDMA_PDSSR1_UART0_RX_CH1, PDMA_PDSSR1_UART0_RX_CH2, \n
 *                       PDMA_PDSSR1_UART0_RX_CH3, PDMA_PDSSR1_UART0_RX_CH4, PDMA_PDSSR1_UART0_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for UART0_RX
 */
#define PDMA_UART0_RX_CH(channel)              (PDMA_GCR->PDSSR1 = PDMA_GCR->PDSSR1 & (~PDMA_PDSSR1_UART0_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for PWM3_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_PMW3_RX_CH0, PDMA_PDSSR2_PMW3_RX_CH1, PDMA_PDSSR2_PMW3_RX_CH2, \n
 *                       PDMA_PDSSR2_PMW3_RX_CH3, PDMA_PDSSR2_PMW3_RX_CH4, PDMA_PDSSR2_PMW3_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for PWM3_RX
 */
#define PDMA_PWM3_RX_CH(channel)               (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_PMW3_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for PWM2_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_PMW2_RX_CH0, PDMA_PDSSR2_PMW2_RX_CH1, PDMA_PDSSR2_PMW2_RX_CH2, \n
 *                       PDMA_PDSSR2_PMW2_RX_CH3, PDMA_PDSSR2_PMW2_RX_CH4, PDMA_PDSSR2_PMW2_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for PWM2_RX
 */
#define PDMA_PWM2_RX_CH(channel)               (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_PMW2_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for PWM1_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_PMW1_RX_CH0, PDMA_PDSSR2_PMW1_RX_CH1, PDMA_PDSSR2_PMW1_RX_CH2, \n
 *                       PDMA_PDSSR2_PMW1_RX_CH3, PDMA_PDSSR2_PMW1_RX_CH4, PDMA_PDSSR2_PMW1_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for PWM1_RX
 */
#define PDMA_PWM1_RX_CH(channel)               (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_PMW1_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for PWM0_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_PMW0_RX_CH0, PDMA_PDSSR2_PMW0_RX_CH1, PDMA_PDSSR2_PMW0_RX_CH2, \n
 *                       PDMA_PDSSR2_PMW0_RX_CH3, PDMA_PDSSR2_PMW0_RX_CH4, PDMA_PDSSR2_PMW0_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for PWM0_RX
 */
#define PDMA_PWM0_RX_CH(channel)               (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_PMW0_RXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for I2S_TX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_I2S_TX_CH0, PDMA_PDSSR2_I2S_TX_CH1, PDMA_PDSSR2_I2S_TX_CH2, \n
 *                       PDMA_PDSSR2_I2S_TX_CH3, PDMA_PDSSR2_I2S_TX_CH4, PDMA_PDSSR2_I2S_TX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for I2S_TX
 */
#define PDMA_I2S_TX_CH(channel)                (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_I2S_TXSEL_Msk) | (channel))


/**
 * @brief       Specify PDMA channel for I2S_RX
 *
 * @param[in]   channel  PDMA channel corresponding to the specified IP. It could be \n
 *                       PDMA_PDSSR2_I2S_RX_CH0, PDMA_PDSSR2_I2S_RX_CH1, PDMA_PDSSR2_I2S_RX_CH2, \n
 *                       PDMA_PDSSR2_I2S_RX_CH3, PDMA_PDSSR2_I2S_RX_CH4, PDMA_PDSSR2_I2S_RX_CH5
 *
 * @return      None
 *
 * @details     Specify PDMA channel for I2S_RX
 */
#define PDMA_I2S_RX_CH(channel)                (PDMA_GCR->PDSSR2 = PDMA_GCR->PDSSR2 & (~PDMA_PDSSR2_I2S_RXSEL_Msk) | (channel))

#endif
