/******************************************************************************
 * @file     main.c
 * @version  V1.00
 * $Revision: 1 $
 * $Date: 14/11/27 11:11a $ 
 * @brief    FMC erase/program/read sample program for NUC400 series MCU
 *
 * @note
 * Copyright (C) 2013 Nuvoton Technology Corp. All rights reserved.
*****************************************************************************/   
#include <stdio.h>
#include <string.h>
#include "NUC472_442.h"
#include "fmc.h"
#include "isp.h"
#include "uart.h"
#include "FMC_USER.h"

#define USE_USB					0
#define SUPPORT_WRITECKSUM		0
#define UART_CLK_SRC_XTL12M 	0			// external 12 MHz
//#define UART_CLK_SRC_HIRC		3			// OSC 22M


static uint8_t volatile		bufhead;
static uint8_t volatile		g_connStatus;
__align(4) uint8_t 	response_buff[64];
__align(4) static uint8_t 	aprom_buf[PAGE_SIZE];

int  bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t 	g_pdid, g_timecnt;

#if(SUPPORT_WRITECKSUM)
static uint32_t g_ckbase = 0xFFFFFFFF;
static void CheckCksumBase(void);
#endif

static volatile int  g_enter_ISP;
static uint32_t gAppBase;


void  fsDumpBufferHex(uint8_t *pucBuff, int nSize)
{
	int  	nIdx, i;
	
	nIdx = 0;
	while (nSize > 0)
	{
		printf("0x%04X  ", nIdx);
		for (i = 0; i < 16; i++)
			printf("%02x ", pucBuff[nIdx + i]);
		printf("  ");
		for (i = 0; i < 16; i++)
		{
			if ((pucBuff[nIdx + i] >= 0x20) && (pucBuff[nIdx + i] < 127))
				printf("%c", pucBuff[nIdx + i]);
			else
				printf(".");
			nSize--;
		}
		nIdx += 16;
		printf("\n");
	}
	printf("\n");
}

void  dump_buffer(uint8_t *pucBuff, int nSize)
{
	int  	nIdx, i;
	
	nIdx = 0;
	while (nSize > 0)
	{
		printf("0x%04X  ", nIdx);
		for (i = 0; i < 16; i++)
			printf("%02x ", pucBuff[nIdx + i]);
		printf("  ");
		for (i = 0; i < 16; i++)
		{
			if ((pucBuff[nIdx + i] >= 0x20) && (pucBuff[nIdx + i] < 127))
				printf("%c", pucBuff[nIdx + i]);
			else
				printf(".");
			nSize--;
		}
		nIdx += 16;
		printf("\n");
	}
	printf("\n");
}

static uint16_t Checksum (unsigned char *buf, int len)
{   
    int i;
    uint16_t c;
    
    for (c=0, i=0; i < len; i++) 
	{
        c += buf[i];
    }
    return (c);
}   
    
static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{   
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}   

__asm spChange(uint32_t _sp)
{
	MSR MSP, r0
	BX lr
}

uint32_t Global_cmd;    
/*static*/ int ParseCmd(unsigned char *buffer, uint8_t len, int bUSB)
{   
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t lcmd, packno, srclen, security , regcnf0;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	

	response = &response_buff[0];

	pSrc = buffer;
	srclen = len;
    	lcmd = inpw(pSrc);
	Global_cmd = lcmd;
	packno = inpw(pSrc + 4);
	outpw(response + 4, 0);
	
	pSrc += 8;
	srclen -= 8;


	ReadData(Config0, Config0 + 8, (uint32_t*)(response + 8));//read config
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

   
	if (lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}

	if ((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;

//	if (lcmd >= 0xA0)
//   	printf("[0x%x] %x, %d, 0x%x, 0x%x\n", lcmd, g_packno, srclen, config[0], config[1]); 
	
	if ((lcmd) && (lcmd != CMD_RESEND_PACKET))
		gcmd = lcmd;
    
	if (lcmd == CMD_GET_FWVER)
	{
		response[8] = FW_VERSION;//version 2.6
	}
	//else if(lcmd == CMD_READ_CONFIG)
	//{
	//	FMC_ReadConfig(config);
	//}
	else if (lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, SYS->PDID);
		//goto out;
	}
	else if ((lcmd == CMD_RUN_APROM) || (lcmd == CMD_DISCONNECT))
	{
		printf("Reset to run APROM...\n");
    	FMC_SetBootSelect(E_FMC_APROM);
    	SYS->IPRST0 = SYS_IPRST0_CPURST_Msk;
    	while (1);  // trap CPU
	}
	else if (lcmd == CMD_RESET)
	{
		printf("RESET request...\n");
    	SYS->IPRST0 = SYS_IPRST0_CPURST_Msk;
    	while (1);  // trap CPU
    }
	else if (lcmd == CMD_CONNECT)
	{
		//g_connStatus = CONNECTED;
		g_packno = 1;
		g_enter_ISP = 1;
		goto out;
	}
	else if ((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{		
		if(lcmd == CMD_ERASE_ALL)  //erase APROM + data flash
		{
			EraseAP(FALSE, 0, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);//don't erase data flash
		
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		
		bUpdateApromCmd = TRUE;		

	}
    
#if(SUPPORT_WRITECKSUM)
	else if(lcmd == CMD_WRITE_CHECKSUM)//write cksum to aprom last
	{
		cktotallen = inpw(pSrc);
		lcksum = inpw(pSrc+4);
		CheckCksumBase();
		ReadData(g_ckbase & 0xFF800, g_ckbase, (uint32_t*)aprom_buf);
		outpw(aprom_buf+PAGE_SIZE - 8, cktotallen);
		outpw(aprom_buf+PAGE_SIZE - 4, lcksum);
		FMC_Erase(g_ckbase & 0xFF800);
		WriteData(g_ckbase & 0xFF800, g_ckbase + 8, (uint32_t*)aprom_buf);
	}
#endif
	else if (lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (FMC->ISPCTL & 0x2) ? 2 : 1);
	}
	
	if ((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if (lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if (g_dataFlashSize)	//g_dataFlashAddr
			//	EraseAP(g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else 
				goto out;
		}
		else
		{
			//StartAddress = 0;
			StartAddress = gAppBase;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;
	}
	else if (lcmd == CMD_UPDATE_CONFIG)
	{
		if (security && (!bUpdateApromCmd))	//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
		
		goto out;
	}
	else if (lcmd == CMD_RESEND_PACKET)	//for APROM & Data Flash only
	{  	   
		StartAddress -= LastDataLen;
		TotalLen += LastDataLen;
		if ((StartAddress & 0xFF800) >= FMC_CONFIG_BASE)
			goto out;
		ReadData(StartAddress & 0xFF800, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFF800);
		WriteData(StartAddress & 0xFF800, StartAddress, (uint32_t*)aprom_buf);
		if ((StartAddress % PAGE_SIZE) >= (PAGE_SIZE - LastDataLen))
	    	FMC_Erase((StartAddress & 0xFF800) + PAGE_SIZE);
		goto out;
    
	}
	
	if ((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if (TotalLen >= srclen)
			TotalLen -= srclen;
		else
		{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
	    }
    	
		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);

		// Read verify. If data is not matched, checksum error will be detected.
		memset(pSrc, 0, srclen);
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);

		StartAddress += srclen;
		LastDataLen =  srclen;
		
		if((Global_cmd == CMD_UPDATE_APROM))
			cksum = Checksum(buffer , len);
		
		if (TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;
	//TotalLen -= srclen;
    
	return 0;
}   
    
//unit is 1 us
void SysTimerDelay(uint32_t us)
{   
	SysTick->LOAD = us * 84; /* using 84MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock
    
    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}   
    
#if(SUPPORT_WRITECKSUM)
static void CheckCksumBase()
{   
	//skip data flash
	unsigned int aprom_end = g_apromSize, data;
	int result;
	
   	FMC_Read(Config0, &data);
   	if ((data & 0x01) == 0) //DFEN enable
   	{
   		FMC_Read(Config1, &aprom_end);
   	}
   	g_ckbase = aprom_end - 8;
   	return;
}   
#endif
    
    
//the smallest of APROM size is 128K 
//static __inline uint32_t GetApromSize()
uint32_t GetApromSize()
{   
	uint32_t size = (128*1024), data;
	int result;
	
	do 
	{
		result = FMC_Read_ISP(size, &data);
		if (result < 0)
	    {
    		return size;
		}
		else 
			size += (128*1024);
	} while(1);
}   
  
    

