/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * $Date: 14/12/03 5:46p $
 * @brief    Simulate an USB mouse and draws circle on the screen
 *
 * @note
 * Copyright (C) 2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC472_442.h"
#include "hid_transfer.h"
#include "string.h"
#include "FMC_USER.h"
#include "isp.h"

#define PLLCON_SETTING      CLK_PLLCON_84MHz_HXT
#define PLL_CLOCK           84000000

static uint8_t volatile	bufhead;
__align(4) uint8_t usb_rcvbuf[64];
__align(4) uint8_t usb_sendbuf[64];
__align(4) static uint8_t 	uart_rcvbuf[64];
__align(4) static uint8_t 	response_buff[64];
uint8_t bUsbDataReady, bUartDataReady;

__align(4) static uint8_t aprom_buf[PAGE_SIZE];

BOOL bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_timecnt;
static volatile int  g_enter_ISP;


static uint32_t gAppBase;
void SysTimerDelay(uint32_t us);


void timer_init()
{
    /* 
     *  Configure Timer A, clock source from XTL_12M. Prescale 12
     *  Give 1 seconds counter => 1000000
     */
    CLK->CLKSEL1 &= ~CLK_CLKSEL1_TMR0SEL_Msk;	/* TIMER0 clock from HXT */
    CLK->APBCLK0 |= CLK_APBCLK0_TMR0CKEN_Msk;
    TIMER0->CTL = 0;                  	/* disable timer */
    TIMER0->INTSTS = 0x3;                 /* write 1 to clear for safty */
    TIMER0->CTL = (11 << TIMER_CTL_PSC_Pos) |	// pre-scale = 12
                  (0x3 << TIMER_CTL_OPMODE_Pos) | 		// continuous mode
                  TIMER_CTL_CNTEN_Msk;
}

uint32_t get_timer_value()
{
    return TIMER0->CNT;
}

/*  
* send data for UART
*/  
static __inline void PutString()
{   
	int i;
    
	for (i = 0; i < HID_MAX_PACKET_SIZE_EP1; i++)
	{
    	while ((UART0->FIFOSTS & UART_FIFOSTS_TXFULL_Msk));
			UART0->DAT = response_buff[i];
	}
}  

static uint16_t Checksum(unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for(c = 0 , i = 0 ; i < len; i++) 
	{
        c += buf[i];
    }
    
	return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	register uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    
    return lcksum;
    
}

//unit is 1 us
void SysTimerDelay(uint32_t us)
{   
		SysTick->LOAD = us * 84; /* using 84MHz cpu clock*/
		SysTick->VAL   =  (0x00);
		SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

		/* Waiting for down-count to zero */
		while((SysTick->CTRL & (1 << 16)) == 0);
}  

/*  
* UART0/UART1 interrupt handler. handle receive only
*/  
void UART0_IRQHandler(void)
{   
	/*----- Determine interrupt source -----*/
	
	if (UART0->INTSTS & (UART_INTSTS_RXTOIF_Msk | UART_INTSTS_RDAIF_Msk)) //RDA FIFO interrupt & RDA timeout interrupt 
   	{
   		while (((UART0->FIFOSTS & UART_FIFOSTS_RXEMPTY_Msk) == 0) && (bufhead < 64)) //RX fifo not empty
			uart_rcvbuf[bufhead++] = UART0->DAT;
	}
	
	if (bufhead == 64)
	{
		bUartDataReady = TRUE;
		bufhead = 0;
	}
}  

uint16_t cksum;
uint32_t Global_cmd;
static int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{
	uint8_t *response;
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint16_t lcksum;
	uint32_t lcmd, srclen, i, regcnf0, security;
//	uint32_t packno;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	cksum = 0;
	
	//response = usb_sendbuf;
#ifndef UART_ONLY
 		response = usb_sendbuf;
 #else
    response = response_buff;
 #endif
	
	pSrc = buffer;
	srclen = len;
	lcmd = inpw(pSrc);
	Global_cmd = inpw(pSrc);
	
//	packno = inpw(pSrc + 4);
	outpw(response + 4, 0);
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response + 8));//read config
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	
	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	
//	if((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
//		goto out;
	
	if((lcmd) && (lcmd!=CMD_RESEND_PACKET))
		gcmd = lcmd;
	
	if(lcmd == CMD_GET_FWVER)
	{
		response[8] = FW_VERSION;//version 2.3
	}
	else if(lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, SYS->PDID);
		goto out;
	}
	else if(lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		outpw(&SYS->RSTSTS, 3);//clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCTL) & 0xFFFFFFFE);//ISP disable
		}

		outpw(&FMC->ISPCTL, i);
		outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

		/* Trap the CPU */
		while(1);
	}
	else if(lcmd == CMD_CONNECT)
	{
		//g_connStatus = CONNECTED;
        g_enter_ISP = 1;
		g_packno = 1;
		goto out;
	}
	else if(lcmd == CMD_DISCONNECT)
	{
		//g_connStatus = DISCONNECTED;
		return 0;
	}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		if(lcmd == CMD_ERASE_ALL)  //erase APROM + data flash
		{
			EraseAP(FALSE, 0, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
			*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		else
			EraseAP(TRUE, 0, 0);//don't erase data flash
				
		bUpdateApromCmd = TRUE;		
	}
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCTL)&0x2)? 2 : 1);
	}
	
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if(g_dataFlashSize)//g_dataFlashAddr
			{
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			}
			else 
				goto out;
		}
		else
		{
			//StartAddress = 0;
            StartAddress = gAppBase;
		}
			
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;
		//printf("StartAddress=%x,TotalPadLen=%d\n",StartAddress, TotalPadLen);
		//return 0;
	}
	
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	else if(lcmd == CMD_RESEND_PACKET)//for APROM&Data flash only
	{  	   
		StartAddress -= LastDataLen;
		TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%PAGE_SIZE) >= (PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+PAGE_SIZE);
		goto out;

	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
//	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
		{
			TotalLen -= srclen;
		}
		else
		{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
	    }

//		if((Global_cmd == CMD_UPDATE_APROM))
//			cksum = Checksum(buffer , len);
		
		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc); //WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		memset(pSrc, 0, srclen);
		
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		
		if((Global_cmd == CMD_UPDATE_APROM))
			cksum = Checksum(buffer , len);
		
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;


	return 0;
}

//the smallest of APROM size is 2K 
static __inline uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}

uint32_t USB_EpAFun(uint8_t *buff)
{
	
	uint8_t i;
	
	for (i=0; i<64; i++)
      USBD->EP[EPA].EPDAT_BYTE = usb_sendbuf[i];
  USBD->EP[EPA].EPTXCNT = 64;
  USBD_ENABLE_EP_INT(EPA, USBD_EPINTEN_INTKIEN_Msk);
	 
 	return 0;
}


/*--------------------------------------------------------------------------*/
void SYS_Init(void)
{
    /*---------------------------------------------------------------------------------------------------------*/
    /* Init System Clock                                                                                       */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

    /* Enable External XTAL (4~24 MHz) */
    CLK_EnableXtalRC(CLK_PWRCTL_HXTEN_Msk);

    /* Waiting for 12MHz clock ready */
    CLK_WaitClockReady( CLK_STATUS_HXTSTB_Msk);

    /* Switch HCLK clock source to HXT */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_HXT,CLK_CLKDIV0_HCLK(1));

    /* Set PLL to power down mode and PLL_STB bit in CLKSTATUS register will be cleared by hardware.*/
    CLK->PLLCTL |= CLK_PLLCTL_PD_Msk;

    /* Set PLL frequency */
    CLK->PLLCTL = CLK_PLLCTL_84MHz_HXT;

    /* Waiting for clock ready */
    CLK_WaitClockReady(CLK_STATUS_PLLSTB_Msk);

    /* Switch HCLK clock source to PLL */
    CLK_SetHCLK(CLK_CLKSEL0_HCLKSEL_PLL,CLK_CLKDIV0_HCLK(1));

    /* Enable IP clock */
    CLK_EnableModuleClock(UART0_MODULE);
    CLK_EnableModuleClock(USBD_MODULE);

    /* Select IP clock source */
    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UARTSEL_HXT, CLK_CLKDIV0_UART(1));

    /* Enable USB PHY */
    SYS->USBPHY = 0x100;  // USB device

    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate SystemCoreClock. */
    SystemCoreClockUpdate();


    /*---------------------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                                 */
    /*---------------------------------------------------------------------------------------------------------*/
    /* Set GPG multi-function pins for UART0 RXD and TXD */
    SYS->GPG_MFPL = SYS_GPG_MFPL_PG1MFP_UART0_RXD | SYS_GPG_MFPL_PG2MFP_UART0_TXD ;

    /* Lock protected registers */
    //SYS_LockReg();
}

/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function                                                                                          */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
    uint8_t volatile	bufhead_bak = 0;
    /* Init System, IP clock and multi-function I/O
       In the end of SYS_Init() will issue SYS_LockReg()
       to lock protected register. If user want to write
       protected register, please issue SYS_UnlockReg()
       to unlock protected register if necessary */
    SYS_Init();

    /* Init UART to 115200-8n1 for print message */
    UART_Open(UART0, 115200);
	
#ifdef UART_ONLY	
    NVIC_SetPriority(UART0_IRQn, 2);
    UART_ENABLE_INT(UART0 , (UART_INTEN_RDAIEN_Msk | UART_INTEN_RLSIEN_Msk | UART_INTEN_RXTOIEN_Msk | UART_INTEN_TOCNTEN_Msk));
    NVIC_EnableIRQ(UART0_IRQn);
#endif
	
	CLK->AHBCLK |= CLK_AHBCLK_ISPCKEN_Msk;
	FMC->ISPCTL |= FMC_ISPCTL_ISPEN_Msk;

	g_apromSize = GetApromSize();
	GetDataFlashInfo(&g_dataFlashAddr , &g_dataFlashSize); 	

    g_enter_ISP = 0;
	
#ifndef UART_ONLY	
	SYS->GPB_MFPH &= 0x0FFFFFFF;                 /* set PB.15 as GPIO pin  */
//	GPIO_SetMode(PB, 15, GPIO_MODE_INPUT);       /* set PB.15 input mode   */

	if ((PB->PIN & 0x8000) == 0)
	{
        g_enter_ISP = 1;	
        USBD_Open(&gsInfo, HID_ClassRequest, NULL);

	    /* Endpoint configuration */
	    HID_Init();
	
	    /* Enable USBD interrupt */
	    NVIC_EnableIRQ(USBD_IRQn);
	
	    /* Start transaction */
	    USBD_Start();
	}
#endif 
	
    timer_init();
    
    while(1)
	{	    
   		if (bUartDataReady == TRUE)
   		{
   			bUartDataReady = FALSE;
    		g_timecnt = 0;
   			ParseCmd(uart_rcvbuf, HID_MAX_PACKET_SIZE_EP1, FALSE);
   			PutString();
    	}
   		
    //timeout happen; but byte is less than 64 bytes; host goes wrong
		if (bufhead > 0)
		{
			if (g_timecnt == 0)
				bufhead_bak = bufhead;
			
			SysTimerDelay(1); //1 us
			g_timecnt++;
			
			if (g_timecnt > 1000) // 1 ms
			{					
				g_timecnt = 0;
				if (bufhead_bak == bufhead)
				{
					bufhead = 0;
				}
			}
		}
		
		if (!g_enter_ISP && (get_timer_value() > 100000))
		{
			// if there's no any ISP connect received in 100ms, timeout
			// reset and boot from APROM
            outpw(&SYS->RSTSTS, 3);//clear bit
            outpw(&FMC->ISPCTL, inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
            outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

            /* Trap the CPU */
            while(1);
		}

#ifndef UART_ONLY
        if(((PB->PIN & 0x8000) == 0) && (bUsbDataReady == TRUE))
        {
            ParseCmd((uint8_t *)usb_rcvbuf, HID_MAX_PACKET_SIZE_EP1, TRUE);
            USB_EpAFun((uint8_t *)&usb_sendbuf[0]);

            //printf("USB process!\n");
            bUsbDataReady = FALSE;
        }
    
        //change to APROM					
        if((PB->PIN & 0x8000) != 0)//GPIO high			
        {
            outpw(&SYS->RSTSTS, 3);//clear bit
            outpw(&FMC->ISPCTL, inpw(&FMC->ISPCTL) & 0xFFFFFFFC);
            outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));

            /* Trap the CPU */
            while(1);
        }
#endif
    }
		
}



/*** (C) COPYRIGHT 2013 Nuvoton Technology Corp. ***/

