/**
  * COPYRIGHT 2010 Nuvoton Technology Corp.
  ******************************************************************************
  * @file    nuc200_i2c.c
  * @author  NuMicro MCU Software Team
  * @version V1.0.0
  * @date    10/18/2010
  * @brief   This file provides all the I2C firmware functions.
  ******************************************************************************
  */ 
  
//* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <NANO1xx.h>
#include "DrvI2C.h"
#include "Driver/DrvGPIO.h"

/* Definitons ------------------------------------------------------------------*/
#define I2C_TRIG             0x10
#define I2C_STA              0x08
#define I2C_STO              0x04
#define I2C_AA               0x02

static I2C_CALLBACK_T I2CHandler0 = {0};
static I2C_CALLBACK_T I2CHandler1 = {0};

/**
  * @brief  ISR to handle I2C0 interrupt event.
  * @param  None.
  * @retval None.
  */
void I2C0_IRQHandler(void)
{
    uint32_t status;
	
	// clear interrupt flag
	I2C0->INTSTS_BITS.STAINTSTS = 1;
	
  	status  = I2C0->STATUS;

    if (I2C0->INTSTS_BITS.TOUTSTS)
	{
		I2C0->INTSTS_BITS.TOUTSTS = 1;	/* Clear TIF */
       	
		if (I2CHandler0.TimeoutCallBackFn)
		{
			I2CHandler0.TimeoutCallBackFn(status); 
		}
	}
	else
	{
		switch (status)
    	{	
     		case 0x38:	/* Arbitration loss */
     		{
        		if (I2CHandler0.ArbitLossCallBackFn)
					I2CHandler0.ArbitLossCallBackFn(status); 
     	    	break;
     		}
 			case 0x00: 	/* Bus error */
			{
        		if (I2CHandler0.BusErrCallBackFn)
					I2CHandler0.BusErrCallBackFn(status); 
     	    	break;
			}
			default:
			{
        		if (I2CHandler0.I2CCallBackFn)
					I2CHandler0.I2CCallBackFn(status); 			
			}		
		}
		
	}
}

/**
  * @brief  ISR to handle I2C1 interrupt event.
  * @param  None.
  * @retval None.
  */
void I2C1_IRQHandler(void)
{
    uint32_t status;
	
	// clear interrupt flag
	I2C1->INTSTS_BITS.STAINTSTS = 1;
	
  	status  = I2C1->STATUS;

    if (I2C1->INTSTS_BITS.TOUTSTS)
	{
		I2C1->INTSTS_BITS.TOUTSTS = 1;	/* Clear TIF */
       	
		if (I2CHandler1.TimeoutCallBackFn)
		{
			I2CHandler1.TimeoutCallBackFn(status); 
		}
	}
	else
	{
		switch (status)
    	{	
     		case 0x38:	/* Arbitration loss */
     		{
        		if (I2CHandler1.ArbitLossCallBackFn)
					I2CHandler1.ArbitLossCallBackFn(status); 
     	    	break;
     		}
 			case 0x00:	/* Bus error */
			{
        		if (I2CHandler1.BusErrCallBackFn)
					I2CHandler1.BusErrCallBackFn(status); 
     	    	break;
			}
			default:
			{
        		if (I2CHandler1.I2CCallBackFn)
					I2CHandler1.I2CCallBackFn(status); 			
			}		
		}
		
	}  
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:	DrvGPIO_InitI2C0																		   */
/*																										   */
/* Parameter:																							   */
/*				 I2CFun	 -[in] the struct parameter	to I2C pin function									   */
/*						   include of																	   */
/*							   i2c0SDA - E_I2C0_SDA_GPA4 / E_I2C0_SDA_GPA8 / E_I2C0_SDA_GPA12 /			   */
/*										 E_I2C0_SDA_GPC12 /	E_I2C0_SDA_GPF4	/ E_I2C0_SDA_NONE			   */
/*																										   */
/*							   i2c0SCK - E_I2C0_SCK_GPA5 / E_I2C0_SCK_GPA9 / E_I2C0_SCK_GPA13 /			   */
/*										 E_I2C0_SCK_GPC13 /	E_I2C0_SCK_GPF5	/ E_I2C0_SCK_NONE			   */
/* Returns:																								   */
/*				E_SUCCESS										Operation successful					   */
/*				E_DRVGPIO_ARGUMENT								Incorrect argument						   */
/* Description:																							   */
/*				Initialize the specified function and configure	the	relative pins						   */
/*				for	specified function used.															   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t	DrvGPIO_InitI2C0(STR_GPIO_I2C0_T *I2CFun)
{
	switch (I2CFun->i2c0SDA)
	{
		case E_I2C0_SDA_GPA4:
		{
			GCR->PA_L_MFP_BITS.PA4 = 0x5;
		}break;

		case E_I2C0_SDA_GPA8:
		{
			GCR->PA_H_MFP_BITS.PA8 = 0x1;
		}break;

		case E_I2C0_SDA_GPA12:
		{
			GCR->PA_H_MFP_BITS.PA12	= 0x5;
		}break;

		case E_I2C0_SDA_GPC12:
		{
			GCR->PC_H_MFP_BITS.PC12	= 0x6;
		}break;

		case E_I2C0_SDA_GPF4:
		{
			GCR->PF_L_MFP_BITS.PF4 = 0x1;
		}break;

		case E_I2C0_SDA_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (I2CFun->i2c0SCK)
	{
		case E_I2C0_SCK_GPA5:
		{
			GCR->PA_L_MFP_BITS.PA5 = 0x5;
		}break;

		case E_I2C0_SCK_GPA9:
		{
			GCR->PA_H_MFP_BITS.PA9 = 0x1;
		}break;

		case E_I2C0_SCK_GPA13:
		{
			GCR->PA_H_MFP_BITS.PA13	= 0x5;
		}break;

		case E_I2C0_SCK_GPC13:
		{
			GCR->PC_H_MFP_BITS.PC13	= 0x6;
		}break;

		case E_I2C0_SCK_GPF5:
		{
			GCR->PF_L_MFP_BITS.PF5 = 0x1;
		}break;

		case E_I2C0_SCK_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}
	return E_SUCCESS;
}

/*---------------------------------------------------------------------------------------------------------*/
/* Function:	DrvGPIO_InitI2C1																		   */
/*																										   */
/* Parameter:																							   */
/*				 I2CFun	 -[in] the struct parameter	to I2C pin function									   */
/*						   include of																	   */
/*							   i2c1SDA - E_I2C1_SDA_GPA10 /	E_I2C1_SDA_GPC8	/ E_I2C1_SDA_NONE			   */
/*																										   */
/*							   i2c1SCK - E_I2C1_SCK_GPA11 /	E_I2C1_SCK_GPC9	/ E_I2C1_SCK_NONE			   */
/* Returns:																								   */
/*				E_SUCCESS										Operation successful					   */
/*				E_DRVGPIO_ARGUMENT								Incorrect argument						   */
/* Description:																							   */
/*				Initialize the specified function and configure	the	relative pins						   */
/*				for	specified function used.															   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t	DrvGPIO_InitI2C1(STR_GPIO_I2C1_T *I2CFun)
{
	switch (I2CFun->i2c1SDA)
	{
		case E_I2C1_SDA_GPA10:
		{
			GCR->PA_H_MFP_BITS.PA10	= 0x1;
		}break;

		case E_I2C1_SDA_GPC8:
		{
			GCR->PC_H_MFP_BITS.PC8 = 0x5;
		}break;

		case E_I2C1_SDA_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}

	switch (I2CFun->i2c1SCK)
	{
		case E_I2C1_SCK_GPA11:
		{
			GCR->PA_H_MFP_BITS.PA11	= 0x1;
		}break;

		case E_I2C1_SCK_GPC9:
		{
			GCR->PC_H_MFP_BITS.PC9 = 0x5;
		}break;

		case E_I2C1_SCK_NONE:
			break;

		default:
			return E_DRVGPIO_ARGUMENT;
	}
	return E_SUCCESS;
}


/**
  * @brief  Set I2C control bit.
  * @param  port: I2C port number.
  *         start: Set I2C START bit.
  *			stop: Set I2C STOP bit.
  *			ack: set I2C ACK bit.
  * @retval None.
  */
void DrvI2C_Ctrl(E_I2C_PORT port, uint8_t start, uint8_t stop, uint8_t ack)
{	
	uint32_t Reg = 0;
	
	if (start)
		Reg |= I2C_STA;
	if (stop)
	    Reg |= I2C_STO;
	if (ack)
		Reg |= I2C_AA;
	
	if (port)
		*((__IO uint32_t *)&I2C1->CON) = (*((__IO uint32_t *)&I2C1->CON) & ~0x1E) | Reg;
	else
		*((__IO uint32_t *)&I2C0->CON) = (*((__IO uint32_t *)&I2C0->CON) & ~0x1E) | Reg;	    	
}

/**
  * @brief  Active I2C operation.
  * @param  port: I2C port number.
  * @retval None.
  */
void DrvI2C_Trig(E_I2C_PORT port)
{
	if (port)
		I2C1->CON_BITS.TRIG = 1;
	else
		I2C0->CON_BITS.TRIG = 1;
}

/**
  * @brief  Clear interrupt flag.
  * @param  port: I2C port number.
  * @retval None.
  */
void DrvI2C_ClearIntFlag(uint8_t port)
{
	if(port)
		I2C1->INTSTS_BITS.STAINTSTS = 1;
	else
		I2C0->INTSTS_BITS.STAINTSTS = 1;
}

/**
  * @brief  Write Data into I2C Data register.
  * @param  port: I2C port number.
  *			u8data: I2C data.
  * @retval None.
  */
void DrvI2C_WriteData(E_I2C_PORT port, uint8_t u8data)
{
	if (port)
	{						   
		I2C1->DATA = u8data;
	}	
	else
	{
		I2C0->DATA = u8data;	
	}
}

/**
  * @brief  Read Data from I2C Data register.
  * @param  port: I2C port number.  
  * @retval I2C data.
  */
uint8_t DrvI2C_ReadData(E_I2C_PORT port)
{
	if (port)
	{						   
		return I2C1->DATA;
	}	
	else
	{
		return I2C0->DATA;	
	}
}

/**
  * @brief  Get I2C interrupt flag.
  * @param  port: I2C port number.  
  * @retval None.
  */
uint8_t DrvI2C_GetIntFlag(E_I2C_PORT port)
{
	if (port)
	{						   
		return I2C1->INTSTS_BITS.STAINTSTS;
	}	
	else
	{
		return I2C0->INTSTS_BITS.STAINTSTS;	
	}
}

/**
  * @brief  Clear I2C time out flag.
  * @param  port: I2C port number.  
  * @retval None.
  */
void DrvI2C_ClearTimeoutFlag(E_I2C_PORT port)
{
	if (port)
	{						   
		I2C1->INTSTS_BITS.TOUTSTS = 1;
	}	
	else
	{
		I2C0->INTSTS_BITS.TOUTSTS = 1;	
	}	    	
}

/**
  * @brief  Get I2C status.
  * @param  port: I2C port number.  
  * @retval I2C status.
  */
uint32_t DrvI2C_GetStatus(E_I2C_PORT port)
{
	if (port)
	{
		return I2C1->INTSTS;
	}
	else
	{
		return I2C0->INTSTS;
	}	
}
/**
  * @brief  Enable I2C function and set clock divider. I2C clock = PCLK / (4x(divider+1)).
  * @param  port: I2C port number.
  *			u32BusClock: I2C bus clock frequency (Hz)  
  * @retval 0: Success.
  */
int32_t DrvI2C_Open(E_I2C_PORT port, uint32_t u32BusClock)
{
    uint32_t divider;     
 
	divider = (uint32_t) (((SystemCoreClock*10)/(u32BusClock * 4) + 5) / 10 - 1);	/* Compute proper divider for I2C clock */
	
    if (port)
	{
		CLK->APBCLK_BITS.I2C1_EN = 1;
		GCR->IPRST_CTL2_BITS.I2C1_RST = 1;
		GCR->IPRST_CTL2_BITS.I2C1_RST = 0;
		I2C1->CON_BITS.INTEN = 1;
		I2C1->DIV = divider;
		I2C1->CON_BITS.IPEN = 1;
	}
	else
	{
		CLK->APBCLK_BITS.I2C0_EN = 1;
		GCR->IPRST_CTL2_BITS.I2C0_RST = 1;
		GCR->IPRST_CTL2_BITS.I2C0_RST = 0;	
		I2C0->CON_BITS.INTEN = 1;
		I2C0->DIV = divider;
		I2C0->CON_BITS.IPEN = 1;
	}

	return 0;
}

/**
  * @brief  Disable I2C function and clock source.
  * @param  port: I2C port number.  
  * @retval 0: Success.
  */
int32_t DrvI2C_Close(E_I2C_PORT port)
{
    if (port)
	{
		I2C1->CON_BITS.INTEN = 0;
		GCR->IPRST_CTL2_BITS.I2C1_RST = 1;
		GCR->IPRST_CTL2_BITS.I2C1_RST = 0;
		CLK->APBCLK_BITS.I2C1_EN = 0;
	}
	else
	{
		I2C0->CON_BITS.INTEN = 0;
		GCR->IPRST_CTL2_BITS.I2C0_RST = 1;
		GCR->IPRST_CTL2_BITS.I2C0_RST = 0;
		CLK->APBCLK_BITS.I2C0_EN = 0;
	}
	
	return 0;
}

/**
  * @brief  Set clock divider. I2C clock = PCLK / (4x(divider+1)).
  * @param  port: I2C port number. 
  *			u32BusClock: I2C bus clock frequency (Hz)    
  * @retval 0: Success.
  */
int32_t DrvI2C_SetClockFreq(E_I2C_PORT port, uint32_t u32BusClock)
{
    uint32_t divider;     
 
	divider = (uint32_t) (((SystemCoreClock*10)/(u32BusClock * 4) + 5) / 10 - 1);
    if (port)
	{
		I2C1->DIV = divider;
	}
	else
	{
		I2C0->DIV = divider;
	}
	return 0;
}

/**
  * @brief  Get I2C bit rate.
  * @param  port: I2C port number.   
  * @retval I2C Bus Clock Frequency
  */
uint32_t DrvI2C_GetClockFreq(E_I2C_PORT port)
{
    uint32_t divider;     
 
	divider = ( (port)? I2C1->DIV:I2C0->DIV );
	return ( SystemCoreClock / ((divider+1)<<2) );
}

/**
  * @brief  Enable I2C interrupt and NVIC corresponding to I2C.
  * @param  port: I2C port number.  
  * @retval 0: Success.
  */
int32_t DrvI2C_EnableInt(E_I2C_PORT port)
{
    if (port)
	{
		I2C1->CON_BITS.INTEN = 1;
		NVIC_EnableIRQ(I2C1_IRQn);
	}
	else
	{
	    I2C0->CON_BITS.INTEN = 1;
		NVIC_EnableIRQ(I2C0_IRQn);
	}

	return 0;
}

/**
  * @brief  Disable I2C interrupt and NVIC corresponding to I2C.
  * @param  port: I2C port number.  
  * @retval 0: Success.
  */
int32_t DrvI2C_DisableInt(E_I2C_PORT port)
{
    if (port)
	{
		I2C1->CON_BITS.INTEN= 0;
		NVIC_DisableIRQ(I2C1_IRQn);
	}
	else
	{
	    I2C0->CON_BITS.INTEN = 0;
		NVIC_DisableIRQ(I2C0_IRQn);
	}

	return 0;
}

/**
  * @brief  Enable I2C 14-bit timeout counter and set div4 bit of timeout counter.
  * @param  port: I2C port number.
  *			i32enable: Enable timeout counter
  *			u8div4:	Enable timeout counter input clock is didider by 4
  * @retval 0: Success.
  */
int32_t DrvI2C_SetTimeoutCounter(E_I2C_PORT port, int32_t i32enable, uint8_t u8div4)
{
    if (port)
	{
		I2C1->TOUT_BITS.DIV4 = u8div4;
		I2C1->TOUT_BITS.TOUTEN = i32enable;
	}
	else
	{
	    I2C0->TOUT_BITS.DIV4 = u8div4;
		I2C0->TOUT_BITS.TOUTEN = i32enable;
	}
	return 0;
}

/**
  * @brief  Set 2 7-bit slave addresses and enable General Call function.
  * @param  port: I2C port number.
  *			SlaveNo: There are 2 slave addrss. The slaveNo is 0/1.
  *			salve_addr: 7-bit data
  *			GC_Flag: Enable/Disable GC function
  * @retval 0: Success.
  *			<0: Failed.
  */
int32_t DrvI2C_SetAddress(E_I2C_PORT port, uint8_t slaveNo, uint8_t slave_addr, uint8_t GC_Flag)
{
	if (port)
	{
		switch (slaveNo)
		{
			case 0:
				I2C1->SADDR0_BITS.SADDR = slave_addr;
				I2C1->SADDR0_BITS.GCALL = GC_Flag;
				break;
			case 1:
				I2C1->SADDR1_BITS.SADDR = slave_addr;
				I2C1->SADDR1_BITS.GCALL = GC_Flag;
				break;			
			default:
				return -1;
		}
	}
	else
	{
		switch (slaveNo)
		{
			case 0:
				I2C0->SADDR0_BITS.SADDR = slave_addr;
				I2C0->SADDR0_BITS.GCALL = GC_Flag;
				break;
			case 1:
				I2C0->SADDR1_BITS.SADDR = slave_addr;
				I2C0->SADDR1_BITS.GCALL = GC_Flag;
				break;			
			default:
				return -1;
		}	
	}
	return 0;
}

/**
  * @brief  Set 2 7-bit slave address mask. The corresponding address bit is "Don't Care".
  * @param  port: I2C port number.
  *			SlaveNo: There are 4 slave addrss. The slaveNo is 0/1.
  *			salve_addr: 7-bit data  
  * @retval 0: Success.
  *			<0: Failed.
  */
int32_t DrvI2C_SetAddressMask(E_I2C_PORT port, uint8_t slaveNo, uint8_t slaveAddrMask)
{
	if (port)
	{
		switch (slaveNo)
		{
			case 0:
				I2C1->SAMASK0_BITS.SAMASK = slaveAddrMask;
				break;
			case 1:
				I2C1->SAMASK1_BITS.SAMASK = slaveAddrMask;
				break;			
			default:
				return -1;
		}
	}
	else
	{
		switch (slaveNo)
		{
			case 0:
				I2C0->SAMASK0_BITS.SAMASK = slaveAddrMask;
				break;
			case 1:
				I2C0->SAMASK1_BITS.SAMASK = slaveAddrMask;
				break;			
			default:
				return -1;
		}	
	}
	return 0;
}

/**
  * @brief  Install I2C call back function for I2C normal function, Arbitration loss, Bus error and Counter timeout
  * @param  port: I2C port number.
  *			Type: Error type(I2CFUNC / ARBITLOSS / BUSERROR / TIMEOUT)
  *			callbackfn:	Call back function  
  * @retval 0: Success.
  *			<0: Failed.
  */
int32_t DrvI2C_InstallCallback(E_I2C_PORT port, E_I2C_CALLBACK_TYPE Type, I2C_CALLBACK callbackfn)
{
	if (port == I2C_PORT0)
	{	
		switch(Type)
    	{
        	case I2CFUNC:
        	{
            	I2CHandler0.I2CCallBackFn = callbackfn;  
        		break;
        	}
        	case ARBITLOSS:
        	{
            	I2CHandler0.ArbitLossCallBackFn = callbackfn;  
        		break;
        	}
        	case BUSERROR:
        	{
            	I2CHandler0.BusErrCallBackFn = callbackfn;  
        		break;
        	}        
        	case TIMEOUT:
        	{
            	I2CHandler0.TimeoutCallBackFn = callbackfn;  
        		break;
        	}
			default:
				return -1;                             	
		}
	}
	else if (port == I2C_PORT1)
	{	
		switch(Type)
    	{
        	case I2CFUNC:
        	{
            	I2CHandler1.I2CCallBackFn = callbackfn;  
        		break;
        	}
        	case ARBITLOSS:
        	{
            	I2CHandler1.ArbitLossCallBackFn = callbackfn;  
        		break;
        	}
        	case BUSERROR:
        	{
            	I2CHandler1.BusErrCallBackFn = callbackfn;  
        		break;
        	}        
        	case TIMEOUT:
        	{
            	I2CHandler1.TimeoutCallBackFn = callbackfn;  
        		break;
        	}
			default:
				return -1;                             	
		}
	}		
	return 0;
}

/**
  * @brief  Uninstall I2C call back function for I2C normal function, Arbitration loss, Bus error and Counter timeout
  * @param  port: I2C port number.
  *			Type: Error type(I2CFUNC / ARBITLOSS / BUSERROR / TIMEOUT)
  * @retval 0: Success.
  *			<0: Failed.
  */
int32_t DrvI2C_UninstallCallBack(E_I2C_PORT port, E_I2C_CALLBACK_TYPE Type)
{
	if (port == I2C_PORT0)
	{ 
		switch(Type)
    	{
        	case I2CFUNC:
        	{
            	I2CHandler0.I2CCallBackFn = 0;  
        		break;
        	}
        	case ARBITLOSS:
        	{
            	I2CHandler0.ArbitLossCallBackFn = 0;  
        		break;
        	}
        	case BUSERROR:
        	{
            	I2CHandler0.BusErrCallBackFn = 0;  
        		break;
        	}        
        	case TIMEOUT:
        	{
            	I2CHandler0.TimeoutCallBackFn = 0;  
        		break;
        	}
			default:
				return -1;                             	
		}	                  	
	}
	else if (port == I2C_PORT1)
	{ 
		switch(Type)
    	{
        	case I2CFUNC:
        	{
            	I2CHandler1.I2CCallBackFn = 0;  
        		break;
        	}
        	case ARBITLOSS:
        	{
            	I2CHandler1.ArbitLossCallBackFn = 0;  
        		break;
        	}
        	case BUSERROR:
        	{
            	I2CHandler1.BusErrCallBackFn = 0;  
        		break;
        	}        
        	case TIMEOUT:
        	{
            	I2CHandler1.TimeoutCallBackFn = 0;  
        		break;
        	}
			default:
				return -1;                             	
		}	                  	
	}		
	return 0;
}


/*** (C) COPYRIGHT 2010 Nuvoton Technology Corp. ***/
