/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "NANO1xx.h"

#include "Driver/DrvUSB.h"
#include "Driver/DrvGPIO.h"
#include "Driver/DrvI2C.h"
#include "USB/HIDDevice.h"
#include "USB/USBInfra.h"
#include "FMC.h"

#define DBG_PRINTF          printf
//#define DBG_PRINTF(...)          


#define CMD_UPDATE_APROM	    0x000000A0
#define CMD_UPDATE_CONFIG	    0x000000A1
#define CMD_READ_CONFIG		    0x000000A2
#define CMD_ERASE_ALL		    0x000000A3
#define CMD_SYNC_PACKNO		    0x000000A4
#define CMD_GET_FWVER		    0x000000A6
#define CMD_RUN_APROM		    0x000000AB
#define CMD_RUN_LDROM		    0x000000AC
#define CMD_RESET			    0x000000AD
#define CMD_CONNECT			    0x000000AE

#define CMD_GET_DEVICEID	    0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2


#define PACKET_SIZE     64

#define RETRY_CNT	1000000//4


#define WaitSI_Timeout() \
	cnt = 0;\
	while(I2C0->INTSTS_BITS.STAINTSTS == 0)\
	{\
		SysTimerDelay(30);\
		cnt++;\
		if(cnt >= RETRY_CNT)\
			break;\
	}


E_USBINFRA_USB_STATE eUsbState;
static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus = DISCONNECTED;
__align(4) static uint8_t rcvbuf[PACKET_SIZE];
__align(4) static uint8_t sendbuf[PACKET_SIZE];

volatile BOOL bUsbDataReady, bUartDataReady;
volatile BOOL bUsbInReady;
uint32_t g_apromSize=0x20000;
volatile uint32_t g_pdid, g_timecnt;

void Delay(uint32_t delayCnt);
void my_memcpy(void *dest, void *src, int32_t size);
extern void WordsCpy(void *dest, void *src, int32_t size);

// I2C
static uint8_t Device_Addr0;


/*
* USB interrupt handler
*/
void USBD_IRQHandler(void)
{
    //printf("USB relative events\n");
	USB_ParseEvent();
}

/*
* init HID usb
*/
static void UsbHid(void)
{
	//USB_Init();
	HID_Open();	
	
    NVIC_SetPriority (USBD_IRQn, 2);//(1<<__NVIC_PRIO_BITS) - 2);
    NVIC_EnableIRQ(USBD_IRQn);

	/* Enable float-detection interrupt. */
	_DRVUSB_ENABLE_FLDET_INT();

	/* Enable USB-related interrupts. */
	_DRVUSB_ENABLE_WAKEUP();//enable usb wakeup function
	_DRVUSB_ENABLE_MISC_INT(INTEN_WAKEUP | INTEN_FLDET | INTEN_USBEVT | INTEN_BUSEVT);
	
}



//unit is 0.5us
void SysTimerDelay(uint32_t us)
{
#ifndef UART_ONLY
    SysTick->LOAD = us * 24; /* using 48MHz cpu clock*/
#else
	SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
#endif
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}


static __inline void UartInit(void)
{
   	/* Multi-Function Pin: Enable UART0:Tx Rx */
   	//GCR->PA_H_MFP_BITS.PA14	= 0x6;
   	//GCR->PA_H_MFP_BITS.PA15	= 0x6;
   	outpw(&GCR->PA_H_MFP, 0x66000000);

   	/* Configure GCR to reset UART0 */
   	GCR->IPRST_CTL2_BITS.UART0_RST = 1;
   	GCR->IPRST_CTL2_BITS.UART0_RST = 0;

   	/* Enable UART clock */
   	CLK->APBCLK_BITS.UART0_EN = 1;

    /* Select UART clock source */
    CLK->CLKSEL1_BITS.UART_S = 3;//0:12M; 3:HIRC:12M

    /* Data format */
    //g_pUART->TLCTL.DATA_LEN = 3;
	//UART0->FCR.RFR = 1;//reset rx fifo
	//UART0->FCR.RFITL = 5;//46
	//outpw(&UART0->FCR, 0x52);//46
	outpw(&UART0->TLCTL, 0x303);//14
	UART0->TMCTL = 0x40;

    /* Configure the baud rate */
#if 1    
    *((__IO uint32_t *)&UART0->BAUD) = 0x00000067;//115200
    //*((__IO uint32_t *)&g_pUART->BAUD) = 0xB;//57600
     //*((__IO uint32_t *)&g_pUART->BAUD) = 0x3F0009C2;//4800
#else
    //if SYSCLK->CLKSEL1.UART_S = 2//22.1184M
    *((__IO uint32_t *)&UART0->BAUD) = 0x2F00000A;//115200
    //*((__IO uint32_t *)&UART0->BAUD) = 0x16;//57600
#endif
    
    //NVIC_SetPriority (UART0_IRQn, 2);
    //NVIC_EnableIRQ(UART0_IRQn);
    
    //UART0->IER.RDA_IEN = 1;UART0->IER.TOC_IEN = 1;UART0->IER.TOC_EN = 1;//Enable Timeout Counter
	//outpw(&UART0->IER, 0x811);
}

BOOL I2C_MasterSendData()
{
	int i, cnt;
	BOOL result = TRUE;
	
	/* I2C0 as master sends START signal */
	DrvI2C_Ctrl(I2C_PORT0, 1, 0, 0);
	I2C0->CON_BITS.TRIG = 1;
	WaitSI_Timeout();
	//while( I2C0->CON.SI == 0);
	
	//start signal sent; I2C0->STATUS == 0x08
	//and then send SLA+W to slave
	DrvI2C_WriteData(I2C_PORT0, Device_Addr0<<1);
	DrvI2C_Ctrl(I2C_PORT0, 0, 0, 0);//send SLA+W and receive AA
	I2C0->CON_BITS.TRIG = 1;
	WaitSI_Timeout();
	
	//SLA+W sent, ACK received
	if((I2C0->INTSTS_BITS.STAINTSTS == 0) || (I2C0->STATUS != 0x18))
	{
		result = FALSE;
		goto out;
	}

	i = 0;
	do
	{
		I2C0->DATA = sendbuf[i];
		DrvI2C_Ctrl(I2C_PORT0, 0, 0, 0);//send data and receive AA
		I2C0->CON_BITS.TRIG = 1;
		WaitSI_Timeout();
		
		//SI don't set or NACK received
		if((I2C0->INTSTS_BITS.STAINTSTS == 0) || (I2C0->STATUS != 0x28)) break;
		//printf("i %d\n", i);
		i++;
		
	}while(i<PACKET_SIZE);

out:
	DrvI2C_Ctrl(I2C_PORT0, 0, 1, 0);//send STOP
	I2C0->CON_BITS.TRIG = 1;
	cnt = 0;	
	//while( I2C0->CON.STO == 1);

	while(I2C0->CON_BITS.STOP == 1)
	{
		SysTimerDelay(30);
		cnt++;
		if(cnt >= RETRY_CNT)
			break;
	}

	
	return result;
}

BOOL I2C_MasterRcvData()
{
	int i, cnt;
	BOOL result = TRUE;

	/* I2C0 as master sends START signal */
	DrvI2C_Ctrl(I2C_PORT0, 1, 0, 0);
	I2C0->CON_BITS.TRIG = 1;
	WaitSI_Timeout();
	//while( I2C0->CON.SI == 0);
	
	//(I2C0->STATUS == 0x08) || (I2C0->STATUS == 0x10)
	DrvI2C_WriteData(I2C_PORT0, Device_Addr0<<1 | 0x01);//SLA + R
	DrvI2C_Ctrl(I2C_PORT0, 0, 0, 0);//send SLA+R and receive AA
	I2C0->CON_BITS.TRIG = 1;
	WaitSI_Timeout();
		
	if((I2C0->INTSTS_BITS.STAINTSTS == 0) || (I2C0->STATUS != 0x40)) 
	{
		result = FALSE;
		goto out;
	}
	
	i = 0;
	//for(i = 0; i<PACKET_SIZE; i++)
	do
	{
		DrvI2C_Ctrl(I2C_PORT0, 0, 0, 1);//rcv data and send AA
		I2C0->CON_BITS.TRIG = 1;
		WaitSI_Timeout();
		
		//don't care NACK/ACK, the data is OK for me
		rcvbuf[i] = I2C0->DATA;
		i++;
	}while(i<PACKET_SIZE);
out:	
	DrvI2C_Ctrl(I2C_PORT0, 0, 1, 0);//send STOP
	I2C0->CON_BITS.TRIG = 1;
	//while( I2C0->CON.STO == 1);
	cnt = 0;
	while(I2C0->CON_BITS.STOP == 1)
	{
		SysTimerDelay(30);
		cnt++;
		if(cnt >= RETRY_CNT)
			break;
	}
	
	return result;
}


/*
* init I2C slave
*/
static __inline void I2CInit(void)
{
	uint32_t u32data;
	STR_GPIO_I2C0_T I2C0MuntiFunPin;
	
	/* Set multi function pin for I2C0 */
    I2C0MuntiFunPin.i2c0SDA = E_I2C0_SDA_GPA12;
    I2C0MuntiFunPin.i2c0SCK = E_I2C0_SCK_GPA13;
    DrvGPIO_InitI2C0(&I2C0MuntiFunPin);


	/* Open I2C0 and I2C1, and set clock = 100Kbps */
	DrvI2C_Open(I2C_PORT0, 100000);
	

	/* Get I2C0 clock */
	u32data = DrvI2C_GetClockFreq(I2C_PORT0);
	printf("I2C0 clock %d Hz\n", u32data);

	/* Enable I2C0 interrupt and set corresponding NVIC bit */
	//DrvI2C_EnableInt(I2C_PORT0);
		
	/* Install I2C0 call back function for write data to slave */
	//DrvI2C_InstallCallback(I2C_PORT0, I2CFUNC, I2C0_Callback_Rx);
	
	Device_Addr0 = 0x36;

#if 0
	/* Uninstall I2C0 call back function for write data to slave */
	DrvI2C_UninstallCallBack(I2C_PORT0, I2CFUNC);
	
	/* Disable I2C0 interrupt and clear corresponding NVIC bit */
	DrvI2C_DisableInt(I2C_PORT0);

	/* Close I2C0 and I2C1 */
	DrvI2C_Close(I2C_PORT0);
#endif
	
}


static BOOL bIsUpdate;
static uint32_t gu32TotalLength;

int ByPassCmd(unsigned char *buffer, uint8_t len)
{
    int32_t status;
    uint32_t cmd;

    /* Host didn't get previous status back, return error */
    if(bUsbInReady == TRUE)
        return FALSE;

    status = TRUE;

    memcpy(sendbuf, buffer, PACKET_SIZE);
    status = I2C_MasterSendData();
    if(!status) return status;
    
    cmd = inp32(buffer);
    
    DBG_PRINTF("cmd:%x %x\n", cmd, inp32(buffer+4));

    if((cmd == 0) && bIsUpdate && gu32TotalLength && 0) 
    {
        /* data phase of aprom update */
        DBG_PRINTF("pkt:%d  len:%d\n", inp32(&sendbuf[4]), gu32TotalLength);
        gu32TotalLength -= 56;

        if(gu32TotalLength <= 0)
        {
            bIsUpdate = 0;
            SysTimerDelay(100000);//50ms

            /* Get update status */
            status = I2C_MasterRcvData();
            if(!status) return status;
            memcpy(g_HID_sDevice.au8IntInBuffer, rcvbuf, PACKET_SIZE);
        	my_memcpy((void*) g_HID_ar8UsbBuf1, rcvbuf, HID_INT_BUFFER_SIZE);
        	
        	_DRVUSB_TRIG_EP(2,HID_MAX_PACKET_SIZE_EP1);
        
            bUsbInReady = TRUE;
            
            return TRUE;
        }    
    }
    else
    {

    
        /* Delay according to different commands */
        if((cmd == CMD_SYNC_PACKNO) || 
           (cmd == CMD_GET_FWVER) ||
           (cmd == CMD_GET_DEVICEID) ||
           (cmd == CMD_READ_CONFIG) ||
           (cmd == CMD_UPDATE_CONFIG) ||
           (cmd == CMD_GET_FLASHMODE) )
        {
            //while(bUsbInReady == TRUE);
            SysTimerDelay(100000);        
    
        }
        else if((cmd == CMD_WRITE_CHECKSUM))
        {
            SysTimerDelay(400000);//0.2s
        }
        else if((cmd == CMD_RUN_APROM) || 
                (cmd == CMD_RUN_LDROM) || 
                (cmd == CMD_RESET) || 
              0)
        {
            /* No ack */
            return TRUE;
        }
        else if(cmd == CMD_UPDATE_APROM)
        {
            uint32_t j;

            bIsUpdate = 1;
            gu32TotalLength =  inp32(&sendbuf[0xc]);
            DBG_PRINTF("Write AP, size:%d\n", gu32TotalLength);
            gu32TotalLength -= 48;
    
            for (j = 0; j < 10; j++)
            {
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
            }

        }
        else if (cmd == CMD_ERASE_ALL)
        {
            uint32_t i;

            for (i = 0; i < 6; i++)
            {
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
                SysTimerDelay(100000);
            }
        }
        else
        {
                SysTimerDelay(100000);//50ms

        }
    
        status = I2C_MasterRcvData();
        if(!status) while(1);
        //if(!status) return status;
        memcpy(g_HID_sDevice.au8IntInBuffer, rcvbuf, PACKET_SIZE);
    	my_memcpy((void*) g_HID_ar8UsbBuf1, rcvbuf, HID_INT_BUFFER_SIZE);
    	
    	_DRVUSB_TRIG_EP(2,HID_MAX_PACKET_SIZE_EP1);
    
        bUsbInReady = TRUE;
    
        DBG_PRINTF("Rcv: %x %x %x %x\n", inp32(rcvbuf), inp32(rcvbuf+4),inp32(rcvbuf+8), inp32(rcvbuf+0xc));
    
    }

    return status;
}

int32_t main()
{
    extern uint32_t SystemCoreClock;
    volatile uint32_t pinst;
    uint8_t volatile	bufhead_bak;
	
    UNLOCKREG();

	CLK->PWRCTL_BITS.HIRC_EN = 1;
    /* Waiting for 12M Xtal stable */
	while (CLK->CLKSTATUS_BITS.HIRC_STB != 1);
    /* ISPEN must be set before FMC_Read */
    FMC->ISPCON_BITS.ISPEN = 1;

    /* Waiting for 12M/22M Xtal stalble */
    SysTimerDelay(5000);
    FMC->ISPCON_BITS.ISPEN = 1;


    CLK->PLLCTL = 0x20220;
    //CLK->PLLCTL_BITS.PLL_SRC=0;//HXT
    //CLK->PLLCTL_BITS.PD = 0;
    /* Waiting for PLL stalble */
	while (CLK->CLKSTATUS_BITS.PLL_STB != 1);
	outpw(&CLK->CLKDIV0, 0x12);//HCLK = PLL/3; USB = PLL/2
	CLK->CLKSEL0 = 2;

	/* The PLL must be 48MHz x N times when using USB */

    bUsbDataReady = 0;
    bUartDataReady = 0;
    bUsbInReady = 0;
    bIsUpdate = 0;
    gu32TotalLength = 0;

    UartInit();

	I2CInit();
   
	/* Process USB event by interrupt */    
	UsbHid();

    DBG_PRINTF(" >> ISP to I2C converter <<\n");

    while(1)
    {
    
	   	if (bUsbDataReady == TRUE)
    	{
    		ByPassCmd(g_HID_sDevice.au8IntOutBuffer, HID_MAX_PACKET_SIZE_EP1);
            
            bUsbDataReady = FALSE;
			_DRVUSB_TRIG_EP(3,HID_MAX_PACKET_SIZE_EP1);
    	}
	}

}

void my_memcpy(void *dest, void *src, int32_t size)
{
	WordsCpy(dest, src, size);
}










