/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2009 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <stdio.h>
#include "DrvUSB.h"
#include "UDC.h"
#include "FMC.h"

#define CONFIG_BASE      0x00300000
#define DATA_FLASH_BASE  0x00008000

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL



void Delay(uint32_t delayCnt)
{
	while(delayCnt--)
	{
		__NOP();
		__NOP();
	}
}

/*----------------------------------------------------------------------------
  MAIN function
 *----------------------------------------------------------------------------*/
int32_t main(void)
{

	volatile uint32_t u32INTSTS;

  UNLOCKREG();

  /* Enable 12M OSC */
	CLK->PWRCTL_BITS.HIRC_EN = 1;
	CLK->PWRCTL_BITS.HXT_EN = 1;
  /* Waiting for 12M/HIRC:12M Xtal stalble */
  while (CLK->CLKSTATUS_BITS.HIRC_STB != 1);
	
	FMC->ISPCON_BITS.ISPEN = 1;


	/* Enable PLL */
	CLK->PLLCTL = 0x220;
	CLK->PLLCTL_BITS.PLL_SRC = 0; //E_SYS_EXTERNAL_HXT
	
	CLK->PLLCTL_BITS.PD = 0;
	while(1)
	if (CLK->CLKSTATUS_BITS.PLL_STB == 1)
			break;
	
  CLK->CLKDIV0 = 0x00000010;
 
  CLK->CLKDIV0 = 0x00000012;

	CLK->CLKSEL0  = 2;


	if ((inp32(GPIOC_BASE + 0x10) & BIT4)!=0)	
	{
		/* Boot from AP */		
		outpw(&GCR->RST_SRC, 3);//clear bit
	  outpw(&FMC->ISPCON, inpw(&FMC->ISPCON) & 0xFFFFFFFC);
	  outpw(&SCB->AIRCR, (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ));
		while(1);
	}
	
	
	/* Enable ISP function */
	CLK->AHBCLK_BITS.ISP_EN = 1;
	FMC->ISPCON_BITS.ISPEN = 1;


	/* Initialize USB Device function */
	CLK->APBCLK_BITS.USBD_EN = 1;

    /* Reset IP */	
	GCR->IPRST_CTL2_BITS.USBD_RST = 1;
  RoughDelay(500);
	GCR->IPRST_CTL2_BITS.USBD_RST = 0;

	USBD->CTRL = (CTRL_WAKEUPEN|CTRL_DPPUEN|CTRL_DRVSE0|CTRL_PWRDB|CTRL_PHYEN|CTRL_USBEN);
	USBD->PDMA_BITS.BYTEM = 1;
	USBD->CTRL_BITS.DRVSE0 = 1;
	
    
	RoughDelay(0x100);
	USBD->CTRL_BITS.DRVSE0 = 0;
	
	g_u8UsbState = USB_STATE_DETACHED;
	USBD->MXPLD1 = 0x08;
	UsbFdt();

	/* Initialize mass storage device */
	udcFlashInit();

	/* Start USB Mass Storage */
  /* Handler the USB ISR by polling */
	while(1)
	{
	    u32INTSTS = _DRVUSB_GET_EVENT_FLAG();
	
      if (u32INTSTS & 0x00000004)
	    {
        /* Handle the USB attached/detached event */
		    UsbFdt();
	    }
	    else if(u32INTSTS & 0x00000001)
	    {
        /* Handle the USB bus event: Reset, Suspend, and Resume */
		    UsbBus();
	    }
	    else if(u32INTSTS & 0x00000002)
	    {
        /* Handle the USB Protocol/Clase event */
   	    UsbUsb(u32INTSTS);
      }
	}
}

#ifdef USE_ASSERT
/*---------------------------------------------------------------------------------------------------------*/
/* Function:     assert_error                                                                              */
/*                                                                                                         */
/* Parameter:                                                                                              */   
/*               file           -[in] source file name                                                     */
/*               line           -[in] line number                                                          */
/* Returns:                                                                                                */
/*               None.                                                                                     */
/* Description:                                                                                            */
/*               The function prints the source file name and line number where the assert_param() error   */
/*               occurs, and then stops in an infinite loop.                                               */
/*               User can add his own codes here if necessary.                                                  */
/*---------------------------------------------------------------------------------------------------------*/

void assert_error(uint8_t * file, uint32_t line)
{
    GCR->PA_H_MFP_BITS.PA14 = 0x6;   /* UART0 multi-function pin*/
		GCR->PA_H_MFP_BITS.PA15 = 0x6;
    CLK->APBCLK_BITS.UART0_EN = 1;  /* Enable UART0 clock */
    CLK->CLKDIV0_BITS.UART_N = 0;   /* UART clock divider is 0 */
    CLK->CLKSEL1_BITS.UART_S = 0;   /* Select HXT (12MHz) as UART clock */
 
    /* Set UART to 115200,n,8,1,none */
    UART0->BAUD = 0x67;             /* Baud Rate:115200 for 12MHz */
    UART0->TLCTL = 0x03;            /* Word len is 8 bits */          
	
	printf("[%s] line %d : wrong parameters.\r\n", file, line);

	/* Infinite loop */
	while(1) ;

}
#endif



