/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
#include <string.h>
#include "Nano1X2Series.h"
#include "fmc.h"

//#define SUPPORT_WRITECKSUM
#define USING_AUTODETECT //using autodetect for UART download
//#define USING_RS485 //default using GPD13 act as Rx/Tx switch for RS485 tranceiver

#define PACKET_SIZE				64

#define CMD_UPDATE_APROM		0x000000A0
#define CMD_UPDATE_CONFIG		0x000000A1
#define CMD_READ_CONFIG			0x000000A2
#define CMD_ERASE_ALL			0x000000A3
#define CMD_SYNC_PACKNO			0x000000A4
#define CMD_GET_FWVER			0x000000A6
#define CMD_RUN_APROM			0x000000AB
#define CMD_RUN_LDROM			0x000000AC
#define CMD_RESET				0x000000AD
#define CMD_CONNECT				0x000000AE
#define CMD_DISCONNECT			0x000000AF

#define CMD_GET_DEVICEID		0x000000B1

#define CMD_UPDATE_DATAFLASH 	0x000000C3
#define CMD_WRITE_CHECKSUM 	 	0x000000C9
#define CMD_GET_FLASHMODE 	 	0x000000CA

#define CMD_RESEND_PACKET       0x000000FF

#define	V6M_AIRCR_VECTKEY_DATA	0x05FA0000UL
#define V6M_AIRCR_SYSRESETREQ	0x00000004UL

#define DISCONNECTED	0
#define CONNECTING		1
#define CONNECTED		2

static uint8_t volatile	bufhead;
static uint8_t volatile	g_connStatus;
__align(4) static uint8_t uart_rcvbuf[64];
__align(4) static uint8_t uart_sendbuf[64];
__align(4) static uint8_t aprom_buf[PAGE_SIZE];
BOOL bUsbDataReady, bUartDataReady;
BOOL bUsbInReady, bUpdateApromCmd;
uint32_t g_apromSize, g_dataFlashAddr, g_dataFlashSize;
volatile uint32_t g_pdid, g_timecnt;
#ifdef SUPPORT_WRITECKSUM
static uint32_t g_ckbase = (0x20000 - 8);
static void CheckCksumBase(void);
#endif

static UART_T  *g_pUART = UART0;
static IRQn_Type	g_UARTIRQ = UART0_IRQn;

void Delay(uint32_t delayCnt);
void my_memcpy(void *dest, void *src, int32_t size);
extern void WordsCpy(void *dest, void *src, int32_t size);
void SysTimerDelay(uint32_t us);


/*
* send data for UART
*/
static __inline void PutString()
{
	int i;

	for (i = 0; i<PACKET_SIZE; i++)
	{
		while (g_pUART->FSR & UART_FSR_TX_FULL_F_Msk);
		g_pUART->THR = uart_sendbuf[i];
	}
}

/*
* UART0/UART1 interrupt handler. handle receive only
*/
void UART0_IRQHandler(void)//UART1_IRQHandler
{  
	/* RDA FIFO interrupt & RDA timeout interrupt  */
	if (g_pUART->ISR & (UART_ISR_RTO_IS_Msk | UART_ISR_RDA_IS_Msk))
   	{
   		while (((g_pUART->FSR & UART_FSR_RX_EMPTY_F_Msk) == 0) && (bufhead < 64))
			uart_rcvbuf[bufhead++] = g_pUART->RBR;
	}
	
	if (bufhead == 64)
	{
		bUartDataReady = TRUE;
		bufhead = 0;
	}
}

static __inline void UartInit(void)
{
    SYS->PB_H_MFP = ((SYS->PB_H_MFP & ~0x0FF00000) | 0x07700000);  /* PB.13(RX) PB.14(TX)   */
	CLK->APBCLK |= CLK_APBCLK_UART0_EN_Msk;          /* enable UART0 clock                  */
	CLK->CLKSEL1 = (CLK->CLKSEL1 & ~CLK_CLKSEL1_UART_S_Msk) | 0x3; /* UART clock source from HIRC 12 MHz  */

	g_pUART->BAUD = 0x66;                            /* Baud Rate:115200  OSC:12 MHz        */
	g_pUART->TLCTL = 0x3;                            /* character len is 8 bits             */
	
    NVIC_SetPriority (g_UARTIRQ, 2);
   	NVIC_EnableIRQ(g_UARTIRQ);
	g_pUART->IER = UART_IER_RTO_IE_Msk | UART_IER_RDA_IE_Msk;
}


static uint16_t Checksum (unsigned char *buf, int len)
{
    int i;
    uint16_t c;

    for (c=0, i=0; i < len; i++) {
        c += buf[i];
    }
    return (c);
}

static uint16_t CalCheckSum(uint32_t start, uint32_t len)
{
	int i;
	uint16_t lcksum = 0;
	
	for(i = 0; i < len; i+=PAGE_SIZE)
	{
		ReadData(start + i, start + i + PAGE_SIZE, (uint32_t*)aprom_buf);
		if(len - i >= PAGE_SIZE)
			lcksum += Checksum(aprom_buf, PAGE_SIZE);
		else
			lcksum += Checksum(aprom_buf, len - i);
	}
    return lcksum;
}

static int ParseCmd(unsigned char *buffer, uint8_t len, BOOL bUSB)
{
	static uint32_t StartAddress, StartAddress_bak, TotalLen, TotalLen_bak, LastDataLen, g_packno = 1;
	uint8_t *response;
	uint16_t cksum, lcksum;
	uint32_t	lcmd, packno, srclen, i, regcnf0, security;
	unsigned char *pSrc;
	static uint32_t	gcmd;
	
	response = uart_sendbuf;

	pSrc = buffer;
	srclen = len;

	lcmd = inpw(pSrc);
	packno = inpw(pSrc + 4);
	outpw(response+4, 0);
	
	pSrc += 8;
	srclen -= 8;
	
	ReadData(Config0, Config0 + 8, (uint32_t*)(response+8));//read config
	regcnf0 = *(uint32_t*)(response + 8);
	security = regcnf0 & 0x2;

	if(lcmd == CMD_SYNC_PACKNO)
	{
		g_packno = inpw(pSrc);
	}
	if ((packno != g_packno) && ((lcmd != CMD_CONNECT))) //Connection state, packno always == 1
		goto out;
	

	if ((lcmd)&&(lcmd!=CMD_RESEND_PACKET))
		gcmd = lcmd;

	if (lcmd == CMD_GET_FWVER)
		response[8] = FW_VERSION;    //version 2.4

	else if (lcmd == CMD_GET_DEVICEID)
	{
		outpw(response+8, SYS->PDID);
		goto out;
	}
	else if (lcmd == CMD_RUN_APROM || lcmd == CMD_RUN_LDROM || lcmd == CMD_RESET)
	{
		SYS->RST_SRC = 3;         //clear bit
		/* Set BS */
		if(lcmd == CMD_RUN_APROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
		}
		else if(lcmd == CMD_RUN_LDROM)
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFC);
			i |= 0x00000002;
		}
		else
		{
			i = (inpw(&FMC->ISPCON) & 0xFFFFFFFE);//ISP disable
		}

		FMC->ISPCON = i;
		SCB->AIRCR = (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ);

		/* Trap the CPU */
		while(1);
	}
	else if(lcmd == CMD_CONNECT)
	{
		//g_connStatus = CONNECTED;
		g_packno = 1;
		goto out;
	}
	//else if(lcmd == CMD_DISCONNECT)
	//{
		//g_connStatus = DISCONNECTED;
	//}
		
	else if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_ERASE_ALL))
	{
		
		if (lcmd == CMD_ERASE_ALL)           //erase APROM + data flash
		{
			EraseAP(FALSE, 0, (g_apromSize < 0x20000)?0x20000:g_apromSize);//erase all aprom including data flash
		}
		else
			EraseAP(TRUE, 0, 0);             //don't erase data flash
			
		if(lcmd == CMD_ERASE_ALL)
		{
    		*(uint32_t*)(response + 8) = regcnf0|0x02;
    		UpdateConfig((uint32_t*)(response + 8), NULL);
		}
		
		bUpdateApromCmd = TRUE;
	}


#ifdef SUPPORT_WRITECKSUM
	else if(lcmd == CMD_WRITE_CHECKSUM)//write cksum to aprom last
	{
		cktotallen = inpw(pSrc);
		lcksum = inpw(pSrc+4);
		CheckCksumBase();
		ReadData(g_ckbase & 0xFFE00, g_ckbase, (uint32_t*)aprom_buf);
		outpw(aprom_buf+PAGE_SIZE - 8, cktotallen);
		outpw(aprom_buf+PAGE_SIZE - 4, lcksum);
		FMC_Erase(g_ckbase & 0xFFE00);
		WriteData(g_ckbase & 0xFFE00, g_ckbase + 8, (uint32_t*)aprom_buf);
	}
#endif
	else if(lcmd == CMD_GET_FLASHMODE)
	{
		//return 1: APROM, 2: LDROM
		outpw(response+8, (inpw(&FMC->ISPCON)&0x2)? 2 : 1);
	}
	
	
	if((lcmd == CMD_UPDATE_APROM) || (lcmd == CMD_UPDATE_DATAFLASH))
	{
		if(lcmd == CMD_UPDATE_DATAFLASH)
		{
			StartAddress = g_dataFlashAddr;
		
			if (g_dataFlashSize)//g_dataFlashAddr
				EraseAP(FALSE, g_dataFlashAddr, g_dataFlashAddr + g_dataFlashSize);
			else goto out;
		}
		else
		{
			StartAddress = 0;
		}
		
		//StartAddress = inpw(pSrc);
		TotalLen = inpw(pSrc+4);
		pSrc += 8;
		srclen -= 8;
		StartAddress_bak = StartAddress;
		TotalLen_bak = TotalLen;
		//printf("StartAddress=%x,TotalPadLen=%d\n",StartAddress, TotalPadLen);
		//return 0;
	}
	else if(lcmd == CMD_UPDATE_CONFIG)
	{
		if((security == 0) && (!bUpdateApromCmd))//security lock
			goto out;
		
		UpdateConfig((uint32_t*)(pSrc), (uint32_t*)(response+8));
		GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);

		goto out;
	}
	else if(lcmd == CMD_RESEND_PACKET)//for APROM&Data flash only
	{  	   
		StartAddress -= LastDataLen;
		TotalLen += LastDataLen;
		if((StartAddress & 0xFFE00) >= Config0)
			goto out;
		ReadData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		FMC_Erase(StartAddress & 0xFFE00);
		WriteData(StartAddress & 0xFFE00, StartAddress, (uint32_t*)aprom_buf);
		if((StartAddress%PAGE_SIZE) >= (PAGE_SIZE-LastDataLen))
	    	FMC_Erase((StartAddress & 0xFFE00)+PAGE_SIZE);
		goto out;

	}
	
	if((gcmd == CMD_UPDATE_APROM) || (gcmd == CMD_UPDATE_DATAFLASH))
	{
		if(TotalLen >= srclen)
			TotalLen -= srclen;
		else{
			srclen = TotalLen;//prevent last package from over writing
			TotalLen = 0;
	    }

		WriteData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		//memset(pSrc, 0, srclen);
		{
			uint32_t *p = (uint32_t*)pSrc;
			for(i = 0; i<srclen/4; i++)
				p[i] = 0;
		}
		ReadData(StartAddress, StartAddress + srclen, (uint32_t*)pSrc);
		StartAddress += srclen;
		LastDataLen =  srclen;
		if(TotalLen == 0)
		{
			lcksum = CalCheckSum(StartAddress_bak, TotalLen_bak);
			outps(response + 8, lcksum);
		}
	}
out:
	cksum = Checksum(buffer, len);
	//NVIC_DisableIRQ(USBD_IRQn);
	outps(response, cksum);
	++g_packno;
	outpw(response+4, g_packno);
	g_packno++;
	//NVIC_EnableIRQ(USBD_IRQn);
	//TotalLen -= srclen;

	return 0;
}

//unit is 0.5us
void SysTimerDelay(uint32_t us)
{
	//SysTick->LOAD = us * 11; /* using 22MHz cpu clock*/
	SysTick->LOAD = us * 16; /* using 32MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock

    /* Waiting for down-count to zero */
    while((SysTick->CTRL & (1 << 16)) == 0);
}

#ifdef SUPPORT_WRITECKSUM
static void CheckCksumBase()
{
	//skip data flash
	unsigned int aprom_end = g_apromSize, data;
	int result;
	
	result = FMC_Read(0x1F000 - 8, &data);
	if(result == 0)//128K flash
    {
    	FMC_Read(Config0, &data);
    	if((data&0x01)==0)//DFEN enable
    	{
    		FMC_Read(Config1, &aprom_end);
    	}
    	g_ckbase = aprom_end - 8;
    	return;
    }
    else// less than 128K
    {
    	aprom_end = 0x10000;//64K
    	do{
		    result = FMC_Read(aprom_end - 8, &data);
		    if(result == 0)
		    	break;
    		aprom_end = aprom_end/2;
    	}while(aprom_end > 4096);

		g_ckbase = aprom_end - 8;
	}
}
#endif

//the smallest of APROM size is 2K 
static __inline uint32_t GetApromSize()
{
	uint32_t size = 0x800, data;
	int result;
	
	do{
		result = FMC_Read(size, &data);
		if(result < 0)
	    {
    		return size;
		}
		else 
			size *= 2;
	}while(1);
}



int32_t main()
{
    extern uint32_t SystemCoreClock;
    volatile uint32_t pinst;
    uint8_t volatile	bufhead_bak=0;
#ifdef SUPPORT_WRITECKSUM
    uint32_t totallen, cksum;
#endif
	
    UNLOCKREG();

	CLK->PWRCTL |= CLK_PWRCTL_HIRC_EN_Msk | CLK_PWRCTL_HXT_EN_Msk;
    
    /* Waiting for 12M/HIRC:12M Xtal stalble */
    while (!(CLK->CLKSTATUS & CLK_CLKSTATUS_HIRC_STB_Msk));
    
    FMC->ISPCON |= FMC_ISPCON_ISPEN_Msk;
	
	g_apromSize = GetApromSize();
	GetDataFlashInfo(&g_dataFlashAddr, &g_dataFlashSize);
	
	g_pdid = SYS->PDID;
    
	g_pUART = UART0;
	g_UARTIRQ = UART0_IRQn;
	pinst = (PB->PIN & (1 << 0));    // PB.0 low

    UartInit();
    
    memcpy(uart_sendbuf, "Hello!", 6);
    PutString();

#if defined(USING_AUTODETECT)
    //timeout 30ms
    SysTick->LOAD = 30000 * 48; /* using 48MHz cpu clock*/
    SysTick->VAL   =  (0x00);
    SysTick->CTRL = SysTick->CTRL|(1 << SysTick_CTRL_CLKSOURCE_Pos) | (1<<SysTick_CTRL_ENABLE_Pos);//using cpu clock


    while(1)
    {
		uint32_t	lcmd;

		if((bufhead >= 4) || (bUartDataReady == TRUE))
		{
			lcmd = inpw(uart_rcvbuf);
			if (lcmd == CMD_CONNECT) 
			{
				//pinst = 0;
				goto _ISP;
				//break;
			}
			else
			{
				bUartDataReady = FALSE;
				bufhead = 0;
			}
		}

		if((SysTick->CTRL & (1 << 16)) != 0)//timeout, then goto APROM
		{
			goto _APROM;
    	}
    }
#endif


#ifdef SUPPORT_WRITECKSUM	
	CheckCksumBase();
    
    FMC_Read(g_ckbase, &totallen);
	FMC_Read(g_ckbase+4, &cksum);

    if ((pinst == 0) || ((SYS->RST_SRC & 0x3) == 0x00))
    	|| (totallen > g_apromSize) || (CalCheckSum(0x0, totallen) != cksum))//if GPIO low or SYSRESETREQ reset or checksum error, run ISP
#else
// 	  if ((pinst == 0) || ((SYS->RST_SRC & 0x3) == 0x00)) //if GPIO low or SYSRESETREQ reset, run ISP
#endif
//     {
//     	printf("ISP mode...\n");
//     }
//     else//change to APROM
    {
_APROM:
		//if((inpw(&SYS->RSTSRC)&0x3) == 0x1)
		{ 	//after power on
			SYS->RST_SRC = 3;           //clear bit
	   		FMC->ISPCON &= 0xFFFFFFFC;
	   		SCB->AIRCR = (V6M_AIRCR_VECTKEY_DATA | V6M_AIRCR_SYSRESETREQ);//SYSRESETREQ
		}

		/* Trap the CPU */
		while(1);
    }
    
_ISP:	
    while (1)
    {
    	if (bUartDataReady == TRUE)
	   	{
    		bUartDataReady = FALSE;
	    	g_timecnt = 0;
    		ParseCmd(uart_rcvbuf, PACKET_SIZE, FALSE);
	   		PutString();
	    }
    		
    	//timeout happen; but byte is less than 64 bytes; host goes wrong
		if (bufhead > 0)
		{
			if (g_timecnt == 0)
				bufhead_bak = bufhead;
			SysTimerDelay(1);//0.5us
			g_timecnt++;
			if (g_timecnt > 2000)//1ms
			{					
				g_timecnt = 0;
				if (bufhead_bak == bufhead)
				{
					bufhead = 0;
				}
			}
		}
	}
}


